package it.polito.elite.ontology;


/**
 * NOT COMPLETE AND NOT USABLE AT THE PRESENT TIME April 07
 * only few methods are actually implemented
 */
import it.polito.elite.visualization.ontoviewer.enums.NodeTypesEnum;
import it.polito.elite.visualization.ontoviewer.models.ConceptData;
import it.polito.elite.visualization.ontoviewer.models.InstanceData;
import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.models.RelationType;

import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.logging.Level;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.SimpleInstance;
import edu.stanford.smi.protege.model.Slot;

public class FrameLogicOntologyWrapper extends OntologyAbstraction {
	
	private KnowledgeBase kb;
	
	public FrameLogicOntologyWrapper(KnowledgeBase kb) {
		super();
		this.kb = kb;
	}
	

	
	/**
	 * Returns all the classes (concepts) in a knowldge base
	 * 
	 * @param listImportedClasses
	 *            true if also imported classes should be shown
	 * @return a HashSet containing the names of all the classes in the
	 *         knowledge base
	 */
	public HashSet<String> getAllClasses(boolean listImportedClasses) {
		// extracting all classes from the knowledge base
		Collection classesCollection = this.kb.getClses();

		// creating the HashSet for containing the classes names
		HashSet<String> classes = new HashSet<String>();

		// getting the iterator over the kb classes
		Iterator classesIterator = classesCollection.iterator();

		/*
		 * composing the final HashSet by iterating through the kb classes and
		 * applying filtering rules
		 */
		while (classesIterator.hasNext()) {
			// get a class object
			Cls cls = (Cls) classesIterator.next();

			// checks whether it is not a SYSTEM CLASS and is VISIBLE
			if ((!cls.isSystem()) && (cls.isVisible())) {
				classes.add(cls.getName());
			}
		}

		/*
		 * if the resulting HashSet is not empty returns the HashSet otherwise
		 * returns an empty set and logs a warning on the console
		 */
		if (!classes.isEmpty()) {
			this.log
					.log(Level.INFO,
							"Successfully retrieved classes, returning a non-empty HashSet");

		} else {
			this.log
					.log(
							Level.WARNING,
							"Something bad occurred, perhaps the ontology is empty? returning a empty HashSet");
		}
		return classes;
	}
	
	/**
	 * Lists all the root classes of the ontology (leaved here for backward
	 * compatibility)
	 * 
	 * @param listImportedClasses
	 *            true if also imported classes should be shown
	 * @return an Iterator over the names of all the root classes in the
	 *         knowledge base
	 */
	public HashSet<String> getRoots(boolean listImportedClasses) {
		// extracting the root classes from the knowledge base
		Cls thing = this.kb.getRootCls();

		// the roots of the ontology are all the direct subclasses of thing
		// that are not SYSTEM classes and that are VISIBLE
		Collection rootsCollection = thing.getDirectSubclasses();

		// creating the HashSet for containing the classes names
		HashSet<String> roots = new HashSet<String>();

		// getting the iterator over the kb root classes
		Iterator rootsIterator = rootsCollection.iterator();

		/*
		 * composing the final HashSet by iterating through the kb root classes
		 * and applying filtering rules
		 */
		while (rootsIterator.hasNext()) {
			// get a class object
			Cls cls = (Cls) rootsIterator.next();

			// checks whether it is not a SYSTEM CLASS and is VISIBLE
			if ((!cls.isSystem()) && (cls.isVisible())) {
				roots.add(cls.getName());
			}
		}

		/*
		 * if the resulting HashSet is not empty returns the HashSet otherwise
		 * returns an empty set and logs a warning on the console
		 */
		if (!roots.isEmpty()) {
			this.log
					.log(Level.INFO,
							"Successfully retrieved root classes, returning a non-empty HashSet");

		} else {
			this.log
					.log(
							Level.WARNING,
							"Something bad occurred, perhaps the ontology is empty? returning a empty HashSet");
		}
		return roots;
	}
	
	/**
	 * Lists all the slots declared in a protege knowledge base, excluding the
	 * imported slots if listImported is true. (In this version, listImported is
	 * actually ignored)
	 * 
	 * @param listImported  true if slots coming from imported classes shall be shown -at the TIME is IGNORED
	 * @return an Hastable containing the name of the slots together with the
	 *         eventual annotation properties of the slot
	 */
	public Hashtable<String, RelationType> getAllRelations(boolean listImported) {
		// extracts the collection of all declared slots
		Collection slotsCollection = this.kb.getSlots();

		// gets an Iterator over the slots collection
		Iterator slotsIterator = slotsCollection.iterator();

		// creates the Hashtable for storing results
		Hashtable<String, RelationType> slots = new Hashtable<String, RelationType>();

		/*
		 * Iterates through the slots, hiding the system slots and the
		 * non-visible slots and for each slot tries to load metadata
		 */

		while (slotsIterator.hasNext()) {
			// gets the slot
			Slot slot = (Slot) slotsIterator.next();

			// if slot != null continue
			if (slot != null) {
				// if the slot is not a system slot and is visible
				if ((!slot.isSystem()) && (slot.isVisible())) {
					// gets the slot name
					String slotName = slot.getName();

					// debug
					this.log.log(Level.INFO, "found slot: " + slotName);

					// stores the slot name
					slots.put(slotName, new RelationType());
				}
			}
		}
		return slots;
	}
	
	/**
	 * A method for retrieving all the subclasses of a given class given its
	 * name
	 * 
	 * @param clsName
	 *            the class name
	 * @return an Iterator &lt;String&gt; over the subclasses names
	 */
	public HashSet<String> getSubClasses(String clsName) {
		// getting the class
		Cls cls = this.kb.getCls(clsName);

		// returning the subclasses
		Collection subClassesCollection = this.kb.getDirectSubclasses(cls);

		// creating a new set item in the required form
		HashSet<String> subClassesNames = new HashSet<String>();

		// filling up the HashSet
		Iterator subClassesIterator = subClassesCollection.iterator();

		while (subClassesIterator.hasNext()) {
			// the subclass
			Cls subCls = (Cls) subClassesIterator.next();

			// if the class is not a system class and is visible
			if ((subCls.isVisible()) && (!subCls.isSystem())) {
				// the subclass name
				String subClsName = subCls.getName();

				// debug
				this.log.log(Level.INFO, " " + subClsName + " subclass of: "
						+ clsName);

				// fills-up the HashSet
				subClassesNames.add(subClsName);
			}

		}
		return subClassesNames;
	}
	
	/**
	 * A method that provides the number of direct subclasses for a given class
	 * (identified by its name)
	 * 
	 * @param clsName
	 *            the name of the class
	 * @return the number of subclasses as an int
	 */
	public int getNumberOfSubClasses(String clsName) {
		// getting the class
		Cls cls = this.kb.getCls(clsName);

		// returning the number of subclasses
		return cls.getDirectSubclassCount();

	}
	
	/**
	 * A method for retrieving all the superclasses of a given class given its
	 * name
	 * 
	 * @param clsName
	 *            the class name
	 * @return an Iterator &lt;String&gt; over the superclasses names
	 */
	public HashSet<String> getSuperClasses(String clsName) {
		// getting the class
		Cls cls = this.kb.getCls(clsName);

		// returning the subclasses
		Collection superClassesCollection = this.kb.getDirectSuperclasses(cls);

		// creating a new set item in the required form
		HashSet<String> superClassesNames = new HashSet<String>();

		// filling up the HashSet
		Iterator superClassesIterator = superClassesCollection.iterator();

		while (superClassesIterator.hasNext()) {
			// the superclass
			Cls superCls = (Cls) superClassesIterator.next();

			// if the class is not a system class and is visible
			if ((superCls.isVisible()) && (!superCls.isSystem())) {
				// the subclass name
				String superClsName = superCls.getName();

				// debug
				this.log.log(Level.INFO, " " + superClsName
						+ " superclass of: " + clsName);

				// fills-up the HashSet
				superClassesNames.add(superClsName);
			}
		}
		return superClassesNames;
	}
	
	/**
	 * A method for getting all the instances  in the ontology (for
	 * a RDF/RDFS model)
	 * 
	 * @return the HashSet containing all the instances of a given ontology
	 */
	public HashSet<String> getAllInstances() {
		// the HashSet of all instances
		HashSet<String> instances = new HashSet<String>();

		// getting all instances from the knowledge base
		Collection instancesCollection = this.kb.getInstances();

		// extracting an iterator over the instances
		Iterator instancesIterator = instancesCollection.iterator();

		// iterating through the available instances
		while (instancesIterator.hasNext()) {
			// getting the current instance
			Instance instance = (Instance) instancesIterator.next();

			// checks whether the instance is a system instance or not and if it
			// is
			// visible or not
			if ((instance instanceof SimpleInstance) && (instance.isVisible())
					&& (!instance.isSystem())) {
				// gets the instance name
				String instanceName = instance.getName();

				// adds the instance to the set
				instances.add(instanceName);

				// debug
				this.log.log(Level.INFO, "Found instance: " + instanceName);
			}
		}
		return instances;
	}
	
	/**
	 * A method for getting all the instances of a concept in the ontology (for
	 * a RDF/RDFS model)
	 * 
	 * @param concept The concept URI
	 * @return the HashSet containing all the instances of a given ontology
	 */
	public HashSet<String> getInstancesOfConcept(String concept) {
		// create a set 4 results
		HashSet<String> result = new HashSet<String>();

		// gets the class object
		Cls cls = this.kb.getCls(concept);

		// checks if there are instances for that class
		Collection instancesCollection = cls.getDirectInstances();

		// gets an iterator over the instances
		Iterator instancesIterator = instancesCollection.iterator();

		// search the first simple instance
		while (instancesIterator.hasNext()) {
			// gets the current instance
			Instance instance = (Instance) instancesIterator.next();

			// checks if the istance is a simple instance
			if ((instance instanceof SimpleInstance) && (instance.isVisible())
					&& (!instance.isSystem())) {
				result.add(instance.getName());
			}
		}
		return result;
	}

	/**
	 * A method for checking if a given concept c has some instances or not
	 * 
	 * @param concept
	 *            the concept for which instances might exist
	 * @return true if it has some instances, false otherwise
	 */
	public boolean hasInstances(String concept) {
		// gets the class object
		Cls cls = this.kb.getCls(concept);

		// checks if there are instances for that class
		Collection instancesCollection = cls.getDirectInstances();

		// gets an iterator over the instances
		Iterator instancesIterator = instancesCollection.iterator();

		// search the first simple instance
		while (instancesIterator.hasNext()) {
			// gets the current instance
			Instance instance = (Instance) instancesIterator.next();

			// checks if the istance is a simple instance
			if ((instance instanceof SimpleInstance) && (instance.isVisible())
					&& (!instance.isSystem())) {
				// is the instance is a simple istance true shall be returned
				return true;
			}
		}
		return false;
	}



	@Override
	public boolean addAnnotationProperty4URI(String uri, String property, Object value) {
		// TODO Auto-generated method stub
		return false;
	}



	@Override
	public void confirmLogicViewSupport() {
		// TODO Auto-generated method stub
		
	}



	@Override
	public NodeTypeAbstraction createNodeTypeAbstraction(NodeTypesEnum t, String expression) {
		// TODO Auto-generated method stub
		return null;
	}



	@Override
	public Hashtable<String, Object> getAnnotationProperties(String clsName, boolean isClass) {
		// TODO Auto-generated method stub
		return null;
	}



	@Override
	public Hashtable<String, HashSet<RelationData>> getDatatypeProperties(String concept) {
		// TODO Auto-generated method stub
		return null;
	}



	@Override
	public Hashtable<String, HashSet<NodeTypeAbstraction>> getInstancesDatatypeProperties(String instance) {
		// TODO Auto-generated method stub
		return null;
	}



	@Override
	public Hashtable<String, HashSet<NodeTypeAbstraction>> getInstancesObjectProperties(String instance) {
		// TODO Auto-generated method stub
		return null;
	}



	@Override
	public Hashtable<String, HashSet<RelationData>> getObjectProperties(String uri, boolean direct, boolean listImportedClass) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public boolean removeAnnotationProperty4URI(String clsURI, String property, Object value) {
		// TODO Auto-generated method stub
		return false;
	}



	@Override
	public Hashtable<String, HashSet<NodeTypeAbstraction>> getRestrictions(String clsURI) {
		// TODO Auto-generated method stub
		return null;
	}



	@Override
	public NodeTypeAbstraction getPropertyRange(String uri) {
		// TODO Auto-generated method stub
		return null;
	}



	@Override
	public void setLogicalConstraints(String clsURI, ConceptData data) {
		// TODO Auto-generated method stub
		return ;
	}

	public boolean canQuery()
	{
		return false;
	}
/////////////////////////////// TO BE IMPLEMENTED



	@Override
	public void setLogicalConstraints(String instanceURI, InstanceData data) {
		// TODO Auto-generated method stub
		
	}



@Override
public RelationType getRelationType(String relURI) {
	// TODO Auto-generated method stub
	return null;
}



	
}
