package it.polito.elite.ontology;

import it.polito.elite.visualization.ontoviewer.enums.NodeTypesEnum;

import java.util.Collection;
import java.util.HashSet;

import edu.stanford.smi.protegex.owl.model.OWLAllValuesFrom;
import edu.stanford.smi.protegex.owl.model.OWLCardinality;
import edu.stanford.smi.protegex.owl.model.OWLComplementClass;
import edu.stanford.smi.protegex.owl.model.OWLEnumeratedClass;
import edu.stanford.smi.protegex.owl.model.OWLHasValue;
import edu.stanford.smi.protegex.owl.model.OWLIndividual;
import edu.stanford.smi.protegex.owl.model.OWLIntersectionClass;
import edu.stanford.smi.protegex.owl.model.OWLLogicalClass;
import edu.stanford.smi.protegex.owl.model.OWLMaxCardinality;
import edu.stanford.smi.protegex.owl.model.OWLMinCardinality;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;
import edu.stanford.smi.protegex.owl.model.OWLRestriction;
import edu.stanford.smi.protegex.owl.model.OWLSomeValuesFrom;
import edu.stanford.smi.protegex.owl.model.OWLUnionClass;
import edu.stanford.smi.protegex.owl.model.RDFObject;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import edu.stanford.smi.protegex.owl.model.RDFResource;
import edu.stanford.smi.protegex.owl.model.RDFSClass;
import edu.stanford.smi.protegex.owl.model.RDFSDatatype;
import edu.stanford.smi.protegex.owl.model.RDFSLiteral;

public class NodeTypeOWL extends NodeTypeAbstraction{
	
	
	private String resourceName, onProperty;
	private HashSet<NodeTypeOWL> operands;//Union/intersection operands or OneOf
	private int cardinality=-1;
	private NodeTypeOWL filler;//quantifier filler or complement
	
	
	protected NodeTypeOWL(){}
	
	public NodeTypeOWL(RDFObject resource){
		if(resource instanceof OWLNamedClass){
			this.type=NodeTypesEnum.NamedClass;
			OWLNamedClass classe = (OWLNamedClass) resource;
			this.resourceName=classe.getName();
		}
		else if(resource instanceof OWLIndividual){
			this.type=NodeTypesEnum.NamedInstance;
			OWLIndividual classe = (OWLIndividual) resource;
			this.resourceName=classe.getName();
		}
		else if(resource instanceof OWLEnumeratedClass){
			this.isEnumeration=true;
			this.type = NodeTypesEnum.EnumeratedClass;
			this.operands = new HashSet<NodeTypeOWL>();
			OWLEnumeratedClass classe = (OWLEnumeratedClass) resource;
			Collection<RDFResource> collection= classe.getOneOf();
			for(RDFResource res: collection){
				NodeTypeOWL entry = new NodeTypeOWL(res);
				if(entry.type != NodeTypesEnum.Error)
					this.operands.add(entry);
			}
			if(this.operands.size() == 0)
				this.type = NodeTypesEnum.Error;
		}
		else if(resource instanceof OWLLogicalClass){
			this.isLogicalClass=true;
			if(resource instanceof OWLComplementClass){
				this.type=NodeTypesEnum.ComplementOf;
				OWLComplementClass classe=(OWLComplementClass)resource;
				RDFSClass complemento=classe.getComplement();
				NodeTypeOWL complement=new NodeTypeOWL(complemento);
				if(complement.type != NodeTypesEnum.Error)
					this.filler = complement;
				else
					this.type = NodeTypesEnum.Error;
			}
			else if(resource instanceof OWLUnionClass){
				this.type=NodeTypesEnum.Union;
				OWLUnionClass classe=(OWLUnionClass)resource;
				this.operands = new HashSet<NodeTypeOWL>();
				Collection<RDFSClass> unionSet  = classe.getOperands();
				for(RDFSClass rdfsClasse: unionSet){
					NodeTypeOWL entry=new NodeTypeOWL(rdfsClasse);
					if(entry.type != NodeTypesEnum.Error)
						this.operands.add( entry );
				}
			}
			else if(resource instanceof OWLIntersectionClass){
				this.type=NodeTypesEnum.Intersection;
				OWLIntersectionClass classe=(OWLIntersectionClass)resource;
				this.operands = new HashSet<NodeTypeOWL>();
				Collection<RDFSClass> intersectionSet  = classe.getOperands();
				for(RDFSClass rdfsClasse: intersectionSet){
					NodeTypeOWL entry=new NodeTypeOWL(rdfsClasse);
					if(entry.type != NodeTypesEnum.Error)
						this.operands.add( entry );
				}
			}
		}
		else if(resource instanceof OWLRestriction){
			this.isRestriction=true;
			RDFProperty property=((OWLRestriction)resource).getOnProperty();
			this.onProperty=property.getName();
			if(resource instanceof OWLCardinality){
				this.type=NodeTypesEnum.Cardinality;
				OWLCardinality classe=(OWLCardinality)resource;
				this.cardinality=classe.getCardinality();
				if(classe.isQualified()){
					this.filler = new NodeTypeOWL(classe.getValuesFrom());
					this.type = NodeTypesEnum.QualifiedCardinality;
				}
			}
			else if(resource instanceof OWLMaxCardinality){
				this.type=NodeTypesEnum.MaxCardinality;
				OWLMaxCardinality classe=(OWLMaxCardinality)resource;
				this.cardinality=classe.getCardinality();
				if(classe.isQualified()){
					this.filler = new NodeTypeOWL(classe.getValuesFrom());
					this.type = NodeTypesEnum.QualifiedMaxCardinality;
				}
			}
			else if(resource instanceof OWLMinCardinality){
				this.type=NodeTypesEnum.MinCardinality;
				OWLMinCardinality classe=(OWLMinCardinality)resource;
				this.cardinality=classe.getCardinality();
				if(classe.isQualified()){
					this.filler = new NodeTypeOWL(classe.getValuesFrom());
					this.type = NodeTypesEnum.QualifiedMinCardinality;
				}
			}
			else if(resource instanceof OWLHasValue){
				this.type=NodeTypesEnum.HasValue;
				OWLHasValue classe=(OWLHasValue)resource;
				Object objFiller=classe.getHasValue();//RDFResource, an RDFSLiteral or a primitive value
				if(objFiller instanceof RDFResource){
					NodeTypeOWL myFiller = new NodeTypeOWL((RDFResource)objFiller);
					if(myFiller.type != NodeTypesEnum.Error)
						this.filler = myFiller;
					else
						this.type = NodeTypesEnum.Error;
				}
				else 
					this.type = NodeTypesEnum.Error;
			}
			else if(resource instanceof OWLSomeValuesFrom){
				this.type=NodeTypesEnum.ExistentialQuantifier;
				OWLSomeValuesFrom classe=(OWLSomeValuesFrom)resource;
				RDFResource res=classe.getFiller();//RDFSDatatype, an OWLDataRange, or a RDFSClass.
				if(res instanceof RDFSClass){
					NodeTypeOWL myFiller = new NodeTypeOWL(res);
					if(myFiller.type != NodeTypesEnum.Error)
						this.filler = myFiller;
					else
						this.type = NodeTypesEnum.Error;
				}
				else 
					this.type = NodeTypesEnum.Error;
			}
			else if(resource instanceof OWLAllValuesFrom){
				this.type=NodeTypesEnum.UniversalQuantifier ;
				OWLAllValuesFrom classe=(OWLAllValuesFrom)resource;
				RDFResource res=classe.getFiller();//RDFSDatatype, an OWLDataRange, or a RDFSClass.
				if(res instanceof RDFSClass){
					NodeTypeOWL myFiller = new NodeTypeOWL(res);
					if(myFiller.type != NodeTypesEnum.Error)
						this.filler = myFiller;
					else
						this.type = NodeTypesEnum.Error;
				}
				else 
					this.type = NodeTypesEnum.Error;
			}
		}
		else if(resource instanceof RDFSDatatype){
			this.type=NodeTypesEnum.DataType;
			RDFSDatatype classe = (RDFSDatatype) resource;
			this.resourceName=classe.getName();
		}
		else if(resource instanceof RDFSLiteral){
			this.type=NodeTypesEnum.Literal;
			RDFSLiteral classe = (RDFSLiteral) resource;
			this.resourceName=classe.getString();
		}
		else 
			this.type = NodeTypesEnum.Error;
	}

	@Override
	public String toString() {
		StringBuffer result=new StringBuffer();
		if( this.type == NodeTypesEnum.NamedClass || this.type == NodeTypesEnum.NamedInstance ||
				this.type == NodeTypesEnum.Literal || this.type == NodeTypesEnum.DataType){
			
			result.append(this.resourceName);
		}
		else if( this.type == NodeTypesEnum.Union ||
				this.type == NodeTypesEnum.Intersection){
			
			if( this.type == NodeTypesEnum.Union)
				result.append("UNION of(");
			else 
				result.append("INTERSECTION of(");
			
			int cont = 0, size = this.operands.size();
			for(NodeTypeOWL entry: this.operands){
				cont++;
				result.append(entry.toString());
				if( cont < size )
					result.append(", ");
			}
			result.append(")");
			
		}
		else if( this.type == NodeTypesEnum.ComplementOf){
			NodeTypeOWL complemento=this.filler;
			result.append("COMPLEMENT of("+complemento.toString()+")\n");
		}
		else if( this.type == NodeTypesEnum.EnumeratedClass){
			result.append("ONE of(");
			int cont = 0, size = this.operands.size();
			for(NodeTypeOWL entry: this.operands){
				cont++;
				result.append(entry.toString());
				if( cont < size )
					result.append(", ");
			}
			result.append(")");
		}
		else if(this.isRestriction){//Restriction
			result.append("RESTRICTION on "+this.onProperty+" : ");
			if(this.type == NodeTypesEnum.Cardinality || this.type == NodeTypesEnum.QualifiedCardinality){
				result.append(" CARDINALITY = "+this.cardinality);
				if(this.type == NodeTypesEnum.QualifiedCardinality)
					result.append(" TOWARDS ("+this.filler+")");
			}
			else if(this.type == NodeTypesEnum.MinCardinality || this.type == NodeTypesEnum.QualifiedMinCardinality){
				result.append(" CARDINALITY > "+(this.cardinality-1));
				if(this.type == NodeTypesEnum.QualifiedMinCardinality)
					result.append(" TOWARDS ("+this.filler+")");
			}				
			else if(this.type == NodeTypesEnum.MaxCardinality || this.type == NodeTypesEnum.QualifiedMaxCardinality){
				result.append(" CARDINALITY < "+(this.cardinality+1));
				if(this.type == NodeTypesEnum.QualifiedMaxCardinality)
					result.append(" TOWARDS ("+this.filler+")");
			}		
			else if(this.type == NodeTypesEnum.ExistentialQuantifier)
				result.append(" SOME to (" + this.filler.toString()+")");
			else if(this.type == NodeTypesEnum.UniversalQuantifier)
				result.append(" ALL to ("+ this.filler.toString()+")");
			else if(this.type == NodeTypesEnum.HasValue)
				result.append(" VALUE ("+ this.filler.toString()+")");
		}
		else if(type == NodeTypesEnum.Error)
			result.append("INVALID EXPRESSION");
		return result.toString().trim();
	}
	
	static public NodeTypeAbstraction parseExpression(NodeTypesEnum t, String expression){
		NodeTypeOWL node = null;
		switch(t){
		case NamedClass:
		case NamedInstance:
			node=new NodeTypeOWL();
			node.type = t;
			node.resourceName=expression;
			break;
		default:
		}
		return node;
	}

	@Override
	public HashSet<NodeTypeAbstraction> getLogicSet() {
		HashSet<NodeTypeAbstraction> result=new HashSet<NodeTypeAbstraction>();
		if(this.type == NodeTypesEnum.Union ||
				this.type == NodeTypesEnum.Intersection){
			for(NodeTypeOWL node: this.operands){
				result.add(node);
			}
			return result;
		}
		else
			return null;
	}

	@Override
	public NodeTypeAbstraction getRestrictionFiller() {
		if(this.type == NodeTypesEnum.ExistentialQuantifier ||
				this.type == NodeTypesEnum.UniversalQuantifier ||
				this.type == NodeTypesEnum.HasValue ||
				this.type == NodeTypesEnum.QualifiedCardinality ||
				this.type == NodeTypesEnum.QualifiedMinCardinality ||
				this.type == NodeTypesEnum.QualifiedMaxCardinality){
			return (NodeTypeAbstraction)this.filler;
		}
		else
			return null;
	}

	@Override
	public String getRestrictionPropertyURI() {
		if(this.isRestriction)
			return this.onProperty;
		else 
			return null;
	}

	@Override
	public int getCardinality() {
		return this.cardinality;
	}

	@Override
	public HashSet<NodeTypeAbstraction> getEnumerationSet() {
		HashSet<NodeTypeAbstraction> result=new HashSet<NodeTypeAbstraction>();
		if(this.type == NodeTypesEnum.EnumeratedClass){
			for(NodeTypeOWL node: this.operands){
				result.add(node);
			}
			return result;
		}
		else
			return null;
	}


}
