package it.polito.elite.protege.plugin;



import it.polito.elite.protege.plugin.listeners.ConceptSelectionListener;
import it.polito.elite.protege.plugin.listeners.ModelListener;
import it.polito.elite.visualization.ontoviewer.userInterfaces.ControlPanel;
import it.polito.elite.visualization.ontoviewer.userInterfaces.LogicViewPanel;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;
import it.polito.elite.visualization.ontoviewer.userInterfaces.VisualizationOptions;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.resource.Icons;
import edu.stanford.smi.protege.ui.ClsesPanel;
import edu.stanford.smi.protege.widget.AbstractTabWidget;


public class OntoSpherePlugin extends AbstractTabWidget
{
	/**
	 *  PLUGIN SERIAL UID
	 */
	private static final long	serialVersionUID	= 5383052458452429321L;
	
	//the class logger
	private Logger logger;
	
	//the viewer panel
	public Pane3DScene viewerPanel;
	
	//the knowledge base
	private KnowledgeBase kb;

	//the split pane
	private JSplitPane splitPaneHor;
	
	//the classes tree
	private ClsesPanel classPanel;
	
	//the classes tree listener
	private ConceptSelectionListener conceptSelectionListener;

	public OntoSpherePlugin()
	{
		super();
		// TODO Auto-generated constructor stub
		this.logger = Logger.getLogger("it.polito.elite");
		this.logger.setLevel(Level.INFO);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		// TODO Auto-generated method stub
		// barely copied from protege samples
		edu.stanford.smi.protege.Application.main(args);
	}

	public void initialize()
	{
		// TODO Auto-generated method stub
		this.setLabel("OntoSpherePlugin");
		//sets the icon of the tab
		this.setIcon(Icons.getInstanceIcon());
		this.kb = this.getProject().getKnowledgeBase();
		
		//initialize Visualization Preferences if exist
		VisualizationOptions.initializeVisualizationsOptions();
		
		this.viewerPanel = new Pane3DScene(kb);
		
		
		//debug
		this.logger.log(Level.INFO,"created viewerPane, value: "+viewerPanel);
		
		/*
		 * split the parent panel of the plugin in two parts
		 */
		//the lower part with commands and 3d view
		splitPaneHor =this.createLeftRightSplitPane("horizontal_split",1);      
        
        //creates s buttons container JPanel
        ControlPanel ctrlPane = new ControlPanel(this);
        this.addUserToolBar(ctrlPane.getToolBar());
        
        //creates the class tree panel
        classPanel = new ClsesPanel(this.getProject());
        
        //creates the query panel
        LogicViewPanel lvPane = new LogicViewPanel(this);
        //creates logic view Panel
        
        this.conceptSelectionListener = new ConceptSelectionListener(this);
        //adding a selection listener
        classPanel.addSelectionListener(this.conceptSelectionListener);

        
        //a JTabbedPane for the various interfaces
        JTabbedPane toolTabs = new JTabbedPane();
        
        //adding the old commandPanel to the tabbed pane
        toolTabs.addTab("IsA tree", new ImageIcon(this.getClass().getClassLoader().getResource("resources/classes.gif")),classPanel);
        toolTabs.addTab("Control Panel", new ImageIcon(this.getClass().getClassLoader().getResource("resources/controlPanel.gif")),ctrlPane);
        toolTabs.addTab("Logic Views", new ImageIcon(this.getClass().getClassLoader().getResource("resources/SPARQLlogicViews.gif")),lvPane);

        //adding the command panel
		splitPaneHor.setLeftComponent(toolTabs);

		//adding the 3d panel
		splitPaneHor.setRightComponent(this.viewerPanel);	
		
        //adds the SplitPane to the plugin panel
        add(splitPaneHor);
        
        //adds a listener to the knowledge base
        this.kb.addKnowledgeBaseListener(new ModelListener(this));
	}
	
	public void resetViewer()
	{
		this.viewerPanel = new Pane3DScene(this.kb);
		this.splitPaneHor.setRightComponent(this.viewerPanel);
	}
	


	/**
	 * @return Returns the classPanel.
	 */
	public ClsesPanel getClassPanel() {
		return classPanel;
	}
	
}
