package it.polito.elite.visualization.ontoviewer.models;

import it.polito.elite.ontology.NodeTypeAbstraction;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;


/**
 * ConceptData class summarizes useful information (structural, semantic and positional)
 * related to a given concept. 
 * @author Alessio
 *
 */
public class InstanceData {
	
	private float instanceCubeSide=0;
	private String pertainingConceptURI=null;
	
	/**
	 * 	The table contains semantic arcs toward other concepts 	
	 * each table entry consists of a String key as the relation uri  
	 * and a set of RelationData as Object
	 */
	protected Hashtable<String,HashSet<NodeTypeAbstraction>> outgoingLinks; 
	protected Hashtable<String,HashSet<NodeTypeAbstraction>> incomingLinks;
	protected HashSet<String> selfLoopLinks;
	protected Hashtable<String,HashSet<NodeTypeAbstraction>> datatypeLinks;
	protected HashSet<String> differentFrom, sameAs;
	
	protected String uri = null;
	
	public InstanceData(String uri){
		incomingLinks = new Hashtable<String,HashSet<NodeTypeAbstraction>>();
		outgoingLinks = new Hashtable<String,HashSet<NodeTypeAbstraction>>();
		selfLoopLinks = new HashSet<String>();
		datatypeLinks = new Hashtable<String,HashSet<NodeTypeAbstraction>>();
		this.uri = uri;
	}
		
	public String getPertainingConceptURI() {
		return this.pertainingConceptURI;
	}
	
	public void setPertainingConceptURI(String s) {
		this.pertainingConceptURI = s;
	}
	
	public float getInstanceCubeSide() {
		return instanceCubeSide;
	}
	
	public boolean hasPertainingFacts(){
		if((this.outgoingLinks.size()+this.incomingLinks.size())>0)
			return true;
		else
			return false;
	}
	
	/**
	 * @param instanceCubeSide It sets the instanceCubeSide.
	 */
	public void setInstanceCubeSide(float instanceCubeSide) {
		this.instanceCubeSide = instanceCubeSide;
	}
	
	public void addOutgoingLinks(Hashtable<String,HashSet<NodeTypeAbstraction>> t) {
		this.outgoingLinks.putAll(t);	
	}
	
	public void removeOutgoingLink(String destUri){
		outgoingLinks.remove(destUri);
	}
	
	public void addDatatypeLinks(Hashtable<String,HashSet<NodeTypeAbstraction>> t) {
		this.datatypeLinks.putAll(t);	
	}
		
	public void addIncomingLink(String relURI,NodeTypeAbstraction endURI){
		if(this.incomingLinks.containsKey(relURI)){
			Set<NodeTypeAbstraction> set=this.incomingLinks.get(relURI);
			set.add(endURI);
		}
		else{
			HashSet<NodeTypeAbstraction> set=new HashSet<NodeTypeAbstraction>();
			set.add(endURI);
			this.incomingLinks.put(relURI, set);
		}
	}
	
	/**
	 * @return Returns the datatypeLinks.
	 */
	public Hashtable<String, HashSet<NodeTypeAbstraction>> getDatatypeLinks() {
		return datatypeLinks;
	}

	/**
	 * @param datatypeLinks The datatypeLinks to set.
	 */
	public void setDatatypeLinks(Hashtable<String, HashSet<NodeTypeAbstraction>> datatypeLinks) {
		this.datatypeLinks = datatypeLinks;
	}

	/**
	 * @return Returns the incomingLinks.
	 */
	public Hashtable<String, HashSet<NodeTypeAbstraction>> getIncomingLinks() {
		return incomingLinks;
	}

	/**
	 * @param incomingLinks The incomingLinks to set.
	 */
	public void setIncomingLinks(Hashtable<String, HashSet<NodeTypeAbstraction>> incomingLinks) {
		this.incomingLinks = incomingLinks;
	}

	/**
	 * @return Returns the outgoingLinks.
	 */
	public Hashtable<String, HashSet<NodeTypeAbstraction>> getOutgoingLinks() {
		return outgoingLinks;
	}

	/**
	 * @param outgoingLinks The outgoingLinks to set.
	 */
	public void setOutgoingLinks(Hashtable<String, HashSet<NodeTypeAbstraction>> outgoingLinks) {
		this.outgoingLinks = outgoingLinks;
	}

	/**
	 * @return Returns the uri.
	 */
	public String getUri() {
		return uri;
	}

	/**
	 * @return Returns the selfLoopLinks.
	 */
	public HashSet<String> getSelfLoopLinks() {
		return selfLoopLinks;
	}


	public void addSelfLoopLink(String relURI) {
		this.selfLoopLinks.add(relURI);
	}

	public HashSet<String> getDifferentFrom() {
		return differentFrom;
	}

	public void setDifferentFrom(HashSet<String> differentFrom) {
		this.differentFrom = differentFrom;
	}

	public HashSet<String> getSameAs() {
		return sameAs;
	}

	public void setSameAs(HashSet<String> sameAs) {
		this.sameAs = sameAs;
	}
}
