package it.polito.elite.visualization.ontoviewer.scenes3D;

import it.polito.elite.visualization.ontoviewer.behaviors.MyMouseCursorSelector;
import it.polito.elite.visualization.ontoviewer.behaviors.MyMouseTranslate;
import it.polito.elite.visualization.ontoviewer.behaviors.MyPickBehaviour;
import it.polito.elite.visualization.ontoviewer.models.ConceptData;
import it.polito.elite.visualization.ontoviewer.models.InstanceData;
import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;
import it.polito.elite.visualization.ontoviewer.userInterfaces.VisualizationOptions;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Random;

import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.Bounds;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.Font3D;
import javax.media.j3d.FontExtrusion;
import javax.media.j3d.Group;
import javax.media.j3d.OrientedShape3D;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Text3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Point2d;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.behaviors.mouse.MouseRotate;
import com.sun.j3d.utils.behaviors.mouse.MouseWheelZoom;

/**
 * Scene3D is the abstract base class in the package and any concrete 
 * scene has to extend it. It contains abstract methods wich define an interface and 
 * it provides some utility procedures for creating the scene as
 * 
 * @author alessio.bosca@polito.it
 *
 */
public abstract class Scene3D {
	
	public static final int numScenes=5;
	
	protected Vector3f rootPosition=new Vector3f(0f,0f,0f);
	protected Hashtable<String,Vector3f> positions;
	
	/**
	 * reference to the main Panel
	 */
	public Pane3DScene refMainPanel;
	/**
	 * dimensions of the 3D area
	 */
	private BoundingSphere bounds;
	
	/**
	 * Gruppi che incapsulano la scena
	 */
	protected BranchGroup bgSCENE, bgLabelScene=null;
	protected TransformGroup tgSCENE = null, tgROT;
	
	/**
	 * Behaviours per l'interazione con la scena tramite il mouse
	 */
	protected MyMouseTranslate behaviorTRAS;
	protected MouseWheelZoom behaviorZOOM;
	protected MouseRotate behaviorROT;
	protected MyMouseCursorSelector behaviorCURS;
	/**
	 * Behaviour that manages that captures the mouse clicks and retrieves the 
	 * 3D object picked by its USER DATA field
	 */
	protected MyPickBehaviour simplePick;
	
	/**
	 * number of scene id
	 */
	protected int sceneID=0;
	
	/**
	 * the uris of the current concept 
	 * 	 */
	protected String currentConceptURI;
	protected String currentInstanceURI;
	
	protected Scene3D(Pane3DScene _mf, String conceptUri, String instanceUri, int sceneID){
		this.refMainPanel = _mf;
		this.currentConceptURI = conceptUri;
		this.currentInstanceURI = instanceUri;
		this.sceneID = sceneID;
		this.positions = new Hashtable<String, Vector3f>();
		this.createBGandTGforTheScene();
	}
	
//	Abstract method each scene has to implement
	abstract public void createScene();
	abstract public void selectElement(String uri);
	abstract public void expand_OR_collapseElement(String uri);
	abstract public boolean showContextMenu(final PopupMenu menu, final String uri);
	abstract public boolean showRelationContextMenu(final PopupMenu menu, final HashSet<RelationData> relSet);
	abstract public void fadeInAnimation();
	abstract public void fadeOutAnimation(String uri, Scene3D nextScene);
	
	public void recreateSceneFromScratch(){
		this.tgSCENE.removeAllChildren();
		this.createScene();
		this.repaintCanvas();
	}
	
	public BranchGroup getSceneBranchGroup() {
		//return this.mainBg;
		return this.bgSCENE;
	}
		
	public Bounds getBounds() {
		return this.bounds;
	}
		
	protected void createBGandTGforTheScene() {
		// Create a scene and attach it to the virtual universe
				
		this.bgSCENE= new BranchGroup();
		this.tgROT= new TransformGroup();
		this.tgSCENE = new TransformGroup();
				
		this.tgROT.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		this.tgROT.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		this.tgROT.setCapability(Group.ALLOW_LOCAL_TO_VWORLD_READ);
		this.tgROT.setCapability(Group.ALLOW_CHILDREN_READ);
		this.tgROT.setCapability(Group.ALLOW_CHILDREN_WRITE);
		this.tgROT.setCapability(Group.ALLOW_CHILDREN_EXTEND);
		
		this.tgSCENE.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		this.tgSCENE.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		this.tgSCENE.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		this.tgSCENE.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		this.tgSCENE.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		this.tgSCENE.setCapability(TransformGroup.ALLOW_BOUNDS_READ);
		
		this.bgSCENE.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		this.bgSCENE.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		this.bgSCENE.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		this.bgSCENE.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		this.bgSCENE.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		this.bgSCENE.setCapability(TransformGroup.ALLOW_BOUNDS_READ);
	}
		
	/**
	 * Adds mouse related behaviour to manage scene interaction (pan, zoom, rotate, pick...)
	 *
	 */
	protected void addBehaviors() {	
		
		//Create behaviours and add it to the scene

		//a bounding sphere specifies a region a behavior is active
		BoundingSphere bounds = new BoundingSphere(new Point3d(0.0, 0.0, 0.0),
				Double.POSITIVE_INFINITY);
		
		this.behaviorROT = new MouseRotate();
		this.behaviorROT.setTransformGroup(this.tgROT);
		this.behaviorROT.setSchedulingBounds(bounds);
		this.tgROT.addChild(behaviorROT);
		
		this.behaviorZOOM = new MouseWheelZoom();
		this.behaviorZOOM.setTransformGroup(this.tgROT);
		this.behaviorZOOM.setSchedulingBounds(bounds);
		this.tgROT.addChild(behaviorZOOM);
		
		this.behaviorTRAS = new MyMouseTranslate();
		behaviorTRAS.setTransformGroup(this.tgROT);
		behaviorTRAS.setSchedulingBounds(bounds);
		this.tgROT.addChild(behaviorTRAS);
		
		this.behaviorCURS = new MyMouseCursorSelector(this.refMainPanel);
		behaviorCURS.setTransformGroup(this.tgROT);
		behaviorCURS.setSchedulingBounds(bounds);
		this.tgROT.addChild(behaviorCURS);
		
		this.tgROT.addChild(tgSCENE);
		this.bgSCENE.addChild(tgROT);
		
		this.simplePick=new MyPickBehaviour(this.refMainPanel.myCanvas3D,this); 		
		this.bgSCENE.addChild(simplePick);
	}
	
	/**
	 * remove mouse related behaviours from the scene (actually disables them..)
	 * mainly used during the animations (this way users can't interfere during animation process)
	 */
	protected void removeBehaviours(){
		this.behaviorROT.setEnable(false);
		this.behaviorTRAS.setEnable(false);
		this.behaviorZOOM.setEnable(false);
		this.behaviorCURS.setEnable(false);
		simplePick.setEnable(false);
	}
	
	public void repaintCanvas() {
		this.refMainPanel.myCanvas3D.repaint();
	}
	
	/**
	 * Centers the scene in a new position
	 * @param center Update scene with this new center
	 */
	public void updateView(Vector3f center) {
		Vector3f translation=new Vector3f(-center.x,-center.y,-center.z);
		Transform3D backToInitialView = new Transform3D();
		backToInitialView.setTranslation(translation);
		tgROT.setTransform(backToInitialView);
		repaintCanvas();
	}
	
	public void backToInitialSceneView(){
		Transform3D backToInitialView = new Transform3D();
		backToInitialView.setTranslation(this.rootPosition);
		tgROT.setTransform(backToInitialView);
		repaintCanvas();
	}
	
	/**
	 * Utility methods that create a set of random integers,
	 * useful for assigning void positions in sparse visualizations
	 */
	protected HashSet<Integer> createRandomSet(int numOfElem, int limit){

		HashSet<Integer> voidPositions = new HashSet<Integer>();
		Random rand = new Random(System.currentTimeMillis());
		// randomly assign void positions
		for (int i = 0; i < numOfElem; i++) {
			int r = rand.nextInt(limit);
			while (voidPositions.contains(r)) {
				r = rand.nextInt(limit);
			}
			voidPositions.add(r);
		}
		
		return voidPositions;
	}
	
	/**
	 * 
	 * @return currentConceptURI
	 */
	public String getCurrentConceptURI() {
		return this.currentConceptURI;
	}
	
	public String getCurrentInstanceURI(){
		return this.currentInstanceURI;
	}
	
	/**
	 * Creates a proper BranchGroup
	 * @return a new BranchGroup
	 */
	protected BranchGroup createBranchGroup()
	{
		BranchGroup bgx = new BranchGroup();
		bgx.setCapability(BranchGroup.ALLOW_DETACH);
		bgx.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		bgx.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		bgx.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		return bgx;
	}
	/**
	 * Creates a proper TransformGroup
	 * @return a new TransformGroup
	 */
	protected TransformGroup createTransformGroup()
	{
		TransformGroup tgx = new TransformGroup();
		tgx.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		tgx.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		tgx.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		tgx.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		tgx.setCapability(BranchGroup.ALLOW_DETACH);
		return tgx;
	}
	
	public void addPopUpMenu(PopupMenu menu){
		this.refMainPanel.add(menu);
	}
	
	public void removePopUpMenu(PopupMenu menu){
		this.refMainPanel.remove(menu);
	}
	
	/**
	 * @return Returns the sceneID.
	 */
	public int getSceneID() {
		return sceneID;
	}
	/**
	 * @param currentConceptURI The currentConceptURI to set.
	 */
	public void setCurrentConceptURI(String currentConceptURI) {
		this.currentConceptURI = currentConceptURI;
	}
	/**
	 * @param currentInstanceURI The currentInstanceURI to set.
	 */
	public void setCurrentInstanceURI(String currentInstanceURI) {
		this.currentInstanceURI = currentInstanceURI;
	}
	
	public Point2d get3DTo2DPoint(Point3d point3d)
    {

       Transform3D temp = new Transform3D();
       this.refMainPanel.myCanvas3D.getVworldToImagePlate(temp);
       temp.transform(point3d);
       Point2d point2d =new Point2d();
       this.refMainPanel.myCanvas3D.getPixelLocationFromImagePlate(point3d,point2d);
       return point2d;
    }
	
	public Point3f get2DTo3DPoint(int x , int y)
    {
       Point3d point3d =new Point3d();
       this.refMainPanel.myCanvas3D.getPixelLocationInImagePlate(x, y, point3d);
       Transform3D temp = new Transform3D();
       this.refMainPanel.myCanvas3D.getImagePlateToVworld(temp);
       temp.transform(point3d);
       return new Point3f(point3d);
    }
	
	public Dimension getCanvas3DSize(){
		return this.refMainPanel.myCanvas3D.getSize();
	}
	
	public Vector3f centralPositionAtGivenHeightScale(float scale){
//		setto il nodo iniziale in base alle dimensioni dello schermo
		Dimension screenSize = this.getCanvas3DSize();
		int x = screenSize.width/2;
		int y = (int) (screenSize.height*scale);
		Vector3f center = new Vector3f(this.get2DTo3DPoint(x,y));
		return center;
	}
	
	/**
	 * Gets all the Shapes3D Objects rooted in a given TransformGroup
	 * @param group the Group containing the Shape3D to retrieve
	 * @return an array containing all the retrieved Shapes3Ds
	 */
	public static Shape3D[] getShapes(Group group) {
		ArrayList<Shape3D> shapes = new ArrayList<Shape3D>();
		
		getShapes(group.getAllChildren(), shapes);

		Shape3D[] result = new Shape3D[0];
		result = shapes.toArray(result);

		return result;
	}
	
	/**
	 * A recursive method for exploring the tree and retrieving the Shapes3D
	 * @param p_children
	 * @param p_shapes
	 */
	private static void getShapes(Enumeration p_children, ArrayList<Shape3D> p_shapes) {

		while (p_children.hasMoreElements()) {
			Object object = p_children.nextElement();

			if (object instanceof Shape3D) {
				p_shapes.add((Shape3D)object);
			}

			if (object instanceof Group ) {
				getShapes(((Group) object).getAllChildren(), p_shapes);
			}

		}
	}
	
	protected void showSelectionPopUp(String content){
		PopupMenu menu=createPopupSelectionMenu(content, false);
		this.refMainPanel.add(menu);
		menu.show(this.simplePick.getMevent().getComponent().getParent(), 
				this.simplePick.getMevent().getX(),this.simplePick.getMevent().getY());
	}
	
	private PopupMenu createPopupSelectionMenu(String logicsetDescritpion, boolean isSubMenu){
		
		PopupMenu popup;
		//POPUP MENU TITLE
		int start=logicsetDescritpion.indexOf('(');
		int end=logicsetDescritpion.lastIndexOf(')');
		if(start >=0 && end >0){
			//retrieving header info as INTERSECTION OF or UNION OF.. and insert it as the popup menu title
			String title=logicsetDescritpion.substring(0, start);
			if(title.trim().length()>1){
				if(! isSubMenu){
					popup	= new PopupMenu();
					popup.add(title);
				}
				else
					popup = new PopupMenu(title);
			}
			else{
				//Simple list of elements - tipically this is not a restriction range but a set of reachable
				//elements within a side relation in a tree scene (scene 2, all the elm not in the scene related to a given concept)
				if(! isSubMenu){
					popup	= new PopupMenu();
					popup.add("ELEMENTS:");
				}
				else
					popup = new PopupMenu("ELEMENTS:");
			}
			
			if(! isSubMenu)
				popup.addSeparator();
		   	
			//retrieve elements and iterate over them
			String content=logicsetDescritpion.substring(start + 1, end);
			
			String[] elements=content.split(",");
			for(int idx=0; idx<elements.length; idx++){
				String elm = elements[idx].trim();
				
				if( elm.indexOf('(') > 0){
					//is it a logic set - find its delimitations, aggregating properly involved elements and then creating a submenu..
					String subMenuDescription = new String(elm);
					int totOpen=0, totClosed=0;
					totOpen += this.computeOccurenceOf(elm, '(');
					totClosed += this.computeOccurenceOf(elm, ')');
					while(totOpen != totClosed){
						String str = elements[++idx].trim();
						totOpen += this.computeOccurenceOf(str, '(');
						totClosed += this.computeOccurenceOf(str, ')');
						subMenuDescription = subMenuDescription.concat(","+str);
					}
					PopupMenu submenu=createPopupSelectionMenu(subMenuDescription, true);
					popup.add(submenu);
				}
				else{
					//create and add clickable item
					MenuItem item=new MenuItem(elm);
					final ConceptData cdata=refMainPanel.model.getConceptData(elm);
					final InstanceData idata=refMainPanel.model.getInstanceData(elm);
//					if it is a recognized uri (concept or instance) make it clickable and bold
					if(cdata != null || idata != null){ 
						item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
						item.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								if(cdata != null){
									SceneFocusConcept3 newScene=new SceneFocusConcept3(refMainPanel, cdata.getUri());
									refMainPanel.changeScene(newScene);
								}
								else if(idata != null){
										SceneFocusFacts5 newScene = new SceneFocusFacts5(refMainPanel, 
												idata.getPertainingConceptURI(), idata.getUri());
										refMainPanel.changeScene(newScene);
								}
							}
						});
					}
					popup.add(item);
				}
			}
			
		}
		else{
			popup = new PopupMenu();
			popup.add(logicsetDescritpion);
		}
		return popup;
	}
	
	private int computeOccurenceOf(String source, char ch){
		int tot = 0, start;
		start = source.indexOf(ch);
		while(start>0){
			tot ++;
			start = source.indexOf(ch, start+1);
		}
		return tot;
	}
	
	public void addSceneLabel(String textString) {
		Point3f pos=this.get2DTo3DPoint(65, 85);
		Color3f col=new Color3f(.6f,.0f,.0f);
		
		if(this.bgLabelScene != null)
			this.bgSCENE.removeChild(this.bgLabelScene);
		this.bgLabelScene=this.createBranchGroup();
		Transform3D translPos=new Transform3D();
		translPos.setTranslation(new Vector3f(pos));
		TransformGroup tgLabel=new TransformGroup(translPos);
		// inserisce la label e la aggiunge al tgx
		float labelScale=.052f;
		// create 3D text
		Font3D f3d = new Font3D(new Font(VisualizationOptions.sG_FontFamily,
				VisualizationOptions.sG_FontModifier, 1), new FontExtrusion());

		Text3D txt = new Text3D(f3d, textString);
		txt.setAlignment(Text3D.ALIGN_FIRST);

		OrientedShape3D textShape = new OrientedShape3D();
		textShape.setGeometry(txt);
		textShape.setPickable(false);

		TransformGroup objScale = new TransformGroup();
		Transform3D textMat = new Transform3D();
		textMat.setScale(labelScale);
		objScale.setTransform(textMat);

		Appearance apText = new Appearance();
		ColoringAttributes colorTxt = new ColoringAttributes(col, 1);
		apText.setColoringAttributes(colorTxt);
		apText.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
		apText.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		apText.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
		textShape.setAppearance(apText);

		textShape.setAlignmentMode(OrientedShape3D.ROTATE_NONE);

		objScale.addChild(textShape);
		tgLabel.addChild(objScale);
		this.bgLabelScene.addChild(tgLabel);
		this.getSceneBranchGroup().addChild(this.bgLabelScene);
	}
}

