package it.polito.elite.visualization.ontoviewer.scenes3D;

import it.polito.elite.ontology.NodeTypeAbstraction;
import it.polito.elite.visualization.ontoviewer.enums.LinkDirectionEnum;
import it.polito.elite.visualization.ontoviewer.enums.NodeTypesEnum;
import it.polito.elite.visualization.ontoviewer.models.DependencyTreeElm;
import it.polito.elite.visualization.ontoviewer.models.InstanceData;
import it.polito.elite.visualization.ontoviewer.models.OrderedURIsElm;
import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.models.RelationType;
import it.polito.elite.visualization.ontoviewer.tools.InstanceCube;
import it.polito.elite.visualization.ontoviewer.tools.Relation3DObject;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;
import it.polito.elite.visualization.ontoviewer.userInterfaces.VisualizationOptions;

import java.awt.Color;
import java.awt.Font;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Hashtable;

import javax.media.j3d.BranchGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.swing.JColorChooser;
import javax.vecmath.Color3f;
import javax.vecmath.Point2f;
import javax.vecmath.Vector3f;

public class SceneDependencyTree6 extends Scene3D {

	private HashSet<String> nodes2_BeDrawn, expandable, onScene;;

	private Hashtable<String, BranchGroup> conceptsBg, conceptSpheresBg;

	private Hashtable<String, DependencyTreeElm> tree;

	private String relationURI;
	private String expanded = null;

	private LinkDirectionEnum dir;

	public SceneDependencyTree6(Pane3DScene _mf, String uri,
			String relationUri, LinkDirectionEnum dir) {
		super(_mf, null, uri, 5);
		this.dir = dir;

		InstanceData elmData = this.refMainPanel.model.getInstanceData(uri);

		this.currentInstanceURI = uri;
		this.currentConceptURI = ((InstanceData) elmData)
				.getPertainingConceptURI();

		this.nodes2_BeDrawn = new HashSet<String>();
		this.expandable = new HashSet<String>();
		this.onScene = new HashSet<String>();
		this.conceptsBg = new Hashtable<String, BranchGroup>();
		this.conceptSpheresBg = new Hashtable<String, BranchGroup>();
		this.conceptsBg = new Hashtable<String, BranchGroup>();
		this.conceptSpheresBg = new Hashtable<String, BranchGroup>();
		this.tree = new Hashtable<String, DependencyTreeElm>();
		this.relationURI = relationUri;

		this.createBGandTGforTheScene();
		this.addSceneLabel(relationURI+" dependency tree");
		this.createScene();
		this.addBehaviors();
		this.repaintCanvas();
	}

	public void createScene() {
		// ricalcolo le posizioni per l'albero corrente
		this.rootPosition = this.centralPositionAtGivenHeightScale(VisualizationOptions.s6_ScreenPercent_as_RootVerticalPosition);
		this.computeNodePosition(this.currentInstanceURI, this.rootPosition,	null);
		//If just a subtree can be expanded expand it
		while (this.expandable.size() == 1) {
			String newUri = this.expandable.iterator().next();
			this.expandable.clear();
			Vector3f center = this.positions.get(newUri);
			String parent = this.tree.get(newUri).getParentUri();
			this.computeNodePosition(newUri, center, parent);
		}
		//draw nodes
		this.drawNodes();
	}

	private HashSet<String> extractFunctionallyRelatedElements(InstanceData data) {
		HashSet<String> functionallyRelatedElments = new HashSet<String>();

		if (dir == LinkDirectionEnum.out) {
			if (data.getOutgoingLinks().containsKey(this.relationURI)) {
				for (NodeTypeAbstraction myRange : data.getOutgoingLinks().get(
						this.relationURI)) {
					NodeTypesEnum type = myRange.getType();
					if (type == NodeTypesEnum.NamedClass || type == NodeTypesEnum.NamedInstance) {
						functionallyRelatedElments.add(myRange.toString());
					}
				}
			}
		} else {
			if (data.getIncomingLinks().containsKey(this.relationURI)) {
				for (NodeTypeAbstraction myRange : data.getIncomingLinks().get(	this.relationURI)) {
					NodeTypesEnum type = myRange.getType();
					if (type == NodeTypesEnum.NamedClass || type == NodeTypesEnum.NamedInstance) {
						functionallyRelatedElments.add(myRange.toString());
					}
				}
			}
		}

		return functionallyRelatedElments;
	}

	private void computeNodePosition(String uri, Vector3f center,
			String uriParent) {
		// Computes the position of the second level taken into account the
		// third one
		nodes2_BeDrawn.add(uri);
		if (!onScene.contains(uri)) {
			onScene.add(uri);
			this.positions.put(uri, center);
		}
		// A LinkedElementData is a Concept OR Instance Data
		InstanceData elmData = this.refMainPanel.model.getInstanceData(uri);
		

		DependencyTreeElm elm;
		if (!this.tree.containsKey(uri)) {
			HashSet<String> functionallyRelated = this
					.extractFunctionallyRelatedElements(elmData);
			elm = new DependencyTreeElm(functionallyRelated, uriParent, 0);
			this.tree.put(uri, elm);
		} else {
			elm = this.tree.get(uri);
		}

		// Filter out element that already are on the scene,
		// their position doesn't have to be computed
		HashSet<String> children2BeDrawn = new HashSet<String>();
		for (String childUri : elm.getChildren()) {
			if (!this.onScene.contains(childUri))
				children2BeDrawn.add(childUri);
		}
		int setSize = children2BeDrawn.size();

		if (setSize > 0) {
			float[] angles = new float[setSize];
			// order the set of functionally relatesd instances by numbers of
			// "children" in the tree
			OrderedURIsElm[] array = new OrderedURIsElm[setSize];
			int ind = 0;
			for (String childUri : children2BeDrawn) {
				if (!this.onScene.contains(childUri)) {
					InstanceData myData = this.refMainPanel.model
							.getInstanceData(childUri);
					HashSet<String> fset = this
							.extractFunctionallyRelatedElements(myData);
					DependencyTreeElm elmx = new DependencyTreeElm(fset, uri,
							elm.getLvl() + 1);
					this.tree.put(childUri, elmx);
					array[ind++] = new OrderedURIsElm(childUri, fset.size());
				}
			}
			Arrays.sort(array);

			// Compute the angular position for each functional "child"
			float alpha = (float) (2 * Math.PI / setSize);
			float alpha0 = (float) (Math.random() * 2 * Math.PI);

			// Assign position of sphere-concepts within the positionin circle
			// line iterleaving children decreasingly having an higher number of
			// grandchildren with ones having a lower number of them
			for (int k = 0, i = 0, j = setSize - 1; k < setSize; k++) {
				if (k % 2 == 0)// even turn
					angles[i++] = (alpha0 + alpha * k);
				else
					angles[j--] = (alpha0 + alpha * k);
			}

			// Calculate the smallest radius (as if no sub levels were present)
			float radius = this.getSubRadius(setSize);

			// Ampliate the radius by detecting collision betwenn circles of
			// children and properly increasing radius if so
			for (int i = 0; i < setSize; i++) {
				if (array[i].getNumberOfChildren() < 1)
					continue;
				float theta = angles[i];
				Point2f a = new Point2f((float) (radius * Math.cos(theta)),
						(float) (radius * Math.sin(theta)));
				float radiusA = this.getSubDistanceRadius(array[i]
						.getNumberOfChildren());
				for (int j = i + 1; j < setSize; j++) {
					if (array[j].getNumberOfChildren() < 1)
						continue;
					float theta1 = angles[j];
					Point2f b = new Point2f(
							(float) (radius * Math.cos(theta1)),
							(float) (radius * Math.sin(theta1)));
					float radiusB = this.getSubDistanceRadius(array[j]
							.getNumberOfChildren());
					float minimumDistance = radiusA + radiusB;
					if (a.distance(b) < minimumDistance) {
						radius = calculateNewRadius(theta, theta1,
								minimumDistance);
					}
				}
			}
			// Set the Position for the children
			for (int i = 0; i < setSize; i++) {
				float x = (float) (radius * Math.cos(angles[i])) + center.x;
				float z = (float) (radius * Math.sin(angles[i])) + center.z;
				Vector3f posChild = new Vector3f(x, center.y
						- VisualizationOptions.s2_TreeLevelHeight, z);
				setPos(array[i].getUri(), posChild, array[i]
						.getNumberOfChildren(), 2, true);
			}
		}
	}

	// Set position to the second lvl and and computes the third position
	private void setPos(String uri, Vector3f center, int numberOfChildren,
			int lvl, boolean propagate) {


		this.positions.put(uri, center);

		nodes2_BeDrawn.add(uri);
		onScene.add(uri);
		
		if (!propagate) {
			DependencyTreeElm elm = this.tree.get(uri);
			if (numberOfChildren > 0){
				boolean anyChildrenNotOnScene=false;
				for(String childrenURI: elm.getChildren()){
					if(!this.onScene.contains(childrenURI))
						anyChildrenNotOnScene = true;
				}
				if(anyChildrenNotOnScene)
					this.expandable.add(uri);
			}
			return;
		}

		// Filter out element that already are on the scene,
		// their position doesn't have to be computed
		HashSet<String> children2BeDrawn = new HashSet<String>();
		for (String childUri : this.tree.get(uri).getChildren()) {
			if ( !this.onScene.contains(childUri) )
				children2BeDrawn.add(childUri);
		}
		int setSize = children2BeDrawn.size();

		OrderedURIsElm[] array = new OrderedURIsElm[setSize];
		int ind = 0;
		for (String childUri : children2BeDrawn) {
			InstanceData data = this.refMainPanel.model
					.getInstanceData(childUri);
			HashSet<String> fset = this
					.extractFunctionallyRelatedElements(data);
			DependencyTreeElm elm = new DependencyTreeElm(fset, uri, lvl);
			this.tree.put(childUri, elm);
			array[ind++] = new OrderedURIsElm(childUri, fset.size());
		}
		Arrays.sort(array);
		float[] angles = new float[array.length];

		// Compute the angular position for each child
		int numSons = array.length;
		float alpha = (float) (2 * Math.PI / numSons);
		float alpha0 = (float) Math.random();

		// Assign position of sphere-concepts within the positionin circle line
		// iterleaving children decreasingly having an higher number of
		// grandchildren
		// with ones having a lower number of them
		for (int k = 0, i = 0, j = numSons - 1; k < numSons; k++) {
			if (k % 2 == 0)// even turn
				angles[i++] = (alpha0 + alpha * k);
			else
				angles[j--] = (alpha0 + alpha * k);
		}

		// Calculate the smallest radius (as if no sub levels were present)
		float radius = getSubRadius(numSons);

		// Set the Position for the children
		for (int i = 0; i < numSons; i++) {
			float x = (float) (radius * Math.cos(angles[i]));
			float z = (float) (radius * Math.sin(angles[i]));
			Vector3f posChild = new Vector3f(center.x + x, center.y
					- VisualizationOptions.s2_TreeLevelHeight, center.z + z);
			setPos(array[i].getUri(), posChild, array[i].getNumberOfChildren(),
					lvl + 1, false);
		}
	}

	private float getSubRadius(int numberOfChildren) {
		float radius = 0;
		if (numberOfChildren > 1 && numberOfChildren < 5) {
			radius = 3 * VisualizationOptions.s2_SimpleSphereRadius;
		} else if (numberOfChildren > 4) {
			radius = (float) (VisualizationOptions.s2_Segment2ArcCorrectionFactor
					* numberOfChildren
					* (VisualizationOptions.s2_SimpleSphereRadius + VisualizationOptions.sG_distance_min) / (Math.PI));
		}
		return radius;
	}

	private float getSubDistanceRadius(int numberOfChildren) {
		float radius = 0;
		if (numberOfChildren == 1) {
			radius = VisualizationOptions.s6_FamilyRadiusMultiplier
					* VisualizationOptions.s2_SimpleSphereRadius;
		}
		if (numberOfChildren > 1 && numberOfChildren < 5) {
			radius = 3 * VisualizationOptions.s6_FamilyRadiusMultiplier
					* VisualizationOptions.s2_SimpleSphereRadius;
		} else if (numberOfChildren > 4) {
			radius = (float) (VisualizationOptions.s2_Segment2ArcCorrectionFactor
					* numberOfChildren
					* VisualizationOptions.s6_FamilyRadiusMultiplier
					* (VisualizationOptions.s2_SimpleSphereRadius + VisualizationOptions.sG_distance_min) / (Math.PI));
		}
		return radius;
	}

	private float calculateNewRadius(float angle1, float angle2, float distance) {
		float k = (float) (Math.pow((Math.cos(angle1) - Math.cos(angle2)), 2) + Math
				.pow((Math.sin(angle1) - Math.sin(angle2)), 2));
		return (float) (Math.sqrt(Math.pow(distance, 2) / k));
	}

	private BranchGroup drawDependencyRelation(Vector3f posChildren,
			Vector3f center, float dependencyElmSize, boolean drawCurved) {

		// disegna la relazioni di dipendenza
		Vector3f relationEnd = new Vector3f(posChildren.x - center.x,
				posChildren.y - center.y, posChildren.z - center.z);
		Relation3DObject relation = new Relation3DObject();
		relation.setTransparency(.3f);
		
		Hashtable<String, RelationType> relationsSet = new Hashtable<String, RelationType>(1);
		relationsSet.put(this.relationURI, this.refMainPanel.model.getRelationTypes().get(this.relationURI));
		
		if (drawCurved)
			return relation.createInstanceCurvedArrowRelation(relationsSet,
					relationEnd, VisualizationOptions.s6_CurvedArrowCurvature,
					this.dir);
		else
			return relation.createInstanceStraightRelation(relationsSet, relationEnd,
					dependencyElmSize, this.dir);
	}

	/**
	 * This method expand a concept on the scene that is expandable(a withe one)
	 * Such action implies the collapsing of all the nodes except the one in the
	 * direct line of ancestor from the current concept to the root. A subtree
	 * of 3 level depth is drawed beneth the selected concept. The nodes
	 * belonging to the ancestor line and the expandable ones in the subtree are
	 * saved in an expandable list
	 * 
	 * @param uri
	 *            the concept to be expanded
	 */
	private void expandNode(String uri) {
		// clears the previuos list
		this.expanded = uri;
		this.expandable.remove(uri);
		this.conceptsBg.get(this.expanded).detach();
		this.nodes2_BeDrawn.add(uri);
		
		Vector3f newPos = this.positions.get(uri);

		// eliminates potentially colliding subtree, if present
		DependencyTreeElm elm = this.tree.get(uri);
		String parent = elm.getParentUri();
		if(parent != null){
			DependencyTreeElm parentElm = this.tree.get(parent);
			for (String uriSon : parentElm.getChildren()) {
				DependencyTreeElm childElm = this.tree.get(uriSon);
				if (!uriSon.equals(uri) && this.isCollapsable(childElm) ){
					collapseSubTree(uriSon, elm);
				}
			}
		}
		// computes the new Position
		this.computeNodePosition(uri, newPos, parent);
		//If just a subtree can be expanded expand it
		while (this.expandable.size() == 1) {
			String newUri = this.expandable.iterator().next();
			this.expandable.clear();
			Vector3f center = this.positions.get(newUri);
			String myParent = this.tree.get(newUri).getParentUri();
			this.computeNodePosition(newUri, center, myParent);
		}
		this.drawNodes();

		// udates the view
		this.updateView(newPos);
		repaintCanvas();
	}

	
	/**
	 * Recursive procedure that Collapse a subTree
	 * 
	 */
	private void collapseSubTree(String uri, DependencyTreeElm myElm) {

//Collapse children with level > levelURI and make its appearence expandable		
		
		if (this.isCollapsable(myElm)) {
			this.expandable.add(uri);
			removeChildrenFromScene(uri, myElm.getLvl());
			
			// Make appareance of node "expandable"
			this.conceptsBg.get(uri).detach();
			InstanceData idata = this.refMainPanel.model.getInstanceData(uri);
			BranchGroup bgx = this.drawDependencyTreeElm(uri, myElm, idata);
			
			String parent = myElm.getParentUri();
			if(parent != null){
				BranchGroup parentBG = this.conceptsBg.get(parent);
				parentBG.addChild(bgx);
			}
			else{
				this.tgSCENE.addChild(bgx);
			}
			this.conceptsBg.put(uri, bgx);
		}
	}
	
	private boolean isCollapsable(DependencyTreeElm myElm){
//		If the concept has "funtional" children with a lvl greather than is own can be collapsed, otherwise no
		boolean isCollapsable = false;
		int lvlURI = myElm.getLvl();
		for(String childURI: myElm.getChildren()){
			if(this.tree.get(childURI).getLvl() > lvlURI)
				isCollapsable = true;
		}
		return isCollapsable;
	}
	
	private void removeChildrenFromScene(String uri, int origLevel) {
		DependencyTreeElm data = this.tree.get(uri);
		String uriParent = data.getParentUri();

			for (String uriSon : data.getChildren()) {
				if(this.onScene.contains(uriSon) && !uriSon.equals(uriParent)){
					DependencyTreeElm sonElm=this.tree.get(uriSon);
					this.onScene.remove(uriSon);
					this.conceptsBg.get(uriSon).detach();
					if(sonElm.getLvl() == origLevel)
						this.nodes2_BeDrawn.add(uriSon);
					removeChildrenFromScene(uriSon, origLevel);
				}
			}

	}
	
	private BranchGroup drawDependencyTreeElm(String uri,
			DependencyTreeElm elm, InstanceData data) {
		//System.out.println("Drawing "+uri);
		BranchGroup bgx = this.createBranchGroup();
		TransformGroup tgx = this.createTransformGroup();

		// posiziona (transla il transform group)
		Vector3f center = this.positions.get(uri);
		Transform3D transf = new Transform3D();
		transf.setTranslation(center);
		tgx.setTransform(transf);

		float size = 0f;// cube side or sphere radius
		// Drawing the element itself
		InstanceCube cubo = new InstanceCube();
		Color3f color;
		if (this.expandable.contains(uri)) {
			color = VisualizationOptions.s2_ParentSphereColor;
			// creating the concept sphere
			cubo.setLabelAppearance(
					VisualizationOptions.s2_LabelParentSphereColor,
					VisualizationOptions.s2_LabelParentSphereScale);
		} else {
			// Set color to the sphere
			color = VisualizationOptions.chooseColor(elm.getLvl());
			cubo.setLabelAppearance(
					VisualizationOptions.s2_LabelSimpleSphereColor,
					VisualizationOptions.s2_LabelSimpleSphereScale);

		}
		((InstanceData) data).setInstanceCubeSide(0.03f);
		size = ((InstanceData) data).getInstanceCubeSide();
		tgx.addChild(cubo.createColoredCube(color, uri, ((InstanceData) data)
				.hasPertainingFacts()));

		// Drawing dependency relations
		for (String uriChild : elm.getChildren()) {
			//System.out.println("\tdrawing relation from/to "+uriChild);
			if (this.nodes2_BeDrawn.contains(uriChild) || this.onScene.contains(uriChild)) {
				InstanceData childData = this.refMainPanel.model.getInstanceData(uriChild);
				String parentUri = elm.getParentUri();
				if (parentUri != null && parentUri.equals(uriChild))
					tgx.addChild(this.drawDependencyRelation(this.positions.get(uriChild), 
							this.positions.get(uri), size, true));
				else
					tgx.addChild(this.drawDependencyRelation(this.positions
							.get(uriChild), this.positions.get(uri), size, false));
			}
		}

		bgx.addChild(tgx);
		this.conceptsBg.put(uri, bgx);
		return bgx;
	}

	/**
	 * procedure which draws the nodes in the internal queue and then clears the
	 * queue itself
	 */
	private void drawNodes() {
		for (String uri : this.nodes2_BeDrawn) {
			InstanceData data = this.refMainPanel.model.getInstanceData(uri);
			DependencyTreeElm elm = this.tree.get(uri);
			this.tgSCENE.addChild(this.drawDependencyTreeElm(uri, elm, data));
		}
		this.nodes2_BeDrawn.clear();
	}

	private float computeConceptSphereRadius(int subTreeDepth) {
		float radius = VisualizationOptions.s2_SimpleSphereRadius
				+ subTreeDepth
				* VisualizationOptions.s2_ParentSphereRadiusMultiplier;
		if ((radius > VisualizationOptions.s2_ParentSphereRadiusLimit)
				&& (VisualizationOptions.s2_ParentSphereRadiusLimit > 0))
			radius = VisualizationOptions.s2_ParentSphereRadiusLimit;
		return radius;
	}

	@Override
	public boolean showContextMenu(final PopupMenu menu, final String uri) {
		if (expandable.contains(uri)) {
			MenuItem item2 = new MenuItem("Expand subtree");
			item2.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					expandNode(uri);
					removePopUpMenu(menu);
				}
			});
			menu.add(item2);
		}

		return true;
	}

	@Override
	public void selectElement(String uri) {
		Scene3D myScene = new SceneFocusFacts5(this.refMainPanel,
					this.currentConceptURI, uri);
		this.refMainPanel.changeScene(myScene);
	}

	@Override
	public void expand_OR_collapseElement(String uri) {
		if (this.expandable.contains(uri))
			expandNode(uri);
		else {
			DependencyTreeElm itemElm = this.tree.get(uri);
			collapseSubTree(uri, itemElm);
		}
	}

	@Override
	public boolean showRelationContextMenu(final PopupMenu menu, HashSet<RelationData> relSet) { 
		
		PopupMenu setColor=new PopupMenu("Set Color To :");
		menu.add(setColor);
		boolean found=false;
		for (RelationData reldata : relSet) {
			final String relname=reldata.getRelationName();
			final LinkDirectionEnum dir=reldata.getDir();
			MenuItem item = new MenuItem(relname);
			item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Color c = JColorChooser.showDialog(null, "Choose a color...", Color.WHITE);
					refMainPanel.model.setColorToRelation(relname, c);
					removePopUpMenu(menu);
				}
			});
			setColor.add(item);
			if(Relation3DObject.getRelationColor(relname)!=null)
				found=true;
		}
		
		if(found){
			PopupMenu unsetColor=new PopupMenu("Remove Custom Color");
			menu.add(unsetColor);
			for (RelationData reldata : relSet) {
				final String relname=reldata.getRelationName();
				if(Relation3DObject.getRelationColor(relname)!=null){
					final LinkDirectionEnum dir=reldata.getDir();
					MenuItem item = new MenuItem(relname);
					item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
					item.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							refMainPanel.model.removeColorToRelation(relname);
							removePopUpMenu(menu);
						}
					});
					unsetColor.add(item);
				}
			}
		}
		
		return true; 
	}

	@Override
	public void fadeInAnimation() {
		switch (this.refMainPanel.getOldSceneID()) {
		default:
			if (refMainPanel.isPrimary)
				this.refMainPanel.primaryBG.removeAllChildren();
			else
				this.refMainPanel.secondaryBG.removeAllChildren();
		}
	}

	@Override
	public void fadeOutAnimation(String uri, Scene3D nextScene) {	}

}
