package it.polito.elite.visualization.ontoviewer.scenes3D;

import it.polito.elite.ontology.NodeTypeAbstraction;
import it.polito.elite.protege.plugin.listeners.PostAnimation_SceneChanger;
import it.polito.elite.visualization.ontoviewer.behaviors.ScaleInterpolatorWithListener;
import it.polito.elite.visualization.ontoviewer.enums.GraphicalLinkTypeEnum;
import it.polito.elite.visualization.ontoviewer.enums.LinkDirectionEnum;
import it.polito.elite.visualization.ontoviewer.enums.NodeTypesEnum;
import it.polito.elite.visualization.ontoviewer.models.ConceptData;
import it.polito.elite.visualization.ontoviewer.models.InstanceData;
import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.tools.ConceptSphere;
import it.polito.elite.visualization.ontoviewer.tools.Relation3DObject;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;
import it.polito.elite.visualization.ontoviewer.userInterfaces.VisualizationOptions;

import java.awt.Color;
import java.awt.Font;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import javax.media.j3d.Alpha;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ScaleInterpolator;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.swing.JColorChooser;
import javax.vecmath.AxisAngle4f;
import javax.vecmath.Color3f;
import javax.vecmath.Vector3f;

/**
 * 
 * @author Alessio
 * 
 */
public class SceneFocusConcept3 extends Scene3D {

	/**
	 * Used to keep track of the Relations inherited by the fathers chain of the
	 * concept
	 */
	Hashtable<String, HashSet<RelationData>> inheritedOutgoingRelations, inheritedIncomingRelations,
	inheritedDataTypeRelations;
	HashSet<RelationData> inheritedSelfLoops;
	
	/**
	 * the main information source for the current concept
	 */
	 private ConceptData currentConceptData;
	/**
	 * Keeps track of the roots(many of them are possible since adoptive
	 * parents..)
	 */
	Hashtable<String, HashSet<String>> AncestorRoots;
	
	Hashtable<String,RelationData> universalQuantifiers;
	
	private boolean showChildren, showDirectRelations, showInheritedRelations, showOutgoing, showIncoming, showDatatype;

	public SceneFocusConcept3(Pane3DScene _mf, String uri) {
		super(_mf,uri,null,3);
		this.inheritedIncomingRelations = new Hashtable<String, HashSet<RelationData>>();
		this.inheritedOutgoingRelations = new Hashtable<String, HashSet<RelationData>>();
		this.inheritedDataTypeRelations = new Hashtable<String, HashSet<RelationData>>();
		this.inheritedSelfLoops = new HashSet<RelationData>();
		this.AncestorRoots = new Hashtable<String, HashSet<String>>();
		showDirectRelations = showInheritedRelations = true;
		showOutgoing=showIncoming= showDatatype=true;
		showChildren = VisualizationOptions.s3_ShowChildern;
		
		this.createScene();
		this.addBehaviors();
		this.repaintCanvas();
	}
	
	private void loadInheritedOutgoingRelations(ConceptData data){
		for (String relURI : data.getOutgoingLinks().keySet()) {
//			Following if blocks match patterns where the relations has not to be inherited and add to relations to skip
			HashSet<RelationData> relations2Inherit =this.qualifyRelationsAsInherited( data.getOutgoingLinks().get(relURI), relURI, true);
			HashSet<RelationData> relations2Skip=new HashSet<RelationData>(relations2Inherit.size());
//			check if there is an universal quantifier if not already set
			if(!this.universalQuantifiers.containsKey(relURI)){}
			
//			If exists a restriction on the current concept or on an already inherited relation 
			if(this.currentConceptData.getOutgoingLinks().containsKey(relURI)  || 	this.inheritedOutgoingRelations.containsKey(relURI)){			
				this.skipRelationsAlreadyWithRestrictions(relURI, relations2Inherit, relations2Skip);
			}
			
//Remove relations to skip from the one to inherit
			relations2Inherit.removeAll(relations2Skip);
//Add relation data to Inherited Set
			if (this.inheritedOutgoingRelations.containsKey(relURI)) {
				HashSet<RelationData> relSet = this.inheritedOutgoingRelations.get(relURI);
				relSet.addAll(relations2Inherit);
			} else{
				HashSet<RelationData> relSet=new HashSet<RelationData>();
				relSet.addAll(relations2Inherit);
				this.inheritedOutgoingRelations.put(relURI, relSet);
			}
		}
	}
	
	private void skipUnnecessaryUniversalQuantifiers(String relURI, HashSet<RelationData> relations2Inherit, 
			HashSet<RelationData> relations2Skip){

		for(RelationData rd:relations2Inherit){
			if(rd.isUniversal()){//adding it
				this.universalQuantifiers.put(relURI, rd);
//			 if matching quantifiers restrictions exist doesen't add to relations to be displayed..
//				Get all the relationData except the universal quantifier
				HashSet<String> notUniversal=new HashSet<String>(relations2Inherit.size());
				for(RelationData myRel:relations2Inherit){
					if(!myRel.isUniversal())
						notUniversal.add(myRel.getRelatedNode().toString());
				}
				if(this.currentConceptData.getOutgoingLinks().containsKey(relURI)){
					for(RelationData myRel:this.currentConceptData.getOutgoingLinks().get(relURI)){
						if(!myRel.isUniversal())
							notUniversal.add(myRel.getRelatedNode().toString());
					}
				}
				//Remove if not needed to be displayed
				NodeTypeAbstraction univRange=rd.getRelatedNode();
				if(univRange.getType() == NodeTypesEnum.Union){
					HashSet<String> logicSet=new HashSet<String>();
					for(NodeTypeAbstraction n:univRange.getLogicSet())
						logicSet.add(n.toString());

					if(notUniversal.containsAll(logicSet)){
						relations2Skip.add(rd);
					}
				}
				else if(univRange.getType() == NodeTypesEnum.NamedClass &&
						(notUniversal.contains(univRange)) ){
					relations2Skip.add(rd);
				}
				
				break;//universal found stop itarating
			}
		}
			
	
	}
	
	private void skipRelationsAlreadyWithRestrictions(String relURI, HashSet<RelationData> relations2Inherit, 
			HashSet<RelationData> relations2Skip){
		HashSet<String> relatedConcepts=new HashSet<String>();
		if(this.currentConceptData.getOutgoingLinks().containsKey(relURI) )
			relatedConcepts.addAll( RelationData.extractRelatedNodes(this.currentConceptData.getOutgoingLinks().get(relURI))  );
		if(this.inheritedOutgoingRelations.containsKey(relURI))
			relatedConcepts.addAll( RelationData.extractRelatedNodes(this.inheritedOutgoingRelations.get(relURI))   );
		
		for(RelationData myRelationData: relations2Inherit){
//			skip inherited simple RelationData  when with the SAME related node 
			if(myRelationData.isSimple() && relatedConcepts.contains(myRelationData.getRelatedNode().toString()) ){
				relations2Skip.add(myRelationData);
			}
//			Checks the coerence of related node in the eventual presence of an universal quantifier
			else if(!myRelationData.isSimple() && this.universalQuantifiers.containsKey(relURI)){
				RelationData universal=this.universalQuantifiers.get(relURI);
				//if related node is not in the Universal Restriction Set switch it
				NodeTypeAbstraction univRange=universal.getRelatedNode();
				if(univRange.getType() == NodeTypesEnum.Union){
					HashSet<String> logicSet=new HashSet<String>();
					for(NodeTypeAbstraction n:univRange.getLogicSet())
						logicSet.add(n.toString());
					
					if(!logicSet.contains(myRelationData.getRelatedNode().toString())){
						myRelationData.setRelatedNode(univRange);
					}
				}
				else if(univRange.getType() == NodeTypesEnum.NamedClass &&
						(univRange.equals(myRelationData.getRelatedNode())) ){
					myRelationData.setRelatedNode(univRange);
				}
			}
		}
	
	}
	
	private void loadInheritedIncomingRelations(ConceptData data){
		for (String relURI : data.getIncomingLinks().keySet()) {
			HashSet<RelationData> relations2Inherit = this.qualifyRelationsAsInherited( data.getIncomingLinks().get(relURI), null, false);
			HashSet<RelationData> relations2Skip=new HashSet<RelationData>(relations2Inherit.size());
			
//			If exists a restriction on the current concept or on an already inherited relation 
			if(this.currentConceptData.getIncomingLinks().containsKey(relURI)  || this.inheritedIncomingRelations.containsKey(relURI)){
				HashSet<String> relatedConcepts=new HashSet<String>();
				if(this.currentConceptData.getIncomingLinks().containsKey(relURI) )
					relatedConcepts.addAll( RelationData.extractRelatedNodes(this.currentConceptData.getIncomingLinks().get(relURI))  );
				if(this.inheritedIncomingRelations.containsKey(relURI))
					relatedConcepts.addAll( RelationData.extractRelatedNodes(this.inheritedIncomingRelations.get(relURI))   );
				
				for(RelationData myRelationData: relations2Inherit){
//					skip inherited simple RelationData  when with the SAME related node 
					if(myRelationData.isSimple() && relatedConcepts.contains(myRelationData.getRelatedNode().toString()) ){
						relations2Skip.add(myRelationData);
					}
				}
			}

//			Remove relations to skip from the one to inherit
			relations2Inherit.removeAll(relations2Skip);
//			specifying such relations as inherited

//			Add relation data to Inherited Set
			if (this.inheritedIncomingRelations.containsKey(relURI)) {
				HashSet<RelationData> relSet = this.inheritedIncomingRelations.get(relURI);
				relSet.addAll(relations2Inherit);
			} else{
				HashSet<RelationData> relSet=new HashSet<RelationData>();
				relSet.addAll(relations2Inherit);
				this.inheritedIncomingRelations.put(relURI, relSet);
			}
		}
	}
	
	private void loadInheritedDatatypeRelations(ConceptData data){
		for (String relURI : data.getDatatypeLinks().keySet()) {
			HashSet<RelationData> relations2Inherit = this.qualifyRelationsAsInherited( data.getDatatypeLinks().get(relURI), null, false);

			if (this.inheritedDataTypeRelations.containsKey(relURI)) {
				HashSet<RelationData> relSet = this.inheritedDataTypeRelations.get(relURI);
				relSet.addAll(relations2Inherit);
			} else{
				HashSet<RelationData> relSet=new HashSet<RelationData>();
				relSet.addAll(relations2Inherit);
				this.inheritedDataTypeRelations.put(relURI, relSet);
			}
		}
	}
	
	/**
	 * Recursive procedure that retrieves ancestors relations
	 * @param uri the uri of the ancestor to inherit from 
	 * @param directParent the direct parent of the concept, used for discriminating among eventual multiple inheritances
	 */
	private void loadInheritedRelations(String uri, String directParent) {
		try{	
		
		ConceptData data = this.refMainPanel.model.getConceptData(uri);
		// recupero le objProperties uscenti
		this.loadInheritedOutgoingRelations(data);

		// recupero le objProperties entranti
		this.loadInheritedIncomingRelations(data);

		// recupero le dataTypeProperties (solo uscenti, entranti non ha senso!)
		this.loadInheritedDatatypeRelations(data);

		// recupero SelfLoops
		if (data.getSelfLoopLinks().size() > 0) {
			this.inheritedSelfLoops.addAll(data.getSelfLoopLinks());
		}

		if (data.getParent() == null) {//top level concept
			if (!uri.equals(directParent)) {//
				HashSet<String> set;
				if (this.AncestorRoots.containsKey(directParent)) {
					set = this.AncestorRoots.get(directParent);
					set.add(uri);
				} else {
					set = new HashSet<String>();
					set.add(uri);
					this.AncestorRoots.put(directParent, set);
				}
			}
			return;
		} else {
			// propago ricerca a tutti i padri
			for (String parentURI : data.getAllParents()) {
				this.loadInheritedRelations(parentURI, directParent );
			}
		}
		
		}
		catch(Exception e){
			System.err.println("ERROR in loading inherited relations for concept \""+this.currentConceptURI+
					"\" could not retrieve ancestor \""+uri+"\"");
		}
	}
	
	private HashSet<RelationData> qualifyRelationsAsInherited (HashSet<RelationData> relations2Inherit, 
			String relURI, boolean skipUniversal){
		HashSet<RelationData> inheritedSet = new HashSet<RelationData>();
//		specifying such relations as inherited
		for(RelationData r: relations2Inherit){
			RelationData inherited=new RelationData(r.getRelationName(), r.getDir(), r.getRelationType());
			if(skipUniversal && this.universalQuantifiers.containsKey(relURI) && (r.isUniversal() || r.isSimple()) )
				continue;//Skip element
			inherited.setUniversal(r.isUniversal());
			inherited.setHasValue(r.isHasValue());
			inherited.setFixedCardinality(r.getFixedCardinality());
			inherited.setMinCardinality(r.getMinCardinality());
			inherited.setMaxCardinality(r.getMaxCardinality());
			inherited.setInherited(true);
			inherited.setRelatedNode(r.getRelatedNode());
			inheritedSet.add(inherited);
		}
		return inheritedSet;
	}

	private void drawChildren(TransformGroup tgx) {
		// Posiziono e Disegno i figli
		float radius = this.getSubRadius(this.currentConceptData.getNumberOfChildren());
		float alpha = (float) (2f * Math.PI / this.currentConceptData.getNumberOfChildren());
		float alpha0 = (float) (Math.PI * Math.random());
		int k = 0;
		for (String childURI : this.currentConceptData.getChildren()) {
			// Positioning child node
			float x = (float) (radius * Math.cos(alpha0 + k * alpha));
			float z = (float) (radius * Math.sin(alpha0 + k * alpha));
			Vector3f position = new Vector3f(x,
					-VisualizationOptions.s3_TreeLevelHeight, z);
			// set the position for the children
			ConceptData cdata = this.refMainPanel.model
					.getConceptData(childURI);

			k++;
			// Translates from orig
			TransformGroup childTgx = this.createTransformGroup();
			Transform3D transf = new Transform3D();
			transf.setTranslation(position);
			childTgx.setTransform(transf);
			// Drawing child sphere & the isA relation
			ConceptSphere childSphere = new ConceptSphere(
					VisualizationOptions.s3_ChildSphereColor, childURI, true,
					cdata.hasInstances());
			childSphere
					.setRadius(VisualizationOptions.s3_ChildSphereRadius);
			childSphere
					.setLabelColor(VisualizationOptions.s3_LabelChildSphereColor);
			childSphere
					.setLabelScale(VisualizationOptions.s3_LabelChildSphereScale);
			childTgx.addChild(childSphere.createColoredSphere());
			// disegno con tgx centrato in sferetta per cui inverto il vettore
			// di posizione
			position.scale(-1);
			Relation3DObject isA = new Relation3DObject();
			isA.setTransparency(VisualizationOptions.s3_RelationTransparency);
			childTgx.addChild(isA.createISARelation(position));
			// adds it to the scene bg
			tgx.addChild(childTgx);
		}
	}

	private void drawParents(TransformGroup tgx, ConceptData data) {
		// Posiziono i padri
		int extra=0;
		if(this.currentConceptData.hasAnonymousParent())
			extra=1;
		float radius = this.getSubRadius(data.getNumberOfParents() + extra);

		float alpha = (float) (2f * Math.PI /  (data.getNumberOfParents() + extra) );
		float alpha0 = (float) (Math.PI * Math.random());
		int k = 0;
		for (String parentURI : data.getAllParents()) {
			// Positioning parent node
			float x = (float) (radius * Math.cos(alpha0 + k * alpha));
			float z = (float) (radius * Math.sin(alpha0 + k * alpha));
			Vector3f position = new Vector3f(x,
					VisualizationOptions.s3_SphereRadius+VisualizationOptions.s3_TreeLevelHeight, z);
			k++;
			// Translates from orig
			TransformGroup parentTgx = new TransformGroup();
			Transform3D transf = new Transform3D();
			transf.setTranslation(position);
			parentTgx.setTransform(transf);
			// Drawing parent sphere & the isA relation
			ConceptSphere parentSphere = new ConceptSphere(
					VisualizationOptions.s3_ParentSphereColor, parentURI, true,	false);
			parentSphere.setRadius(VisualizationOptions.s3_ParentSphereRadius);
			parentSphere.setLabelColor(VisualizationOptions.s3_LabelParentSphereColor);
			parentSphere.setLabelScale(VisualizationOptions.s3_LabelParentSphereScale);
			parentTgx.addChild(parentSphere.createColoredSphere());
			// adds the relation fron current concept to its parent
			Relation3DObject isA = new Relation3DObject();
			isA.setTransparency(VisualizationOptions.s3_RelationTransparency);
			tgx.addChild(isA.createISARelation(position));
			// adds the root..
			HashSet<String> set = this.AncestorRoots.get(parentURI);
			if (set != null)
				this.drawRoots(parentTgx, set);

			// adds it to the scene bg
			tgx.addChild(parentTgx);
		}
		if(extra==1){
//		 Positioning anonymous parent node
			float x = (float) (radius * Math.cos(alpha0 + k * alpha));
			float z = (float) (radius * Math.sin(alpha0 + k * alpha));
			Vector3f position = new Vector3f(x, VisualizationOptions.s3_SphereRadius+VisualizationOptions.s3_TreeLevelHeight, z);
			Relation3DObject logicalParent=new Relation3DObject();
			logicalParent.setTransparency(VisualizationOptions.s3_RelationTransparency);
			logicalParent.setRelationType(GraphicalLinkTypeEnum.isA);

			tgx.addChild(logicalParent.createRelationWithRange(null, this.currentConceptData.getAnonymousParent(), 
					position, LinkDirectionEnum.out));
		}
	}

	private void drawRoots(TransformGroup tgx, HashSet<String> set) {
		// Posiziono i Root
		float radius = this.getSubRadius(set.size());
		float alpha = (float) (2f * Math.PI / set.size());
		float alpha0 = (float) (Math.PI * Math.random());
		int k = 0;
		for (String root : set) {
			// Positioning child node
			float x = (float) (radius * Math.cos(alpha0 + k * alpha));
			float z = (float) (radius * Math.sin(alpha0 + k * alpha));
			Vector3f position = new Vector3f(x,
					VisualizationOptions.s3_TreeLevelHeight, z);
			// set the position for the roots

			k++;
			// Translates from orig
			TransformGroup rootTgx = new TransformGroup();
			Transform3D transf = new Transform3D();
			transf.setTranslation(position);
			rootTgx.setTransform(transf);
			// Drawing child sphere & the isA relation
			ConceptSphere parentSphere = new ConceptSphere(
					VisualizationOptions.s3_RootSphereColor, root, true, false);
			parentSphere.setRadius(VisualizationOptions.s3_ParentSphereRadius);
			parentSphere.setTransparency(VisualizationOptions.s3_RelationTransparency);
			parentSphere.setLabelColor(VisualizationOptions.s3_LabelParentSphereColor);
			parentSphere.setLabelScale(VisualizationOptions.s3_LabelParentSphereScale);
			rootTgx.addChild(parentSphere.createColoredSphere());
			// adds it to the scene bg
			tgx.addChild(rootTgx);
			// adds the relation fron current concept to its parent
			Relation3DObject isA = new Relation3DObject();
			isA.setTransparency(.6f);
			tgx.addChild(isA.createISARelation(position));
		}
	}
	
	private HashSet<RelationData> selectRelationsToDraw(Hashtable<String, HashSet<RelationData>>links,
			Hashtable<String, HashSet<RelationData>> inherited){
		HashSet<RelationData> relationsToDraw=new HashSet<RelationData>();
		
		if(this.showDirectRelations){
		//Analyze each relations and retrieves the relation data to be drawn
			for(String relURI:links.keySet()){
				HashSet<RelationData> relData=new HashSet<RelationData>( links.get(relURI) );		
				
				//Check if the universal quantifier has to be drawn
				if(universalQuantifiers.containsKey(relURI) && relData.size()>1){
					//Get univeral quantifier reldata
					RelationData universal=universalQuantifiers.get(relURI);
					NodeTypeAbstraction univRange=universal.getRelatedNode();
					//Get all the relationData except the universal quantifier
					HashSet<String> notUniversal=new HashSet<String>(relData.size());
					for(RelationData myRel:relData){
						if(!myRel.isUniversal())
							notUniversal.add(myRel.getRelatedNode().toString());
					}
	
	//				Filters out the universal restriction if matching quantifiers restrictions exist
					if(univRange.getType() == NodeTypesEnum.Union){
						HashSet<String> logicSet=new HashSet<String>();
						for(NodeTypeAbstraction n:univRange.getLogicSet())
							logicSet.add(n.toString());
	
						if(notUniversal.containsAll(logicSet)){
							relData.remove(universal);
						}
					}
					else if(univRange.getType() == NodeTypesEnum.NamedClass &&
							(notUniversal.contains(univRange)) ){
						relData.remove(universal);
					}
				}
				
				//Check if in the same set co-exist a cardinality restriction and a simple one and remove the simple in that case
				HashSet<RelationData> clonedSet = (HashSet<RelationData>) relData.clone();
				for(Iterator<RelationData> it = relData.iterator(); it.hasNext();){
					RelationData myRel = it.next();
					if(myRel.isSimple()){
						for(RelationData comparison: clonedSet){
							if(!comparison.isSimple()){
								it.remove();
								break;
							}
						}
					}
				}
				relationsToDraw.addAll(relData);
			}
		}
	
		if(this.showInheritedRelations){
			for(String relURI:inherited.keySet()){
				HashSet<RelationData> relData=inherited.get(relURI);
				relationsToDraw.addAll(relData);
//				Check if in the same set co-exist a cardinality restriction and a simple one and remove the simple in that case
				HashSet<RelationData> clonedSet = (HashSet<RelationData>) relationsToDraw.clone();
				for(Iterator<RelationData> it = relationsToDraw.iterator(); it.hasNext();){
					RelationData myRel = it.next();
					if(myRel.isSimple()){
						for(RelationData comparison: clonedSet){
							if(!comparison.isSimple() && myRel.getRelatedNode().toString().equals(comparison.getRelatedNode().toString())){
								it.remove();
								break;
							}
						}
					}
				}
			}
		}
		return relationsToDraw;
	}
	
	private Hashtable<String, RelationData> getUniversalQuantifiers(Hashtable<String, HashSet<RelationData>> relations){
		Hashtable<String, RelationData> result=new Hashtable<String, RelationData>(relations.size());
//		Retrieving universal Quantifier restriction (ALL FROM ..)
		for(String relURI:relations.keySet()){	
			RelationData universal=null;
			for(RelationData rdata: relations.get(relURI)){
				if(rdata.isUniversal())
					universal=rdata;
			}
			if(universal!=null){
				result.put(relURI, universal);
			}
		}
		return result;
	}
	
	private void drawSemanticRelations(TransformGroup tgx) {
		
		if(this.showOutgoing){
			HashSet<RelationData> relations=this.selectRelationsToDraw(this.currentConceptData.getOutgoingLinks(), this.inheritedOutgoingRelations);
			this.drawRelations(tgx, relations, LinkDirectionEnum.out, 0f);
		}
		if(this.showIncoming){
			HashSet<RelationData> relations=this.selectRelationsToDraw(this.currentConceptData.getIncomingLinks(), this.inheritedIncomingRelations);
			this.drawRelations(tgx, relations, LinkDirectionEnum.in, VisualizationOptions.s3_SemRelationHeigth);
		}
		if(this.showDatatype){
			HashSet<RelationData> relations=this.selectRelationsToDraw(this.currentConceptData.getDatatypeLinks(), this.inheritedDataTypeRelations);
			this.drawRelations(tgx, relations, LinkDirectionEnum.out, -VisualizationOptions.s3_SemRelationHeigth);
		}
		this.drawSelfLoopRelation(tgx);
	}
	
	private void drawRelations(TransformGroup tgx, HashSet<RelationData> relations, 
			LinkDirectionEnum direction, float yValue){
		//Calculating the distance  taking into acount the cone (in semantic relation is placed in the middle) to compute radius value
		float distance = ((float)(relations.size() * VisualizationOptions.sR_sphereRadius * 
				VisualizationOptions.s3_SemanticRelationDistanceMultiplier)/ (float)Math.PI );
		if (distance < (2*VisualizationOptions.s3_TreeLevelHeight))
			distance += VisualizationOptions.s3_TreeLevelHeight;
		else if (distance > (4*VisualizationOptions.s3_TreeLevelHeight))
			distance -= VisualizationOptions.s3_TreeLevelHeight;
		distance+=VisualizationOptions.s3_SphereRadius*1.2f;
		
		// Draw relations
		int k=0;
		double angularStep= 2*Math.PI/relations.size();
		double angularOffset=(2 * Math.PI * Math.random());
		for (RelationData rel : relations) {
			Relation3DObject relation = new Relation3DObject();
			// Positioning related node
			float length=distance;
			if(rel.isInherited()){
				length*=VisualizationOptions.sR_InheritedRelationDistanceMultiplier;
				relation.setTransparency(VisualizationOptions.s3_InheritedElmTransparency);
			}
			Vector3f position = new Vector3f((float) (length * Math.cos((k * angularStep)+angularOffset)),
					yValue, (float) (length * Math.sin((k * angularStep)+angularOffset)));

			// Drawing range & the relation

			BranchGroup bgRel = relation.createRelationWithRange(rel, position, direction);
			// adds it to the scene bg
			tgx.addChild(bgRel);
			
			k++;
		}
	}
	
	private void drawSelfLoopRelation(TransformGroup tgx) {
		// Draws self loops direct & inherited
		if (this.currentConceptData.getSelfLoopLinks().size() > 0 && this.showDirectRelations) {
			Relation3DObject relation = new Relation3DObject();
			AxisAngle4f rotAxis =new AxisAngle4f(new Vector3f(0f,1f,0f),0);
			tgx.addChild(relation.createSelfLoopRelation(this.currentConceptData.getSelfLoopLinks(),
					this.currentConceptData.getConceptSphereRadius(), rotAxis));
		}

		if (this.inheritedSelfLoops.size() > 0 && this.showInheritedRelations) {
			Relation3DObject relation = new Relation3DObject();
			relation.setTransparency(VisualizationOptions.s3_InheritedElmTransparency);

			float outRadius;
			if (this.showDirectRelations)
				outRadius = this.currentConceptData.getConceptSphereRadius()
						* VisualizationOptions.s3_InheritedSelfLoopMultiplier;
			else
				outRadius = this.currentConceptData.getConceptSphereRadius();
			
			AxisAngle4f rotAxis =new AxisAngle4f(new Vector3f(0f,1f,0f),(float)Math.PI);
			tgx.addChild(relation.createSelfLoopRelation(
					this.inheritedSelfLoops, outRadius, rotAxis));
		}
	}
	
	private void drawDisjointsWith(TransformGroup tg){
		if(this.currentConceptData.getDifferentFrom().size()>0){
			Vector3f position = new Vector3f(0f, 	-VisualizationOptions.s3_TreeLevelHeight*1.2f, 0f);
			Relation3DObject rel=new Relation3DObject();
			rel.setRelationType(GraphicalLinkTypeEnum.isA);
			tg.addChild(rel.createRelationWithSetOfElements(null,"DISJOINT WITH", this.currentConceptData.getDifferentFrom(), 
					position, LinkDirectionEnum.in, true));
		}
	}
	
	public void createScene() {

		this.addSceneLabel(this.currentConceptURI);
		// Creates the scene groups
		BranchGroup bgx = this.createBranchGroup();
		TransformGroup tgx = this.createTransformGroup();
		// loads concept data
		this.currentConceptData = this.refMainPanel.model
				.getConceptData(this.currentConceptURI);
		// draws the main concept
		ConceptSphere currentConcept = new ConceptSphere(new Color3f(
				Color.DARK_GRAY), this.currentConceptURI, true, this.currentConceptData.hasInstances());
		currentConcept.setRadius(VisualizationOptions.s3_SphereRadius);
		currentConcept.setWithLabel(false);
		this.currentConceptData.setConceptSphereRadius(VisualizationOptions.s3_SphereRadius);
		BranchGroup centralSphere = currentConcept.createColoredSphere();
		tgx.addChild(centralSphere);
		
//		Get universal quantifiers
		this.universalQuantifiers=this.getUniversalQuantifiers(this.currentConceptData.getOutgoingLinks());
				
// retrieves data & draws closely related concept
		for (String parentURI : this.currentConceptData.getAllParents()) {
			// retrieves the inherited relation of the concept
			this.loadInheritedRelations(parentURI, parentURI);
		}

		// Draws related concepts
		if (this.showChildren)
			this.drawChildren(tgx);
		else
			this.drawDisjointsWith(tgx);

		this.drawParents(tgx, this.currentConceptData);
		this.drawSemanticRelations(tgx);

		bgx.addChild(tgx);
		this.tgSCENE.addChild(bgx);
	}

	private void reInitializeScene() {
		// reinitialize the scene
		this.tgSCENE.removeAllChildren();
		this.inheritedIncomingRelations.clear();
		this.inheritedOutgoingRelations.clear();
		this.inheritedDataTypeRelations.clear();
		this.inheritedSelfLoops.clear();
		this.AncestorRoots.clear();
		this.universalQuantifiers.clear();
		createScene();
	}

	private float getSubRadius(int numberOfChildren) {
		float radius = 0;
		if (numberOfChildren > 1 && numberOfChildren < 5) {
			radius = 1.5f * VisualizationOptions.s3_SphereRadius;
		} else if (numberOfChildren > 4) {
			radius = (float) (VisualizationOptions.s3_Segment2ArcCorrectionFactor
					* numberOfChildren
					* (VisualizationOptions.s3_ChildSphereRadius * VisualizationOptions.s3_SphereRadius +
							VisualizationOptions.sG_distance_min) / (Math.PI));
		}
		return radius;
	}

	@Override
	public boolean showContextMenu(final PopupMenu menu, final String uri) {		
		ConceptData cd = this.refMainPanel.model.getConceptData(uri);
		
		if(cd == null){
			this.showSelectionPopUp(uri);
			return false;
		}
		
		if (this.refMainPanel.model.getConceptData(uri).getNumberOfChildren() > 0){
			MenuItem item = new MenuItem("Explore Subtree");
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
				Scene3D myScene = new SceneFocusTree2(refMainPanel, uri);
				refMainPanel.changeScene(myScene);	
				removePopUpMenu(menu);
			}});
			menu.add(item);
		}

		
		if (cd.hasInstances()) {
			MenuItem item0 = new MenuItem("Show Direct Instances");
			item0.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SceneFocusInstances4 myScene=new SceneFocusInstances4(refMainPanel,uri);
					refMainPanel.changeScene(myScene);
					removePopUpMenu(menu);
				}
			});
			menu.add(item0);
		}
		
		if(cd.hasNonDirectInstances()){
			MenuItem item1 = new MenuItem("Show All Instances");
			item1.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Set<String> set = refMainPanel.model.retrieveAllInstancesOf(uri);
					SceneFocusInstances4 myScene=new SceneFocusInstances4(refMainPanel,set,uri);
					refMainPanel.changeScene(myScene);
					removePopUpMenu(menu);
				}
			});
			menu.add(item1);
		}
		
		if (showChildren) {
			MenuItem item2 = new MenuItem("Hide Children");
			item2.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showChildren = false;
					reInitializeScene();
					removePopUpMenu(menu);
				}
			});
			menu.add(item2);
		} else {
			MenuItem item3 = new MenuItem("Show Children");
			item3.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showChildren = true;
					reInitializeScene();
					removePopUpMenu(menu);
				}
			});
			menu.add(item3);
		}

		if (showDirectRelations) {
			MenuItem item4 = new MenuItem("Hide Direct Relations");
			item4.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showDirectRelations = false;
					reInitializeScene();
					removePopUpMenu(menu);
				}
			});
			menu.add(item4);
		} else {
			MenuItem item5 = new MenuItem("Show Direct Relations");
			item5.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showDirectRelations = true;
					reInitializeScene();
					removePopUpMenu(menu);
				}
			});
			menu.add(item5);
		}

		if (showInheritedRelations) {
			MenuItem item6 = new MenuItem("Hide Inherited Relations");
			item6.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showInheritedRelations = false;
					reInitializeScene();
					removePopUpMenu(menu);
				}
			});
			menu.add(item6);
		} else {
			MenuItem item7 = new MenuItem("Show Inherited Relations");
			item7.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showInheritedRelations = true;
					reInitializeScene();
					removePopUpMenu(menu);
				}
			});
			menu.add(item7);
		}
		
		MenuItem item8 = new MenuItem("Show onlyOutGoing");
		item8.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showOutgoing = true;
				showIncoming = false;
				showDatatype = false;
				reInitializeScene();
				removePopUpMenu(menu);
			}
		});
		menu.add(item8);
		
		MenuItem item9 = new MenuItem("Show only Incoming");
		item9.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showOutgoing = false;
				showIncoming = true;
				showDatatype = false;
				reInitializeScene();
				removePopUpMenu(menu);
			}
		});
		menu.add(item9);
		
		MenuItem item10 = new MenuItem("Show only Datatype");
		item10.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showOutgoing = false;
				showIncoming = false;
				showDatatype = true;
				reInitializeScene();
				removePopUpMenu(menu);
			}
		});
		menu.add(item10);
		
		MenuItem item11 = new MenuItem("Show All");
		item11.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showOutgoing = true;
				showIncoming = true;
				showDatatype = true;
				reInitializeScene();
				removePopUpMenu(menu);
			}
		});
		menu.add(item11);
		
		return true;
	}

	@Override
	public void selectElement(String uri) {
		if (uri.equals(this.currentConceptURI)){
			SceneFocusRoots1 myScene=new SceneFocusRoots1(this.refMainPanel, 
					this.refMainPanel.conceptsOfCurrentLogicView);
			if (VisualizationOptions.sG_useAnimations) {
				this.fadeOutAnimation(uri, myScene);
			} else {
				this.refMainPanel.changeScene(myScene);
			}
		}
		else if(this.refMainPanel.model.getConceptData(uri) == null){//Logic Expression..
			InstanceData idata=this.refMainPanel.model.getInstanceData(uri);
			if(idata != null){
				SceneFocusFacts5 myScene=new SceneFocusFacts5(this.refMainPanel, 
						idata.getPertainingConceptURI(),uri);
				if (VisualizationOptions.sG_useAnimations) {
					this.fadeOutAnimation(uri, myScene);
				} else {
					this.refMainPanel.changeScene(myScene);
				}
			}
			this.showSelectionPopUp(uri);
		}
		else {
			// reinitialize the scene with a new current concept
			this.currentConceptURI = uri;
			this.reInitializeScene();
		}
	}

	@Override
	public void expand_OR_collapseElement(String uri) {	}

	@Override
	public boolean showRelationContextMenu(final PopupMenu menu,	HashSet<RelationData> relSet) {	
		
		PopupMenu setColor=new PopupMenu("Set Color To");
		menu.add(setColor);
		boolean found=false;
		for (RelationData reldata : relSet) {
			final String relname=reldata.getRelationName();
			final LinkDirectionEnum dir=reldata.getDir();
			MenuItem item = new MenuItem(relname);
			item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Color c = JColorChooser.showDialog(null, "Choose a color...", Color.WHITE);
					refMainPanel.model.setColorToRelation(relname, c);
					removePopUpMenu(menu);
				}
			});
			setColor.add(item);
			if(Relation3DObject.getRelationColor(relname)!=null)
				found=true;
		}
		
		if(found){
			PopupMenu unsetColor=new PopupMenu("Remove Custom Color");
			menu.add(unsetColor);
			for (RelationData reldata : relSet) {
				final String relname=reldata.getRelationName();
				if(Relation3DObject.getRelationColor(relname)!=null){
					final LinkDirectionEnum dir=reldata.getDir();
					MenuItem item = new MenuItem(relname);
					item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
					item.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							refMainPanel.model.removeColorToRelation(relname);
							removePopUpMenu(menu);
						}
					});
					unsetColor.add(item);
				}
			}
		}
		return true;
	}

	@Override
	public void fadeInAnimation() {
//		 create the animations
		BranchGroup bgAnimation = createBranchGroup();
		long time = System.currentTimeMillis();

		// timer for scale interpolator (Increasing)
		Alpha alpha = new Alpha(1, VisualizationOptions.sG_AnimationDuration);
		alpha.setStartTime(time-20);
		alpha.setIncreasingAlphaRampDuration(alpha.getIncreasingAlphaDuration() / 4);

		ScaleInterpolator si = new ScaleInterpolator(alpha, tgSCENE);
		si.setSchedulingBounds(new BoundingSphere());
		bgAnimation.addChild(si);
		
		//Adding element to the scene
		tgSCENE.addChild(bgAnimation);
	}


	@Override
	public void fadeOutAnimation(String uri, Scene3D nextScene) {
//		 create the animations
		BranchGroup bgAnimation = createBranchGroup();
		long time = System.currentTimeMillis();
//		 timer for scale interpolator (decreasing)
		Alpha beta = new Alpha(1, 0);
		beta.setDecreasingAlphaDuration(VisualizationOptions.sG_AnimationDuration / 4);
		beta.setDecreasingAlphaRampDuration(beta
						.getDecreasingAlphaDuration() / 3);
		beta.setStartTime(time);
		beta.setMode(Alpha.DECREASING_ENABLE);
		
		PostAnimation_SceneChanger change = new PostAnimation_SceneChanger(
				this.refMainPanel, nextScene, false);
		
		ScaleInterpolatorWithListener si = new ScaleInterpolatorWithListener(beta, tgSCENE,change);
		si.setSchedulingBounds(new BoundingSphere());
		bgAnimation.addChild(si);
		
		//Adding element to the scene
		tgSCENE.addChild(bgAnimation);
	}


}
