package it.polito.elite.visualization.ontoviewer.scenes3D;

import it.polito.elite.ontology.NodeTypeAbstraction;
import it.polito.elite.visualization.ontoviewer.enums.GraphicalLinkTypeEnum;
import it.polito.elite.visualization.ontoviewer.enums.LinkDirectionEnum;
import it.polito.elite.visualization.ontoviewer.models.InstanceData;
import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.models.RelationType;
import it.polito.elite.visualization.ontoviewer.tools.ConceptSphere;
import it.polito.elite.visualization.ontoviewer.tools.InstanceCube;
import it.polito.elite.visualization.ontoviewer.tools.Relation3DObject;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;
import it.polito.elite.visualization.ontoviewer.userInterfaces.VisualizationOptions;

import java.awt.Color;
import java.awt.Font;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Hashtable;

import javax.media.j3d.Alpha;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ScaleInterpolator;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.swing.JColorChooser;
import javax.vecmath.AxisAngle4f;
import javax.vecmath.Color3f;
import javax.vecmath.Vector3f;

public class SceneFocusFacts5 extends Scene3D {
	
	private InstanceData currentInstanceData;
	private boolean showOutgoing, showIncoming, showDatatype;
	
	public SceneFocusFacts5(Pane3DScene _mf, String uriConcept, String uriInstance) {
		super(_mf,uriConcept,uriInstance,5);
		showOutgoing=showIncoming= showDatatype=true;
		this.currentInstanceData = this.refMainPanel.model.getInstanceData(this.currentInstanceURI);
		this.createScene();
		this.addBehaviors();
		this.repaintCanvas();		
	}
	
	private void drawFacts(TransformGroup tgx){
		if(this.showOutgoing){
			this.drawLink(tgx, this.currentInstanceData.getOutgoingLinks() , LinkDirectionEnum.out, 0f);
		}
		if(this.showIncoming){
			this.drawLink(tgx, this.currentInstanceData.getIncomingLinks() , LinkDirectionEnum.in, VisualizationOptions.s5_SemRelationHeigth);		
		}
		if(this.showDatatype){
			this.drawLink(tgx, this.currentInstanceData.getDatatypeLinks() , LinkDirectionEnum.out, -VisualizationOptions.s3_SemRelationHeigth);	
		}
		this.drawSelfLoopRelation(tgx);
	}
	
	private void drawSameDifferent(TransformGroup tg){
		float x=0;
		if(	 this.currentInstanceData.getSameAs()!=null && 
				this.currentInstanceData.getSameAs().size()>0 &&
				this.currentInstanceData.getDifferentFrom().size()>0 )
			x = VisualizationOptions.s5_cubeSide*1.2f;
		
		if( this.currentInstanceData.getDifferentFrom()!=null &&
				this.currentInstanceData.getDifferentFrom().size()>0){
			Vector3f position = new Vector3f(x, 	-VisualizationOptions.s3_TreeLevelHeight*1.2f, 0f);
			Relation3DObject rel=new Relation3DObject();
			rel.setRelationType(GraphicalLinkTypeEnum.isA);
			tg.addChild(rel.createRelationWithSetOfElements(null,"DIFFERENT FROM", this.currentInstanceData.getDifferentFrom(), 
					position, LinkDirectionEnum.in, true));
		}
		if(this.currentInstanceData.getSameAs()!=null && this.currentInstanceData.getSameAs().size()>0){
			Vector3f position = new Vector3f(x, 	-VisualizationOptions.s3_TreeLevelHeight*1.2f, 0f);
			Relation3DObject rel=new Relation3DObject();
			rel.setRelationType(GraphicalLinkTypeEnum.isA);
			tg.addChild(rel.createRelationWithSetOfElements(null,"SAME AS", this.currentInstanceData.getSameAs(), 
					position, LinkDirectionEnum.in, true));
		}
	}
	
	private void drawLink(TransformGroup tgx, Hashtable<String, HashSet<NodeTypeAbstraction>> relations, 
			LinkDirectionEnum direction,  float yValue){
		int size=relations.size();
//		Calculating the distance  taking into acount the cone (in semantic relation is placed in the middle) to compute radius value
		float distance = VisualizationOptions.s5_cubeSide * 5;
		if(size>=5)
			distance = ((float)(relations.size() * VisualizationOptions.sR_sphereRadius * 
					VisualizationOptions.s3_SemanticRelationDistanceMultiplier)/ (float)Math.PI );
		distance+=VisualizationOptions.s5_cubeSide*1.2f;
		
		// Draw relations
		int k=0;
		double angularStep= 2*Math.PI/relations.size();
		double angularOffset=(2 * Math.PI * Math.random());
		for (String relURI : relations.keySet()) {
			HashSet<NodeTypeAbstraction> relatedInstances=relations.get(relURI);
			Relation3DObject relation = new Relation3DObject();
			// Positioning related node

			Vector3f position = new Vector3f((float) (distance * Math.cos((k * angularStep)+angularOffset)),
					yValue, (float) (distance * Math.sin((k * angularStep)+angularOffset)));

			// Drawing range & the relation
			BranchGroup bgRel;
			if(relatedInstances.size() == 1)
				bgRel= relation.createRelationWithRange(relURI, relatedInstances.iterator().next(), position, direction);
			else{
				HashSet<String> labels=new HashSet<String>(relatedInstances.size());
				for(NodeTypeAbstraction node: relatedInstances){
					labels.add(node.toString());
				}
				bgRel= relation.createRelationWithSetOfElements(relURI, null, labels, position, direction, false);
			}
			// adds it to the scene bg
			tgx.addChild(bgRel);
			
			k++;
		}
	}
	
	private void drawSelfLoopRelation(TransformGroup tgx) {
		// Draws self loops direct & inherited
		if (this.currentInstanceData.getSelfLoopLinks().size() > 0 ) {
			Relation3DObject relation = new Relation3DObject();
			AxisAngle4f rotAxis =new AxisAngle4f(new Vector3f(0f,1f,0f),0);
			Hashtable<String, RelationType> relations=new Hashtable<String, RelationType>(this.currentInstanceData.getSelfLoopLinks().size());
			for(String relURI: this.currentInstanceData.getSelfLoopLinks()){
				relations.put(relURI, this.refMainPanel.model.getRelationTypes().get(relURI));
			}
			tgx.addChild(relation.createInstanceSelfLoopRelation(relations,	this.currentInstanceData.getInstanceCubeSide(), rotAxis) );
		}

	}
	
	private void drawConcept(TransformGroup tgx){
		Vector3f position = new Vector3f(0, VisualizationOptions.s5_ConceptHeight, 0);
		//		 Translates from orig
		TransformGroup parentTgx = this.createTransformGroup();
		Transform3D transf = new Transform3D();
		transf.setTranslation(position);
		parentTgx.setTransform(transf);
		// Drawing parent sphere & the isA relation
		ConceptSphere parentSphere = new ConceptSphere(VisualizationOptions.s5_ConceptSphereColor,
				this.currentConceptURI, true, false);
		parentSphere.setRadius(VisualizationOptions.s5_ConceptSphereRadius);
		parentTgx.addChild(parentSphere.createColoredSphere());
		// adds the relation fron current concept to its parent
		Relation3DObject isA = new Relation3DObject();
		isA.setTransparency(VisualizationOptions.s5_Rel2ConceptTransparency);
		tgx.addChild(parentTgx);
		tgx.addChild(isA.createISARelation(position));
		
	}
	
	/* (non-Javadoc)
	 * @see it.polito.elite.visualization.ontoviewer.scenes3D.Scene3D#createScene()
	 */
	public  void createScene() {
		this.addSceneLabel(this.currentInstanceURI);
		// Creates the scene groups
		BranchGroup bgx = this.createBranchGroup();
		TransformGroup tgx = this.createTransformGroup();
		// draws the central instance
		InstanceCube currentInstance = new InstanceCube();
		currentInstance.setIsWithLabel(false);
		currentInstance.setSide(VisualizationOptions.s5_cubeSide);
		BranchGroup centralInstance = currentInstance.createColoredCube(new Color3f(Color.DARK_GRAY), this.currentInstanceURI, true);
		tgx.addChild(centralInstance);
		// draw the current concept & relationArrow
		this.drawConcept(tgx);
		this.drawFacts(tgx);
		this.drawSameDifferent(tgx);
		bgx.addChild(tgx);
		this.tgSCENE.addChild(bgx);
	}
	

	private void reInitializeScene() {
//		 reinitialize the scene drawing the facts about the new instance
		this.tgSCENE.removeAllChildren();
		createScene();
	}

	@Override
	public boolean showContextMenu(final PopupMenu menu, final String uri) {
		final InstanceData idata = refMainPanel.model.getInstanceData(uri);
		if(idata == null){
			this.showSelectionPopUp(uri);
			return false;
		}
		
		MenuItem item1 = new MenuItem("Go To Concept");
		item1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String conceptURI = idata.getPertainingConceptURI();
				SceneFocusConcept3 myScene=new SceneFocusConcept3(refMainPanel,conceptURI);
				refMainPanel.changeScene(myScene);
				removePopUpMenu(menu);
			}
		});
		menu.add(item1);
		return true;
	}

	@Override
	public void selectElement(String uri) {
		if (uri.equals(this.currentConceptURI)){
			SceneFocusInstances4 myScene=new SceneFocusInstances4(this.refMainPanel,this.currentConceptURI);
			this.refMainPanel.changeScene(myScene);
		}
		else if(this.refMainPanel.model.getInstanceData(uri) == null){//Logic Expression..
			this.showSelectionPopUp(uri);
		}
		else{
			//reinitialize the scene with a new current instance
			this.currentInstanceURI=uri;
			this.currentInstanceData = this.refMainPanel.model.getInstanceData(this.currentInstanceURI);
			this.currentConceptURI = this.currentInstanceData.getPertainingConceptURI();
			this.reInitializeScene();
		}
	}

	@Override
	public void expand_OR_collapseElement(String uri) {	}

	@Override
	public boolean showRelationContextMenu(final PopupMenu menu, HashSet<RelationData> relSet) {
		PopupMenu createDependencyTree=new PopupMenu("Create a Dependency Tree for");
		menu.add(createDependencyTree);

		for (RelationData reldata : relSet) {
			final String relname=reldata.getRelationName();
			final LinkDirectionEnum dir=reldata.getDir();
			MenuItem item = new MenuItem(relname);
			item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String uri=getCurrentInstanceURI();
					SceneDependencyTree6 myScene=new SceneDependencyTree6(refMainPanel, uri, relname, dir);
					refMainPanel.changeScene(myScene);
					removePopUpMenu(menu);
				}
			});
			createDependencyTree.add(item);
		}
		PopupMenu setColor=new PopupMenu("Set Color To");
		menu.add(setColor);
		boolean found=false;
		for (RelationData reldata : relSet) {
			final String relname=reldata.getRelationName();
			final LinkDirectionEnum dir=reldata.getDir();
			MenuItem item = new MenuItem(relname);
			item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Color c = JColorChooser.showDialog(null, "Choose a color...", Color.WHITE);
					refMainPanel.model.setColorToRelation(relname, c);
					removePopUpMenu(menu);
				}
			});
			setColor.add(item);
			if(Relation3DObject.getRelationColor(relname)!=null)
				found=true;
		}
		
		if(found){
			PopupMenu unsetColor=new PopupMenu("Remove Custom Color");
			menu.add(unsetColor);
			for (RelationData reldata : relSet) {
				final String relname=reldata.getRelationName();
				if(Relation3DObject.getRelationColor(relname)!=null){
					final LinkDirectionEnum dir=reldata.getDir();
					MenuItem item = new MenuItem(relname);
					item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
					item.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							refMainPanel.model.removeColorToRelation(relname);
							removePopUpMenu(menu);
						}
					});
					unsetColor.add(item);
				}
			}
		}
		
		return true;
	}
	
	@Override
	public void fadeInAnimation() {
//		 create the animations
		BranchGroup bgAnimation = createBranchGroup();
		long time = System.currentTimeMillis();

		// timer for scale interpolator (Increasing)
		Alpha alpha = new Alpha(1, VisualizationOptions.sG_AnimationDuration);
		alpha.setStartTime(time-20);
		alpha.setIncreasingAlphaRampDuration(alpha.getIncreasingAlphaDuration() / 3);

		ScaleInterpolator si = new ScaleInterpolator(alpha, tgSCENE);
		si.setSchedulingBounds(new BoundingSphere());
		bgAnimation.addChild(si);
		
		//Adding element to the scene
		tgSCENE.addChild(bgAnimation);
	}

	@Override
	public void fadeOutAnimation(String uri, Scene3D nextScene) {
		// TODO Auto-generated method stub
		
	}
}
