package it.polito.elite.visualization.ontoviewer.scenes3D;

import it.polito.elite.ontology.NodeTypeAbstraction;
import it.polito.elite.protege.plugin.listeners.PostAnimation_GroupDetaching;
import it.polito.elite.protege.plugin.listeners.PostAnimation_SceneChanger;
import it.polito.elite.visualization.ontoviewer.behaviors.PositionInterpolatorWithListener;
import it.polito.elite.visualization.ontoviewer.behaviors.TransparencyInterpolatorWithListener;
import it.polito.elite.visualization.ontoviewer.enums.GraphicalLinkTypeEnum;
import it.polito.elite.visualization.ontoviewer.enums.LinkDirectionEnum;
import it.polito.elite.visualization.ontoviewer.enums.NodeTypesEnum;
import it.polito.elite.visualization.ontoviewer.models.ConceptData;
import it.polito.elite.visualization.ontoviewer.models.OrderedURIsElm;
import it.polito.elite.visualization.ontoviewer.models.RelationData;
import it.polito.elite.visualization.ontoviewer.tools.ConceptSphere;
import it.polito.elite.visualization.ontoviewer.tools.Relation3DObject;
import it.polito.elite.visualization.ontoviewer.userInterfaces.Pane3DScene;
import it.polito.elite.visualization.ontoviewer.userInterfaces.VisualizationOptions;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

import javax.media.j3d.Alpha;
import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ScaleInterpolator;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.TransparencyAttributes;
import javax.media.j3d.TransparencyInterpolator;
import javax.swing.JColorChooser;
import javax.vecmath.AxisAngle4f;
import javax.vecmath.Color3f;
import javax.vecmath.Point2d;
import javax.vecmath.Point2f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

public class SceneFocusTree2 extends Scene3D {

	private HashSet<String> nodes2_BeDrawn, expandable, onScene;

	private Hashtable<String, BranchGroup> conceptsBg, conceptSpheresBg;

	private int sceneRootLvl;

	private String expanded = null;

	private boolean showRelationWithObjectNotOnScene;

	private BranchGroup fakeBG;

	private Vector3f fakeSpherePosition;
	
	private boolean redraw = false;
	
	public SceneFocusTree2(Pane3DScene _mf, String uri) {
		super(_mf, uri, null, 2);

		this.sceneRootLvl = this.refMainPanel.model.getConceptData(uri)
				.getLayer();

		this.nodes2_BeDrawn = new HashSet<String>();
		this.expandable = new HashSet<String>();
		this.onScene = new HashSet<String>();
		this.conceptsBg = new Hashtable<String, BranchGroup>();
		this.conceptSpheresBg = new Hashtable<String, BranchGroup>();
		this.showRelationWithObjectNotOnScene = VisualizationOptions.sR_ShowRelationTowardsNodeNotOnScreen;

		if(this.sceneRootLvl>1)
			this.addSceneLabel(uri + " subtree");
		else
			this.addSceneLabel(uri + " tree");
		this.createScene();
		this.addBehaviors();
	}

	public void createScene() {
		this.expanded = this.getCurrentConceptURI();
		this.rootPosition = this
				.centralPositionAtGivenHeightScale(VisualizationOptions.s2_ScreenPercent_as_RootVerticalPosition);
		// ricalcolo le posizioni per l'albero corrente
		Transform3D backToInitialView = new Transform3D();
		backToInitialView.setTranslation(this.rootPosition);
		// this.refMainPanel.tgGlobalTransform.setTransform(backToInitialView);
		this.tgROT.setTransform(backToInitialView);

		Vector3f center = new Vector3f(0f, 0f, 0f);
		// computes positions for nodes and creates their branchgroups
		this.computeNodePosition(this.expanded, center);
		// If just one leaf is expandable, there is no possible collision so
		// lets expand
		while (this.expandable.size() == 1) {
			String newUri = this.expandable.iterator().next();
			this.expandable.clear();
			center = this.positions.get(newUri);
			this.computeNodePosition(newUri, center);
		}
		// attach already drawn nodes to the scene BG
		this.attachDrawnNodesToScene(this.nodes2_BeDrawn);
		this.nodes2_BeDrawn.clear();

	}

	private void computeNodePosition(String uri, Vector3f center) {
		// Computes the position of the second level taken into account the
		// third one
		ConceptData conceptData = this.refMainPanel.model.getConceptData(uri);
		nodes2_BeDrawn.add(uri);
		if (!onScene.contains(uri)) {
			onScene.add(uri);
			this.positions.put(uri, center);
		}
		BranchGroup bgx = drawConcept(uri, conceptData, center, null);
		this.conceptsBg.put(uri, bgx);

		// first elm has the smaller number of children, last one the biggest
		OrderedURIsElm[] array = conceptData.getOrderedChildrenURIs();
		float[] angles = new float[array.length];

		// Compute the angular position for each child
		int numSons = array.length;
		float alpha = (float) (2 * Math.PI / numSons);
		float alpha0 = (float) (Math.random() * 2 * Math.PI);

		// Assign position of sphere-concepts within the positionin circle line
		// iterleaving children decreasingly having an higher number of
		// grandchildren with ones having a lower number of them
		for (int k = 0, i = 0, j = numSons - 1; k < numSons; k++) {
			if (k % 2 == 0)// even turn
				angles[i++] = (alpha0 + alpha * k);
			else
				angles[j--] = (alpha0 + alpha * k);
		}
		// Calculate the smallest radius (as if no sub levels were present)
		float radius = this.getSubRadius(numSons);
		float only2ndLvlElements = radius;

		// Ampliate the radius by detecting collision betwenn circles of
		// children and  properly increasing radius if so
		for (int i = 0; i < numSons; i++) {
			if (array[i].getNumberOfChildren() < 1)
				continue;
			float theta = angles[i];
			Point2f a = new Point2f((float) (radius * Math.cos(theta)),
					(float) (radius * Math.sin(theta)));
			float radiusA = this.getSubDistanceRadius(array[i]
					.getNumberOfChildren());
			for (int j = i + 1; j < numSons; j++) {
				if (array[j].getNumberOfChildren() < 1)
					continue;
				float theta1 = angles[j];
				Point2f b = new Point2f((float) (radius * Math.cos(theta1)),
						(float) (radius * Math.sin(theta1)));
				float radiusB = this.getSubDistanceRadius(array[j]
						.getNumberOfChildren());
				float minimumDistance = radiusA + radiusB;
				if (a.distance(b) < minimumDistance) {
					radius = calculateNewRadius(theta, theta1, minimumDistance);
				}
			}
		}

		// Set the Position for the children
		boolean areVisible = true;
		for (int i = 0; i < numSons; i++) {
			float x = (float) (radius * Math.cos(angles[i])) + center.x;
			float z = (float) (radius * Math.sin(angles[i])) + center.z;
			Vector3f posChild = new Vector3f(x, center.y
					- VisualizationOptions.s2_TreeLevelHeight, z);

			// Are all 2nd lvl spheres visible?
			Dimension screenSize = this.getCanvas3DSize();
			Point2d pixel = this.get3DTo2DPoint(new Point3d(posChild));
			float deltaX = screenSize.width
					* VisualizationOptions.s2_PercentageOfScreen_NotVisibleButValid;
			float deltaY = screenSize.height
					* VisualizationOptions.s2_PercentageOfScreen_NotVisibleButValid;
			if (pixel.x < (0 - deltaX) || pixel.x > (screenSize.width + deltaX)
					|| pixel.y < (0 - deltaY)
					|| pixel.y > (screenSize.height + deltaY)) {
				areVisible = false;
				radius = only2ndLvlElements;
			}
		}

		// Set positions for the second level and further
		for (int i = 0; i < numSons; i++) {
			float x = (float) (radius * Math.cos(angles[i])) + center.x;
			float z = (float) (radius * Math.sin(angles[i])) + center.z;
			Vector3f posChild = new Vector3f(x, center.y
					- VisualizationOptions.s2_TreeLevelHeight, z);

			setPos(array[i].getUri(), posChild, uri, center, areVisible);
		}
	}

	// Set position to the second lvl and and computes the third position
	private void setPos(String uri, Vector3f center, String parent,
			Vector3f center_father, boolean propagate) {
		ConceptData conceptData = this.refMainPanel.model.getConceptData(uri);
		this.positions.put(uri, center);

		nodes2_BeDrawn.add(uri);
		onScene.add(uri);

		if (!propagate) {
			if (conceptData.getNumberOfChildren() > 0)
				this.expandable.add(uri);
			BranchGroup bgx = drawConcept(uri, conceptData, center,
					center_father);
			this.conceptsBg.put(uri, bgx);
			return;
		}

		// first elm has the smaller number of children, last one the biggest
		OrderedURIsElm[] array = conceptData.getOrderedChildrenURIs();
		float[] angles = new float[array.length];

		// Compute the angular position for each child
		int numSons = array.length;
		float alpha = (float) (2 * Math.PI / numSons);
		float alpha0 = (float) Math.random();

		// Assign position of sphere-concepts within the positionin circle line
		// iterleaving children decreasingly having an higher number of
		// grandchildren
		// with ones having a lower number of them
		for (int k = 0, i = 0, j = numSons - 1; k < numSons; k++) {
			if (k % 2 == 0)// even turn
				angles[i++] = (alpha0 + alpha * k);
			else
				angles[j--] = (alpha0 + alpha * k);
		}

		// Calculate the smallest radius (as if no sub levels were present)
		float radius = getSubRadius(numSons);

		// Set the Position for the children
		boolean areVisible = true;
		for (int i = 0; i < numSons; i++) {
			float x = (float) (radius * Math.cos(angles[i])) + center.x;
			float z = (float) (radius * Math.sin(angles[i])) + center.z;
			Vector3f posChild = new Vector3f(x, center.y
					- VisualizationOptions.s2_TreeLevelHeight, z);

			// Are all 2nd lvl spheres visible?
			Dimension screenSize = this.getCanvas3DSize();
			Point2d pixel = this.get3DTo2DPoint(new Point3d(posChild));
			if (pixel.x < 0 || pixel.x > screenSize.width || pixel.y < 0
					|| pixel.y > screenSize.height) {
				areVisible = false;
			}
		}

		if (areVisible) {
			// Set the Position for the children
			for (int i = 0; i < numSons; i++) {
				float x = (float) (radius * Math.cos(angles[i]));
				float z = (float) (radius * Math.sin(angles[i]));
				Vector3f posChild = new Vector3f(center.x + x, center.y
						- VisualizationOptions.s2_TreeLevelHeight, center.z + z);
				setPos(array[i].getUri(), posChild, uri, center, false);
			}
		} else {
			this.expandable.add(uri);
		}

		BranchGroup bgx = drawConcept(uri, conceptData, center, center_father);
		this.conceptsBg.put(uri, bgx);
	}

	private float getSubRadius(int numberOfChildren) {
		float radius = 0;
		if (numberOfChildren > 1 && numberOfChildren < 5) {
			radius = 4 * VisualizationOptions.s2_SimpleSphereRadius;
		} else if (numberOfChildren > 4) {
			radius = (float) (VisualizationOptions.s2_Segment2ArcCorrectionFactor
					* numberOfChildren
					* (VisualizationOptions.s2_SimpleSphereRadius + VisualizationOptions.sG_distance_min) / (Math.PI));
		}
		return radius;
	}

	private float getSubDistanceRadius(int numberOfChildren) {
		float radius = 0;
		if (numberOfChildren == 1) {
			radius = VisualizationOptions.s2_FamilyRadiusMultiplier
					* VisualizationOptions.s2_SimpleSphereRadius;
		}
		if (numberOfChildren > 1 && numberOfChildren < 5) {
			radius = 3 * VisualizationOptions.s2_FamilyRadiusMultiplier
					* VisualizationOptions.s2_SimpleSphereRadius;
		} else if (numberOfChildren > 4) {
			radius = (float) (VisualizationOptions.s2_Segment2ArcCorrectionFactor
					* numberOfChildren
					* VisualizationOptions.s2_FamilyRadiusMultiplier
					* (VisualizationOptions.s2_SimpleSphereRadius + VisualizationOptions.sG_distance_min) / (Math.PI));
		}
		return radius;
	}

	private float calculateNewRadius(float angle1, float angle2, float distance) {
		float k = (float) (Math.pow((Math.cos(angle1) - Math.cos(angle2)), 2) + Math
				.pow((Math.sin(angle1) - Math.sin(angle2)), 2));
		return (float) (Math.sqrt(Math.pow(distance, 2) / k));
	}

	private TransformGroup drawISA(TransformGroup tgx, Vector3f posFather,
			Vector3f center) {

		// disegna la relazioni IS_A
		Vector3f relationEnd = new Vector3f(posFather.x - center.x, posFather.y
				- center.y, posFather.z - center.z);
		Relation3DObject relation = new Relation3DObject();
		relation.setTransparency(.3f);
		BranchGroup branchGroup = relation.createISARelation(relationEnd);
		tgx.addChild(branchGroup);
		return tgx;
	}

	private void DrawSemanticRelation(TransformGroup tgx, ConceptData data,
			Vector3f center_father, String conceptUri) {
		// SELF LOOPS
		if (data.getSelfLoopLinks().size() > 0) {
			Relation3DObject relation = new Relation3DObject();
			AxisAngle4f rotAxis;

			// Define end points as new vectors
			Vector3f fromDir = new Vector3f(1f, 0f, 0f);
			Vector3f toDir;
			Vector3f myCenter = this.positions.get(data.getUri());
			if (center_father != null) {
				toDir = new Vector3f(myCenter.x - center_father.x, 0,	myCenter.z - center_father.z);
				toDir.normalize();

				if ((toDir.equals(fromDir) || toDir.equals(new Vector3f(-1f, 0f, 0f)))) {
					rotAxis = new AxisAngle4f();
				} else {
					// Compute the value of the angle between them (no spatial
					// orientation info though)
					float rotInRadians = fromDir.angle(toDir);

					// Compute an orthagonal vector to later define the rotation
					// axis
					Vector3f orthagonalAxis = new Vector3f();
					orthagonalAxis.cross(fromDir, toDir);
					orthagonalAxis.normalize();

					// Create new rotation matrix
					rotAxis = new AxisAngle4f(orthagonalAxis, rotInRadians);
				}
			} else {
				rotAxis = new AxisAngle4f(new Vector3f(0f, 1f, 0f), 0);
			}

			tgx.addChild(relation.createSelfLoopRelation(data.getSelfLoopLinks(), data
							.getConceptSphereRadius(), rotAxis));

		}

		// OBJ PROPERTIES
		this.DrawOutgoingRelation(tgx, center_father, data.getOutgoingLinks(),
				conceptUri, data);
		this.DrawIncomingRelation(tgx, center_father, data.getIncomingLinks(),
				conceptUri, data);
	}

	private void DrawIncomingRelation(TransformGroup tgx, Vector3f center_father,
			Hashtable<String, HashSet<RelationData>> arcs, String conceptUri, ConceptData data) {
		if (arcs.size() > 0) {
			/*
			 * FILTERING NODES (ONLY not ON SCENE)
			 */
			// A table for relations from named concept not on screen. Hashtable
			// with relations as keys and the EndNodes as object
			Hashtable<String, HashSet<NodeTypeAbstraction>> inRelationsNotOnScreen = new Hashtable<String, HashSet<NodeTypeAbstraction>>();
			for (String relURI : arcs.keySet()) {
				HashSet<NodeTypeAbstraction> endNodes = new HashSet<NodeTypeAbstraction>(
						arcs.get(relURI).size());
				for (RelationData rdata : arcs.get(relURI)) {
					NodeTypeAbstraction endNode = rdata.getRelatedNode();
					if (!this.onScene.contains(endNode.toString())) {
						if (rdata.isSimple())
							endNodes.add(endNode);
					}
				}
				if (endNodes.size() > 0)
					inRelationsNotOnScreen.put(relURI, endNodes);
			}
			/*
			 * DRAWING INCOMING RELATIONS
			 */
			Vector3f center = this.positions.get(conceptUri);

			// Draw relations to concepts NOT on the scene
			if (this.showRelationWithObjectNotOnScene && inRelationsNotOnScreen.size()>0) {
				this.drawRelationsToNodeNotOnScreen(tgx, center, center_father,
						inRelationsNotOnScreen, data.getConceptSphereRadius(),
						LinkDirectionEnum.out);
			}

		}
	}

	private void DrawOutgoingRelation(TransformGroup tgx,
			Vector3f center_father,
			Hashtable<String, HashSet<RelationData>> arcs, String conceptUri,
			ConceptData data) {
		if (arcs.size() > 0) {
			/*
			 * Dividing relations towards nodes on screeen and not on screen.
			 * Then groups those relations on the end point Node
			 */
			// A table for relations towards element not on screen. Hashtable
			// with keys: ranges as NodeTypes
			// and as Obj: a set of relation URIs directed to that ranges
			Hashtable<String, HashSet<NodeTypeAbstraction>> outRelationsNotOnScreen = new Hashtable<String, HashSet<NodeTypeAbstraction>>();
			// A table for relations toward elem on screen. Hashtable with
			// relations as keys and the EndNodes as object
			Hashtable<String, HashSet<RelationData>> outRelationsOnScreen = new Hashtable<String, HashSet<RelationData>>(
					3, 2);
			for (String relUri : arcs.keySet()) {
				HashSet<NodeTypeAbstraction> endNodes = new HashSet<NodeTypeAbstraction>(
						arcs.get(relUri).size());
				for (RelationData myRelationData : arcs.get(relUri)) {
					NodeTypeAbstraction myRange = myRelationData
							.getRelatedNode();
					boolean isOnScene = false;
					if (myRange.getType() == NodeTypesEnum.NamedClass) {
						String endUri = myRange.toString();
						if (this.onScene.contains(endUri)) {
							isOnScene = true;
							if (outRelationsOnScreen.containsKey(endUri)) {
								HashSet<RelationData> relations = outRelationsOnScreen
										.get(endUri);
								relations.add(myRelationData);
							} else {
								HashSet<RelationData> relations = new HashSet<RelationData>(
										2, 2);
								relations.add(myRelationData);
								outRelationsOnScreen.put(endUri, relations);
							}
						}
					}
					if (!isOnScene) {
							endNodes.add(myRelationData.getRelatedNode());
					}
				}
				if (endNodes.size() > 0)
					outRelationsNotOnScreen.put(relUri, endNodes);
			}

			Vector3f center = this.positions.get(conceptUri);

			// Draw relations to concepts on the scene
			for (String endUri : outRelationsOnScreen.keySet()) {
				HashSet<RelationData> relationsSet = outRelationsOnScreen
						.get(endUri);
				this.drawRelationsToNodeOnScreen(tgx, endUri, center, data,
						relationsSet);
			}

			// Draw relations to concepts NOT on the scene
			if (this.showRelationWithObjectNotOnScene && outRelationsNotOnScreen.size()>0) {
				this.drawRelationsToNodeNotOnScreen(tgx, center, center_father,
						outRelationsNotOnScreen, data.getConceptSphereRadius(),
						LinkDirectionEnum.out);
			}
		}
	}

	private void drawRelationsToNodeOnScreen(TransformGroup tgx, String endUri,
			Vector3f center, ConceptData data,
			HashSet<RelationData> relationsSet) {
		Relation3DObject relation = new Relation3DObject();
		// Draw a plain relation line to the sphere
		Vector3f posEndUri = this.positions.get(endUri);
		Vector3f posDest = new Vector3f(posEndUri.x - center.x, posEndUri.y
				- center.y, posEndUri.z - center.z);

		// if semantic relation is towards a children or a parent
		// draw a curved rel
		// (so don't overlapping the isA) otherwise a straight one
		if (data.getChildren().contains(endUri)
				|| data.getAllParents().contains(endUri)) {
			tgx.addChild(relation.createCurvedArrowRelation(relationsSet,
					posDest, VisualizationOptions.s2_CurvedArrowCurvature,
					LinkDirectionEnum.out));
		} else {
			tgx.addChild(relation.createStraightRelation(relationsSet, posDest,
					data.getConceptSphereRadius(), LinkDirectionEnum.out));
		}
	}

	private void drawRelationsToNodeNotOnScreen( TransformGroup tgx, Vector3f center, Vector3f center_father,
			Hashtable<String, HashSet<NodeTypeAbstraction>> relationsNotOnScreen,
			float conceptSphereRadius, LinkDirectionEnum dir) {
		// Calculate relative position angle for relations not on Scene
		float alpha0;
		if (center_father == null) {
			alpha0 = 0f;
		} else {
			Point2f a = new Point2f(center.x, center.z);
			Point2f b = new Point2f(center_father.x, center_father.z);
			float distance =  b.distance(a);
			if(distance == 0){
				System.err.println("err");
				alpha0 = 0;
			}
			else
				alpha0 = (float) (Math.acos((a.x - b.x) / distance));

			if (a.y < b.y)
				alpha0 *= -1f;
		}

		float radius = VisualizationOptions.s2_RelationOutOfTree_DistanceMultiplier* conceptSphereRadius + conceptSphereRadius;

		Relation3DObject relation = new Relation3DObject();

		float y = conceptSphereRadius*0.3f;
		if(dir == LinkDirectionEnum.out)
			y=0;
		float x = (float) (radius * Math.cos(alpha0));
		float z = (float) (radius * Math.sin(alpha0));
		
		Vector3f pos=new Vector3f(x, y, z);
		tgx.addChild(relation.createSideRelations(relationsNotOnScreen, 	pos, dir));

		return;
	}

	private BranchGroup drawConcept(String uri, ConceptData data,
			Vector3f center, Vector3f center_father) {

		BranchGroup bgx = this.createBranchGroup();
		TransformGroup tgx = this.createTransformGroup();

		// posiziona (transla il transform group)
		Transform3D transf = new Transform3D();
		transf.setTranslation(center);
		tgx.setTransform(transf);

		ConceptSphere sfera;
		if (this.expandable.contains(uri)) {// expandible concepth spheres are
			// bigger
			Color3f color = VisualizationOptions.s2_ParentSphereColor;
			// creating the concept sphere
			sfera = new ConceptSphere(color, uri, true, data.hasInstances());
			sfera.setRadius(this.computeConceptSphereRadius(data
					.getSubTreeDepth()));
			sfera.setLabelColor(VisualizationOptions.s2_LabelParentSphereColor);
			sfera.setLabelScale(VisualizationOptions.s2_LabelParentSphereScale);
		} else {
			// Set color to the sphere
			Color3f color = VisualizationOptions.chooseColor(data.getLayer()
					- this.sceneRootLvl);
			sfera = new ConceptSphere(color, uri, true, data.hasInstances());
			sfera.setRadius(VisualizationOptions.s2_SimpleSphereRadius);
			sfera.setLabelColor(VisualizationOptions.s2_LabelSimpleSphereColor);
			sfera.setLabelScale(VisualizationOptions.s2_LabelSimpleSphereScale);
		}

		data.setConceptSphereRadius(sfera.getRadius());

		// add the sphere to the scene
		BranchGroup sphereBg = sfera.createColoredSphere();
		this.conceptSpheresBg.put(uri, sphereBg);
		tgx.addChild(sphereBg);
		bgx.addChild(tgx);

		return bgx;
	}

	private void drawConceptRelations(BranchGroup bgx, ConceptData data,
			String uri, Vector3f center_father) {
		Enumeration p_children = bgx.getAllChildren();
		TransformGroup tgx = null;
		while (p_children.hasMoreElements()) {
			Object object = p_children.nextElement();

			if (object instanceof TransformGroup) {
				tgx = (TransformGroup) object;
			}
		}
		if (tgx == null) {
			return;
		}
		// Disegno relazioni isA dirette o adottive
		// if not local root
		for (String parent_URI : data.getAllParents()) {
			if (this.onScene.contains(parent_URI)) {
				tgx = this.drawISA(tgx, this.positions.get(parent_URI),
						this.positions.get(uri));
			}

		}

		// Disegno relazioni semantiche
		this.DrawSemanticRelation(tgx, data, center_father, uri);
	}

	private float computeConceptSphereRadius(int subTreeDepth) {
		float radius = VisualizationOptions.s2_SimpleSphereRadius
				+ subTreeDepth
				* VisualizationOptions.s2_ParentSphereRadiusMultiplier;
		if ((radius > VisualizationOptions.s2_ParentSphereRadiusLimit)
				&& (VisualizationOptions.s2_ParentSphereRadiusLimit > 0))
			radius = VisualizationOptions.s2_ParentSphereRadiusLimit;
		return radius;
	}

	/**
	 * This method expand a concept on the scene that is expandable(a withe one)
	 * Such action implies the collapsing of all the nodes except the one in the
	 * direct line of ancestor from the current concept to the root. A subtree
	 * of 3 level depth is drawed beneth the selected concept. The nodes
	 * belonging to the ancestor line and the expandable ones in the subtree are
	 * saved in an expandable list
	 * 
	 * @param uri
	 *            the concept to be expanded
	 */
	private void expandConcept(String uri) {

		// clears the previuos list
		this.expanded = uri;
		this.expandable.remove(uri);
		this.conceptsBg.get(this.expanded).detach();
		Vector3f newPos = this.positions.get(uri);

		String parent=null; //search for the parent on scene
		for(String myParent: this.refMainPanel.model.getConceptData(uri).getAllParents()){
			if(this.conceptsBg.containsKey(myParent))
				parent = myParent;
		}
		// eliminates potentially colliding subtree, if present
		ConceptData data = this.refMainPanel.model.getConceptData(parent);
		for (String uriSon : data.getChildren()) {
			ConceptData sonData = this.refMainPanel.model
					.getConceptData(uriSon);
			if (!uriSon.equals(uri) && sonData.getChildren().size() > 0)
				collapseSubTree(uriSon, sonData);
		}

		// computes the new Position
		this.computeNodePosition(uri, newPos);
		this.attachDrawnNodesToScene(this.nodes2_BeDrawn);
		this.nodes2_BeDrawn.clear();

		// udates the view
		this.updateView(newPos);
		repaintCanvas();
	}

	/**
	 * Recursive procedure that Collapse a subTree
	 * 
	 * @param data
	 *            Concept that should be collapsed
	 * @param uri
	 *            URI of the concept to be collapsed along with its subtree
	 */
	private void collapseSubTree(String uri, ConceptData data) {
		String parent=null; //search for the parent on scene
		for(String myParent: data.getAllParents()){
			if(this.conceptsBg.containsKey(myParent))
				parent = myParent;
		}
		if (parent != null) {
			this.expandable.add(uri);
			Vector3f center_father = this.positions.get(parent);
			Vector3f center = this.positions.get(uri);
			BranchGroup parentBG = this.conceptsBg.get(parent);
			removeChildrenFromScene(uri);
			// Make appareance of node "expandable"
			this.conceptsBg.get(uri).detach();
			BranchGroup bgx = this.drawConcept(uri, data, center, center_father);
			this.drawConceptRelations(bgx, data, uri, center_father);
			parentBG.addChild(bgx);
			this.conceptsBg.put(uri, bgx);
		}
	}

	private void removeChildrenFromScene(String uri) {
		ConceptData data = this.refMainPanel.model.getConceptData(uri);
		for (String uriSon : data.getChildren()) {
			this.onScene.remove(uriSon);
			removeChildrenFromScene(uriSon);
		}
	}

	/**
	 * procedure which draws the nodes in the internal queue and then clears the
	 * queue itself
	 */
	private void attachDrawnNodesToScene(HashSet<String> set) {
		for (String uri : set) {
			ConceptData data = this.refMainPanel.model.getConceptData(uri);
			String parent=null; //search for the parent on scene
			for(String myParent: data.getAllParents()){
				if(this.onScene.contains(myParent)){
					parent = myParent;
					System.err.println("FOUND "+parent+" as parent of "+uri);
				}
			}
			
			if (uri.equals(this.expanded)) {
				BranchGroup bgx = this.conceptsBg.get(uri);
				this.drawConceptRelations(bgx, data, uri, null);
			} else {
				Vector3f center_father = this.positions.get(parent);
				BranchGroup parentGroup = this.conceptsBg.get(parent);
				BranchGroup bgx = this.conceptsBg.get(uri);
				this.drawConceptRelations(bgx, data, uri, center_father);
				parentGroup.addChild(bgx);
			}
		}
		// adding concepts to the scene
		if (this.expanded.equals(this.currentConceptURI)) {
			this.drawParents( this.conceptsBg.get(this.expanded), this.refMainPanel.model.getConceptData(this.currentConceptURI));
			if (!VisualizationOptions.sG_useAnimations || this.redraw) {
				this.tgSCENE.addChild(this.conceptsBg.get(this.expanded));
			}
		} else {
			ConceptData data = this.refMainPanel.model
					.getConceptData(this.expanded);
			String parent=null; //search for the parent on scene
			for(String myParent: data.getAllParents()){
				if(this.conceptsBg.containsKey(myParent))
					parent = myParent;
			}
			BranchGroup parentGroup = this.conceptsBg.get(parent);
			parentGroup.addChild(this.conceptsBg.get(this.expanded));
		}

	}

	public void selectElement(String uri) {
		Scene3D myScene;
		if (uri.equals(this.currentConceptURI)) {
			myScene = new SceneFocusRoots1(this.refMainPanel,
					this.refMainPanel.conceptsOfCurrentLogicView);
		} else {
			//if the element is a named entities or a logical set..
			if (this.refMainPanel.model.getConceptData(uri) == null) { 
				this.showSelectionPopUp(uri);
				return;
			}
			myScene = new SceneFocusConcept3(this.refMainPanel, uri);
		}
		if (VisualizationOptions.sG_useAnimations) {
			this.fadeOutAnimation(uri, myScene);
		} else {
			this.refMainPanel.changeScene(myScene);
		}

	}

	@Override
	public void expand_OR_collapseElement(String uri) {
		if (this.expandable.contains(uri))
			this.expandConcept(uri);
		else {
			ConceptData data = this.refMainPanel.model.getConceptData(uri);
			if (data.getChildren().size() > 0)
				this.collapseSubTree(uri, data);
		}
	}

	@Override
	public boolean showRelationContextMenu(final PopupMenu menu,	HashSet<RelationData> relSet) { 
		
		PopupMenu setColor=new PopupMenu("Set Color To");
		menu.add(setColor);
		boolean found=false;
		for (RelationData reldata : relSet) {
			final String relname=reldata.getRelationName();
			final LinkDirectionEnum dir=reldata.getDir();
			MenuItem item = new MenuItem(relname);
			item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Color c = JColorChooser.showDialog(null, "Choose a color...", Color.WHITE);
					refMainPanel.model.setColorToRelation(relname, c);
					removePopUpMenu(menu);
				}
			});
			setColor.add(item);
			if(Relation3DObject.getRelationColor(relname)!=null)
				found=true;
		}
		
		if(found){
			PopupMenu unsetColor=new PopupMenu("Remove Custom Color");
			menu.add(unsetColor);
			for (RelationData reldata : relSet) {
				final String relname=reldata.getRelationName();
				if(Relation3DObject.getRelationColor(relname)!=null){
					final LinkDirectionEnum dir=reldata.getDir();
					MenuItem item = new MenuItem(relname);
					item.setFont(new Font("Arial",Font.BOLD|Font.ITALIC,12));
					item.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							refMainPanel.model.removeColorToRelation(relname);
							removePopUpMenu(menu);
						}
					});
					unsetColor.add(item);
				}
			}
		}
		return true;
	}

	public boolean showContextMenu(final PopupMenu menu, final String uri) {
		ConceptData cd = this.refMainPanel.model.getConceptData(uri);
		
		if(cd == null){
			this.showSelectionPopUp(uri);
			return false;
		}
		
		if (this.refMainPanel.model.getConceptData(uri).getNumberOfChildren() > 0){
			MenuItem item = new MenuItem("Explore Subtree");
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
				Scene3D myScene = new SceneFocusTree2(refMainPanel, uri);
				refMainPanel.changeScene(myScene);	
				removePopUpMenu(menu);
			}});
			menu.add(item);
		}

		MenuItem item = new MenuItem("Explore Concept");
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			Scene3D myScene = new SceneFocusConcept3(refMainPanel, uri);

			if (VisualizationOptions.sG_useAnimations) {
				fadeOutAnimation(uri, myScene);
			} else {
				refMainPanel.changeScene(myScene);
			}
			removePopUpMenu(menu);
		}});
		menu.add(item);
		
		
		if (cd.hasInstances()) {
			MenuItem item0 = new MenuItem("Show Direct Instances");
			item0.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SceneFocusInstances4 myScene = new SceneFocusInstances4(
							refMainPanel, uri);
					refMainPanel.changeScene(myScene);
					removePopUpMenu(menu);
				}
			});
			menu.add(item0);
		}
		
		if(cd.hasNonDirectInstances()){
			MenuItem item1 = new MenuItem("Show All Instances");
			item1.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Set<String> set = refMainPanel.model
							.retrieveAllInstancesOf(uri);
					SceneFocusInstances4 myScene = new SceneFocusInstances4(
							refMainPanel, set, uri);
					refMainPanel.changeScene(myScene);
					removePopUpMenu(menu);
				}
			});
			menu.add(item1);
		}
		
		if (expandable.contains(uri)) {
			MenuItem item2 = new MenuItem("Expand subtree");
			item2.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					expandConcept(uri);
					removePopUpMenu(menu);
				}
			});
			menu.add(item2);
		}
		
		
		  if (this.showRelationWithObjectNotOnScene) {
			  MenuItem item3 = new MenuItem( "Hide Semantic Relation out of subTree");
			  item3.addActionListener(new ActionListener() { 
			 public void  actionPerformed(ActionEvent e) { 
				 showRelationWithObjectNotOnScene = false; 
				 redraw();
				 removePopUpMenu(menu); 
				 } }); 
			  menu.add(item3); 
			  } 
		  else {
			  MenuItem item4 = new MenuItem( "Show Semantic Relation out of subTree"); 
			  item4.addActionListener(new ActionListener() { 
				  public void actionPerformed(ActionEvent e) { 
					  showRelationWithObjectNotOnScene = true; 
					  redraw();
					  removePopUpMenu(menu); 
				} }); 
			  menu.add(item4); }
		 

		return true;
	}

	// ///////////// ANIMATIONS

	public void fadeInAnimation() {
		// create the animations
		BranchGroup bgAnimation = createBranchGroup();
		long time = System.currentTimeMillis();

		// timer for scale interpolator (Increasing)
		Alpha alpha = new Alpha(1,
				VisualizationOptions.sG_AnimationDuration / 3);
		alpha.setStartTime(time);
		alpha
				.setIncreasingAlphaRampDuration(alpha
						.getIncreasingAlphaDuration() / 4);

		ScaleInterpolator si = new ScaleInterpolator(alpha, tgSCENE);
		si.setSchedulingBounds(new BoundingSphere());
		bgAnimation.addChild(si);

		// Adding element to the scene
		tgSCENE.addChild(bgAnimation);
		this.tgSCENE.addChild(this.conceptsBg.get(this.expanded));
	}

	public void fadeOutAnimation(String uri, Scene3D nextScene) {
		// Disables interaction with the scene
		this.removeBehaviours();
		// create the animations
		BranchGroup bgAnimation = createBranchGroup();
		BranchGroup sphereBG = this.conceptSpheresBg.get(uri);
		sphereBG.detach();
		BranchGroup conceptBG = this.conceptsBg.get(uri);
		conceptBG.removeChild(sphereBG);
		TransformGroup fakeTG = this.createFakeConceptSphere(uri, sphereBG);
		long time = System.currentTimeMillis();
		Vector3f center = this.positions.get(uri);

		Vector3f finalPosition = new Vector3f(.0f, .0f, .0f);
		this.fadeOutScene(bgAnimation, time, center);
		this.fadeOutFakeSphere(bgAnimation, time, finalPosition, fakeTG,
				nextScene);

		fakeBG.addChild(bgAnimation);
	}

	private void fadeOutScene(BranchGroup bgAnimation, long time,
			Vector3f center) {
		// timer for transparency interpolation (Increasing)
		Alpha alpha = new Alpha(1,
				VisualizationOptions.sG_AnimationDuration / 2);
		alpha.setStartTime(time);
		alpha
				.setIncreasingAlphaRampDuration(alpha
						.getIncreasingAlphaDuration() / 3);

		Shape3D array[] = Scene3D.getShapes(this.tgSCENE);
		for (Shape3D shape : array) {

			TransparencyAttributes ta = shape.getAppearance()
					.getTransparencyAttributes();
			if (ta == null) {
				Appearance app = shape.getAppearance();
				ta = new TransparencyAttributes(TransparencyAttributes.BLENDED,
						0f);
				ta.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
				ta.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
				try {
					app.setTransparencyAttributes(ta);
				} catch (Exception e1) {
					System.err
							.println("Fade out sc2: Error in setting transparency attributes"
									+ shape.getClass());
				}

			}
			TransparencyInterpolator ti = new TransparencyInterpolator(alpha,
					ta, ta.getTransparency(), 1f);
			ti.setSchedulingBounds(new BoundingSphere());
			bgAnimation.addChild(ti);
		}
	}

	private void fadeOutFakeSphere(BranchGroup bgAnimation, long time,
			Vector3f finalPosition, TransformGroup fakeTG, Scene3D nextScene) {
		// timer for position interpolator (increasing)
		Alpha gamma = new Alpha(1,
				VisualizationOptions.sG_AnimationDuration / 4);
		gamma.setStartTime(time + VisualizationOptions.sG_AnimationDuration * 3
				/ 4);
		gamma
				.setIncreasingAlphaRampDuration(gamma
						.getIncreasingAlphaDuration() / 3);

		finalPosition.sub(this.fakeSpherePosition);
		float translLenght = finalPosition.length();
		Transform3D transf = computeOrientation(finalPosition);

		PostAnimation_SceneChanger change = new PostAnimation_SceneChanger(
				this.refMainPanel, nextScene, false);
		PositionInterpolatorWithListener piFinal = new PositionInterpolatorWithListener(
				gamma, fakeTG, transf, 0f, translLenght, change);
		piFinal.setSchedulingBounds(new BoundingSphere());
		bgAnimation.addChild(piFinal);
		// timer 4 Fading Away
		Alpha delta = new Alpha(1,
				VisualizationOptions.sG_AnimationDuration / 4);
		delta.setStartTime(time + VisualizationOptions.sG_AnimationDuration);
		delta
				.setIncreasingAlphaRampDuration(delta
						.getIncreasingAlphaDuration() / 3);

		// Fading Away
		Shape3D array[] = Scene3D.getShapes(this.fakeBG);
		for (Shape3D shape : array) {
			TransparencyAttributes ta = shape.getAppearance()
					.getTransparencyAttributes();

			if (ta == null) {
				Appearance app = shape.getAppearance();
				ta = new TransparencyAttributes(TransparencyAttributes.BLENDED,
						0f);
				ta.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
				ta.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
				try {
					app.setTransparencyAttributes(ta);
				} catch (Exception e1) {
					System.err
							.println("Fade out sc1 fakeSph: Error in setting transparency attributes");
				}
			}

			PostAnimation_GroupDetaching cleaner = new PostAnimation_GroupDetaching(
					this.fakeBG);
			TransparencyInterpolatorWithListener ti = new TransparencyInterpolatorWithListener(
					delta, ta, ta.getTransparency(), 1f, cleaner);
			ti.setSchedulingBounds(new BoundingSphere());
			bgAnimation.addChild(ti);
		}
	}

	private TransformGroup createFakeConceptSphere(String uri,
			BranchGroup sphereBG) {
		fakeBG = this.createBranchGroup();
		TransformGroup fakeTG = this.createTransformGroup();
		TransformGroup fakeTG2 = this.createTransformGroup();

		// Computes the position of the fake conceptSphere
		Point3f point = new Point3f(this.positions.get(uri));
		Transform3D currentTransform = new Transform3D();
		this.tgROT.getTransform(currentTransform);
		currentTransform.transform(point);
		Transform3D translation = new Transform3D();
		fakeSpherePosition = new Vector3f(point);
		translation.setTranslation(fakeSpherePosition);

		fakeTG2.setTransform(translation);
		fakeTG.addChild(sphereBG);
		fakeTG2.addChild(fakeTG);
		fakeBG.addChild(fakeTG2);
		if (this.refMainPanel.isPrimary)
			this.refMainPanel.secondaryBG.addChild(fakeBG);
		else
			this.refMainPanel.primaryBG.addChild(fakeBG);
		return fakeTG;
	}

	private Transform3D computeOrientation(Vector3f finalPos) {

		// Define end points as new vectors
		Vector3f fromDir = new Vector3f(1f, 0f, 0f);
		Vector3f toDir = new Vector3f(finalPos);
		toDir.normalize();

		Transform3D resultantTransform = new Transform3D();
		Vector3f translationVector = new Vector3f(finalPos);
		resultantTransform.setTranslation(translationVector);

		if (toDir.equals(fromDir) || toDir.equals(new Vector3f(0f, -1f, 0f))) {
			return resultantTransform;
		}

		// Compute the value of the angle between them (no spatial orientation
		// info though)
		float rotInRadians = fromDir.angle(toDir);

		// Compute an orthagonal vector to later define the rotation axis

		Vector3f orthagonalAxis = new Vector3f();
		orthagonalAxis.cross(fromDir, toDir);
		orthagonalAxis.normalize();

		// Create new rotation matrix
		AxisAngle4f rotAxis = new AxisAngle4f(orthagonalAxis, rotInRadians);

		resultantTransform.setRotation(rotAxis);

		return resultantTransform;
	}
	
	private void drawParents(BranchGroup parentBG, ConceptData data) {
		// Posiziono i padri
		int extra=0;
		if(data.hasAnonymousParent())
			extra=1;
		float radius = this.getSubRadius(data.getNumberOfParents() + extra);

		float alpha = (float) (2f * Math.PI /  (data.getNumberOfParents() + extra) );
		float alpha0 = (float) (Math.PI * Math.random());
		int k = 0;
		for (String parentURI : data.getAllParents()) {
			// Positioning parent node
			float x = (float) (radius * Math.cos(alpha0 + k * alpha));
			float z = (float) (radius * Math.sin(alpha0 + k * alpha));
			Vector3f position = new Vector3f(x, VisualizationOptions.s2_SimpleSphereRadius +
					(VisualizationOptions.s3_TreeLevelHeight/2), z);
			k++;
			
			BranchGroup bgx=this.createBranchGroup();
			// Translates from orig
			TransformGroup tgx = this.createTransformGroup();
			Transform3D transf = new Transform3D();
			transf.setTranslation(position);
			tgx.setTransform(transf);
			this.positions.put(parentURI, position);
			
			// Drawing parent sphere & the isA relation
			ConceptSphere parentSphere = new ConceptSphere(
					VisualizationOptions.s3_ParentSphereColor, parentURI, true,	false);
			parentSphere.setRadius(VisualizationOptions.s3_ParentSphereRadius);
			parentSphere.setLabelColor(VisualizationOptions.s3_LabelParentSphereColor);
			parentSphere.setLabelScale(VisualizationOptions.s3_LabelParentSphereScale);
			BranchGroup parentSphereBG = parentSphere.createColoredSphere();
			tgx.addChild(parentSphereBG);
			this.conceptSpheresBg.put(parentURI, parentSphereBG);
			bgx.addChild(tgx);
			// adds the relation fron current concept to its parent
			Relation3DObject isA = new Relation3DObject();
			isA.setTransparency(VisualizationOptions.s3_RelationTransparency);
			parentBG.addChild(isA.createISARelation(position));
			
			this.conceptsBg.put(parentURI, bgx);
			// adds it to the scene bg
			parentBG.addChild(bgx);
		}
		if(extra==1){
//		 Positioning anonymous parent node
			float x = (float) (radius * Math.cos(alpha0 + k * alpha));
			float z = (float) (radius * Math.sin(alpha0 + k * alpha));
			Vector3f position = new Vector3f(x, VisualizationOptions.s3_SphereRadius+VisualizationOptions.s3_TreeLevelHeight, z);
			Relation3DObject logicalParent=new Relation3DObject();
			logicalParent.setTransparency(VisualizationOptions.s3_RelationTransparency);
			logicalParent.setRelationType(GraphicalLinkTypeEnum.isA);

			parentBG.addChild(logicalParent.createRelationWithRange(null, data.getAnonymousParent(), 
					position, LinkDirectionEnum.out));
		}
	}
	
	private void redraw() {
		this.redraw = true;
		this.tgSCENE.removeAllChildren();
		this.nodes2_BeDrawn.clear();
		this.expandable.clear();
		this.onScene.clear();
		this.conceptsBg.clear();
		this.conceptSpheresBg.clear();
		this.createScene();
		this.redraw = false;
	}
}
