package it.polito.elite.visualization.ontoviewer.userInterfaces;

import it.polito.elite.protege.plugin.OntoSpherePlugin;
import it.polito.elite.visualization.ontoviewer.scenes3D.SceneFocusRoots1;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

public class LogicViewPanel extends JPanel {
	/**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = -1003476137325736416L;
	private OntoSpherePlugin plugin;
	private JTextArea queryText;
	private DefaultListModel resList;
	private JButton execButton;
	private String savedLayout;
	
	public LogicViewPanel (OntoSpherePlugin plugin)
	{
		//take a reference to the plugin
		this.plugin = plugin;
		
		//setting the Panel Layout
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		//this.setBorder(BorderFactory.createLineBorder(Color.black));
	
		/*****************************************
		 * The view management folder title                 *
		 *****************************************/
		//adding the title bar to the query panel
		this.add(this.createHeading("LOGIC VIEWS - MANAGEMENT"), BorderLayout.NORTH);
		
		
		/*****************************************
		 * The  folder contents              *
		 *****************************************/
		//Creating the panel object
		final JPanel contentPanel = new JPanel();
		
		//setting a GridBagLayout for the control panel
		contentPanel.setLayout(new BoxLayout(contentPanel,BoxLayout.Y_AXIS));
		
		//Adding a vertical space from the top
		contentPanel.add(Box.createVerticalStrut(1));		
		this.populateContentPane(contentPanel);
		
		this.add(contentPanel, BorderLayout.CENTER);
		
		JTabbedPane logicViewsComposer=new JTabbedPane();
		
		QueryPanel panelQuery= new QueryPanel(this.plugin);
		ImageIcon iconQuery=new ImageIcon(this.getClass().getClassLoader().getResource("resources/SPARQLlogicViews.gif"));
		logicViewsComposer.addTab("SPARQL Query", iconQuery, panelQuery, "Selects concepts through SPARQL queries (by hand or by wizards)");
		
		SimpleSelectionTreePanel panelTree= new SimpleSelectionTreePanel(this.plugin);
		ImageIcon iconTree=new ImageIcon(this.getClass().getClassLoader().getResource("resources/classes.gif"));
		logicViewsComposer.addTab("Boolean Selection Tree", iconTree, panelTree, "Selects concepts through a simple boolean selection tree viewer");
		
		this.add(logicViewsComposer, BorderLayout.SOUTH);
		//Checks if logicViews Ontology is supported, if not adds it
		this.plugin.viewerPanel.model.confirmLogicViewSupport();
	}
	
	private void populateContentPane(final JPanel contentPanel){
//		creates a combo box with an inpout for each logic view defined in the ontology model
		Set<String> lista=this.plugin.viewerPanel.model.getViews();
		String[] viste=new String[1];
		viste=lista.toArray(viste);
		final JComboBox savedViews=new JComboBox(viste);
//		adding the query input panel
		if(this.plugin.viewerPanel.scene.getSceneID()==1){
			JPanel saveAndRoot = new JPanel(new FlowLayout(FlowLayout.CENTER));
//			Save current View Button
			JButton saveCurrentView = new JButton("Save Current View");
			saveCurrentView.setIcon(new ImageIcon(this.getClass().getClassLoader().getResource("resources/save.gif")));
			final String initialValue=this.plugin.viewerPanel.currentLogicViewName;
			saveCurrentView.addActionListener(new ActionListener()               
	        {                                                         
	            public void actionPerformed(ActionEvent e)               
	            {                                                        
	            	String viewName = (String ) JOptionPane.showInputDialog(contentPanel, "Choose a name..", "SAVE CURRENT LOGIC VIEW AS..", 
	            			JOptionPane.PLAIN_MESSAGE,  null, null, initialValue);
	            	saveLogicView(viewName,plugin.viewerPanel.conceptsOfCurrentLogicView);
	            	savedViews.addItem(viewName);
	            	savedViews.setSelectedItem(viewName);
	            	//salva
	            	
	            }                                                        
	        });         
			saveAndRoot.add(saveCurrentView);
	
//			Load Roots
			JButton loadRoots = new JButton("Load Roots");
			loadRoots.setIcon(new ImageIcon(this.getClass().getClassLoader().getResource("resources/loadRoots.gif")));
			loadRoots.addActionListener(new ActionListener()               
	        {                                                         
	            public void actionPerformed(ActionEvent e)               
	            {                                                        
	            	Set<String> roots=plugin.viewerPanel.model.getRoots();
	            	plugin.viewerPanel.currentLogicViewName = null;
					SceneFocusRoots1 myScene = new SceneFocusRoots1(plugin.viewerPanel, roots);
					plugin.viewerPanel.changeScene(myScene);
	            }                                                        
	        });         
			saveAndRoot.add(loadRoots);
			
			contentPanel.add(saveAndRoot);
		}
// Load/Delete an existing view		
		JPanel viewSelectionPane=new JPanel(new FlowLayout(FlowLayout.CENTER));
		
		JLabel existingViews = new JLabel("Stored Views -> ");
		existingViews.setFont(new Font("Arial",Font.BOLD,13));
		viewSelectionPane.add(existingViews);
		

		if(this.plugin.viewerPanel.currentLogicViewName != null)
			savedViews.setSelectedItem(this.plugin.viewerPanel.currentLogicViewName);
		else
			savedViews.setSelectedIndex(0);
		viewSelectionPane.add(savedViews);
		

		JButton load = new JButton("Load ");
		load.setIcon(new ImageIcon(this.getClass().getClassLoader().getResource("resources/loadScene.gif")));

		load.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String viewName = (String) savedViews.getSelectedItem();
				Set<String> concepts=plugin.viewerPanel.model.getViewElements(viewName);
				plugin.viewerPanel.currentLogicViewName=viewName;
				SceneFocusRoots1 myScene = new SceneFocusRoots1(plugin.viewerPanel, concepts);
				plugin.viewerPanel.changeScene(myScene);
			}
		});
		viewSelectionPane.add(load);
		
		JButton delete = new JButton(" Delete ");
		delete.setIcon(new ImageIcon(this.getClass().getClassLoader().getResource("resources/delScene.gif")));

		delete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String viewName = (String) savedViews.getSelectedItem();
				deleteLogicView(viewName);
				savedViews.removeItem(viewName);
			}
		});
		viewSelectionPane.add(delete);
		
		contentPanel.add(viewSelectionPane);
	}
	
	/**
	 * This method allows the creation of the title bar for the query tab of the 
	 * ontosphere3D control interface. It returns the title bar as a JPanel.
	 * @return the title bar as a JPanel
	 */
	private JPanel createHeading(String title)
	{
		//setting the title
		JLabel panelTitle = new JLabel(title);
		
		//setting the font
		Font labelFont = new Font ("SansSerif",Font.BOLD,12);
		panelTitle.setFont(labelFont);
		
		//setting the foreground color
		panelTitle.setForeground(Color.WHITE);
		
		//loading the panel icon (a circle with a question mark inside)
		JLabel panelIcon = new JLabel(new ImageIcon(this.getClass().getClassLoader().getResource("resources/logicViews.gif")));
		
		//creating the container for the title bar
		JPanel titleContainer = new JPanel();
		
		//setting the container layout
		titleContainer.setLayout(new FlowLayout());
		
		//setting the container background color
		titleContainer.setBackground(Color.LIGHT_GRAY);
		
		//adding the title bar components
		titleContainer.add(panelIcon);
		titleContainer.add(panelTitle);
		
		return titleContainer;
	}
	
	private void deleteLogicView(String viewName){
		this.plugin.viewerPanel.model.deleteView(viewName);
	}
	
	final private void saveLogicView(String viewName, HashSet<String> set){
		this.plugin.viewerPanel.model.saveViewElements(viewName, set);
		this.plugin.viewerPanel.currentLogicViewName = new String(viewName);
		this.plugin.viewerPanel.scene.addSceneLabel(viewName+" view");
	}
}
