package it.polito.elite.visualization.ontoviewer.userInterfaces;

import java.awt.Color;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Scanner;
import java.util.StringTokenizer;

import javax.vecmath.Color3f;


public class VisualizationOptions {
	
	static public final String fname="VisualizationOptions.txt";
	
	/**
	 * Setting for elements of global relevance 
	 */
	static public float sG_distance_min=.08f;
	static public String descriptionOf_sG_distance_min = 
		"Represents the mininum required distance between 2 elements on screen"; 
	static public float sG_instanceShellRadiusMultiplier=1.4f;
	static public String descriptionOf_sG_instanceShellRadiusMultiplier=
		"Represents how many times the \"aura\" representing the presence" +
		"of direct instances should be bigger than the related concept sphere";
	static public float sG_InstanceShellTransparency=.65f;
	static public String descriptionOf_sG_InstanceShellTransparency=
		"Represents the level of transparency associated with the \"aura\" " +
		"representing representing the presence of direct instances";
	static public  String sG_FontFamily="Arial";
	static public String descriptionOf_sG_FontFamily=
		"Represents the font family used for writing labels";
	//The font modifier used for writing the label
	static public  int sG_FontModifier=Font.BOLD;
	static public String descriptionOf_sG_FontModifier=
		"Represents the font modifier used for writing labels, values from constant" +
		" fields of java.awt.Font";
	static public Color3f sG_background=new Color3f(0.7411765f, 0.81960785f, 0.84313726f);
	static public String descriptionOf_sG_background=
		"Represents the background color of scenes";
	static public boolean sG_useAnimations=true;
	static public String descriptionOf_sG_useAnimations=
		"Tells whether the transitions between the different scenes should be animated or not";
	static public int sG_AnimationDuration=2000;
	static public String descriptionOf_sG_AnimationDuration=
		"Duration of transition animations in milliseconds";
	static public String s_LogicViewsOntology="http://ontosphere3d.sourceforge.net/LogicViews.owl#"; 
	static public String descriptionOf_s_LogicViewsOntology=
		"URI of the ontology that models the annotation property belongsToView, necessary to define and use logic views";
	static public String s_LogicViewsOntologyPrefix="lw";
	static public String descriptionOf_s_LogicViewsOntologyPrefix=
		"prefix for the logicViews ontology NameSpace";
	
	/**
	 * Setting for elements related to relations 
	 */
	static public float sR_coneRadius = .025f;
	static public String descriptionOf_sR_coneRadius=
		"Represents the base radius for the cone used as arrow-head";
	static public float sR_coneHeight = 0.045f;
	static public String descriptionOf_sR_coneHeight=
		"Represents the base height for the cone used as arrow-head";
	static public Color3f sR_MultiRelationArrowHeadColor = new Color3f(Color.LIGHT_GRAY);
	static public String descriptionOf_sR_MultiRelationArrowHeadColor=
		"Specifies the sphere color for an arrow head cone representing a set of relations";
	static public float sR_MultiRelationRadiusLimit=0.05f;
	static public String descriptionOf_sR_MultiRelationRadiusLimit=
		"Represent the maximum size of the cone radius associated to a relations set";
	static public float sR_MultiRelationHeightLimit=0.1f;
	static public String descriptionOf_sR_MultiRelationHeightLimit=
		"Represent the maximum size of the cone radius associated to a relations set";
	static public float sR_MultiRelationMultiplier=0.1f;
	static public String descriptionOf_sR_MultiRelationMultiplier=
		"The radius and the height of arrow head cones associated to a relations set increases " +
		"proportionally with the size of the set. This values represents such " +
		"increase factor";
	static public float	sR_sphereRadius = .02f;
	static public String descriptionOf_sR_sphereRadius=
		"Represents the radius for the sphere used as ending point in semantic relations " +
		"to a concept not directly on the scene";
	static public float sR_cubeSide = .02f;
	static public String descriptionOf_sR_cubeSide=
		"Represents the side for the cube used as ending point in semantic relations " +
		"to an instance not directly on the scene";
	static public float sR_dataTypeCylinderHeight = .04f;
	static public String descriptionOf_sR_dataTypeCylinderHeight=
		"Represents the height of the cylinder used as ending point in datatype relations";
	static public float sR_dataTypeCylinderRadius =  .020f;
	static public String descriptionOf_sR_dataTypeCylinderRadius=
		"Represents the radius of the cylinder used as ending point in datatype relations";
	static public float sR_selfLoopRadiusMultiplier = 1.1f;
	static public String descriptionOf_sR_selfLoopRadiusMultiplier=
		"Represents how many times the self-loops relation should be bigger " +
		"than the related concept sphere radius (for instances the side of the cube is used)";	
	static public float sR_selfLoopverticalMultiplier = 0.588f;
	static public String descriptionOf_sR_selfLoopverticalMultiplier=
		"Self loop relations are translated towards height by a an amount of space equals " +
		" to the concept sphere radius multiplied by this scale factor";
	static public float sR_cylinderRadius = 0.005f;
	static public String descriptionOf_sR_cylinderRadius=
		"Represents the thickness of relations lines";
	static public float sR_boxedArrowMultiplier = 1.3f;
	static public String descriptionOf_sR_boxedArrowMultiplier=
		"Represents the increase of thickness of cardinality constrained relations";
	static public float sR_placeHolderLabelScale= .011f;
	static public String descriptionOf_sR_placeHolderLabelScale=
		"Represents the scale factor of labels relative to as ending point " +
		"objects in semantic or datatype relations";
	static public float sR_RelationsLabeScale = .013f;
	static public String descriptionOf_sR_RelationsLabeScale=
		"Represents the scale factor of labels relative to relations";
	static public float sR_LabelPositionOffset = .01f;
	static public String descriptionOf_sR_LabelPositionOffset=
		"Specifies the distance between a relation label and the arrow head cone";
	static public boolean sR_DisplayRelationsLabels=true;
	static public String descriptionOf_sR_DisplayRelationsLabels=
		"Tells wether the labels of relations should be visualized or not";
	static public float sR_ArrowHeadTransparencyMultiplier = 0.6f;
	static public String descriptionOf_sR_ArrowHeadTransparencyMultiplier=
		"Represents a multiplier factor that adjusts the transparency of arrow head cones, " +
		"if its pertaining line has a transparency attribute set";
	static public float sR_InheritedRelationDistanceMultiplier= 1.25f;
	static public String descriptionOf_sR_InheritedRelationDistanceMultiplier=
		"Represents a multiplier factor that adjusts the lenght of inherited relations";
	static public boolean sR_ShowRelationTowardsNodeNotOnScreen= true;
	static public String descriptionOf_sR_ShowRelationTowardsNodeNotOnScreen=
		"Tells wether to show or not relations towards elements not on screen in scene 2 (focus on a branch)";
	static public Color3f sR_IncomingRelationColor = new Color3f(0,.8f,0);
	static public String descriptionOf_sR_IncomingRelationColor=
		"Specifies the color for incoming relations";
	static public Color3f sR_OutgoingRelationColor = new Color3f(.8f,0,0);
	static public String descriptionOf_sR_OutgoingRelationColor=
		"Specifies the color for outgoing relations";
	static public Color3f sR_DatatypeRelationColor = new Color3f(0,0,.8f);
	static public String descriptionOf_sR_DatatypeRelationColor=
		"Specifies the color for datatype relations";
	/**
	 * Setting for elements related to scene 1: the Main Scene
	 */
	static public Color3f s1_LabelSimpleSphereColor = new Color3f(Color.BLUE);
	static public String descriptionOf_s1_LabelSimpleSphereColor=
		"Specifies the label color for a leaf concept";
	static public Color3f s1_LabelParentSphereColor = new Color3f(Color.RED);
	static public String descriptionOf_s1_LabelParentSphereColor=
		"Specifies the label color for a concept containing a subtree";
	static public Color3f s1_MainSphereColor = new Color3f(.352f, .745f, .9f);
	static public String descriptionOf_s1_MainSphereColor=
		"Specifies the color of the earth-like sphere where concepts are placed";
	static public float s1_MainSphereTransparency = .5f;
	static public String descriptionOf_s1_MainSphereTransparency=
		"Represent the transparency level of the earth-like sphere bearing the concepts";
	static public Color3f s1_SimpleSphereColor = new Color3f(Color.BLUE);
	static public String descriptionOf_s1_SimpleSphereColor=
		"Specifies the sphere color for a leaf concept";
	static public Color3f s1_ParentSphereColor = new Color3f(Color.LIGHT_GRAY);
	static public String descriptionOf_s1_ParentSphereColor=
		"Specifies the sphere color for a concept containing a subtree";
	static public float s1_LabelParentSphereScale=.016f;
	static public String descriptionOf_s1_LabelParentSphereScale=
		"Represents the scale factor of labels relative to parent concepts";
	static public float s1_LabelSimpleSphereScale=.013f;
	static public String descriptionOf_s1_LabelSimpleSphereScale=
		"Represents the scale factor of labels relative to leaf concepts";
	static public float s1_ParentSphereRadiusLimit=0.08f;
	static public String descriptionOf_s1_ParentSphereRadiusLimit=
		"Represent the maximum size of the sphere radius associated to a parent concept";
	static public float s1_ParentSphereRadiusMultiplier=.005f;
	static public String descriptionOf_s1_ParentSphereRadiusMultiplier=
		"The radius of spheres related to parent conceps increases proportionally with " +
		"the number of concepts present in their subtree. This values represents the radius " +
		"increase factor";
	static public float s1_SimpleSphereRadius=.04f;
	static public String descriptionOf_s1_SimpleSphereRadius=
		"It specifies the radius of a leaf concept sphere";
	static public float s1_Segment2ArcCorrectionFactor=2f;
	static public String descriptionOf_s1_Segment2ArcCorrectionFactor=
		"The radius of earth-like sphere is proportional to the number of elements " +
		"it bears on its surface. Increasing this factor results in increasing such sphere." +
		" This value is used also for the sphere bearing instances";
	static public int s_ExpansionAnimationOverlapTime=200;
	/**
	 * Setting for elements related to scene 2: the Hierarchy Browsing Scene
	 */
	static public Color3f s2_LabelSimpleSphereColor = new Color3f(Color.BLACK);
	static public String descriptionOf_s2_LabelSimpleSphereColor=
		"Specifies the label color for a concept";
	static public Color3f s2_LabelParentSphereColor = new Color3f(Color.RED);
	static public String descriptionOf_s2_LabelParentSphereColor=
		"Specifies the label color for a concept containing a subtree";
	static public Color3f s2_ParentSphereColor = new Color3f(Color.LIGHT_GRAY);
	static public String descriptionOf_s2_ParentSphereColor=
		"Specifies the sphere color for a collapsed concept node containing a subtree";
	static public float s2_LabelParentSphereScale=.016f;
	static public String descriptionOf_s2_LabelParentSphereScale=
		"Represents the scale factor of labels relative to a collapsed concept node containing a subtree";
	static public float s2_LabelSimpleSphereScale=.013f;
	static public String descriptionOf_s2_LabelSimpleSphereScale=
		"Represents the scale factor of labels relative to a concepts";
	static public float s2_FamilyRadiusMultiplier=2.10f;
	static public String descriptionOf_s2_FamilyRadiusMultiplier=
		"Represents a factor for increasing/decreasing the radius of siblings circles";
	static public float s2_ParentSphereRadiusLimit=0.08f;
	static public String descriptionOf_s2_ParentSphereRadiusLimit=
		"Represents the maximum size of the sphere radius associated to a collapsed concept node containing a subtree";
	static public float s2_ParentSphereRadiusMultiplier=.005f;
	static public String descriptionOf_s2_ParentSphereRadiusMultiplier=
		"The radius of spheres related to parent conceps increases proportionally with " +
		"the number of concepts present in their subtree. This values represents the radius " +
		"increase factor";
	static public float s2_SimpleSphereRadius=.05f;
	static public String descriptionOf_s2_SimpleSphereRadius=
		"It specifies the radius of a concept sphere";
	static public float s2_Segment2ArcCorrectionFactor=1.2f;
	static public String descriptionOf_s2_Segment2ArcCorrectionFactor=
		"The radius of circles of siblings is proportional to their number. " +
		"Increasing this factor results in increasing such circle. This value is also used " +
		"for the dependency tree scene";
	static public float s2_TreeLevelHeight=.35f;
	static public String descriptionOf_s2_TreeLevelHeight=
		"Specifies the vertical distance between elements belonging to 2 consecutive levels";
	static public float s2_CurvedArrowCurvature = 0.2f;
	static public String descriptionOf_s2_CurvedArrowCurvature=
		"Specifies the curvature factor (the higher is the value the wider is the curve) " +
		"of semantic relations between parent and children";
	static public float s2_ScreenPercent_as_RootVerticalPosition = 0.2f;
	static public String descriptionOf_s2_ScreenPercent_as_RootVerticalPosition=
		"Vertically places the root element of the hierarchy at a given percentage of the " +
		"screen height";
	static public float s2_PercentageOfScreen_NotVisibleButValid =0.07f;
	static public String descriptionOf_s2_PercentageOfScreen_NotVisibleButValid=
		"Represents the dimension of region outside the screen where is allowed "+
		"to initially place nodes. If they exceed their subtree is automatically "+
		"collapsed into the parent";
	static public float s2_RelationOutOfTree_DistanceMultiplier =2.5f;
	static public String descriptionOf_s2_RelationOutOfTree_DistanceMultiplier=
		"Specifies how much a relation towards an element that is not part of the  tree "+
		" currently explored should be distant from the originating concept";

	/**
	 * Setting for elements related to scene 3: the Concept Focus Scene
	 */
	static public Color3f s3_LabelChildSphereColor = new Color3f(Color.BLACK);
	static public String descriptionOf_s3_LabelChildSphereColor=
		"Specifies the label color for a child concept sphere";
	static public Color3f s3_LabelParentSphereColor = new Color3f(Color.BLUE);
	static public String descriptionOf_s3_LabelParentSphereColor=
		"Specifies the label color for the parent(s) of the concept at the center of the scene";
	static public Color3f s3_ChildSphereColor = new Color3f(Color.MAGENTA);
	static public String descriptionOf_s3_ChildSphereColor=
		"Specifies the color for a child concept sphere";
	static public Color3f s3_ParentSphereColor = new Color3f(Color.YELLOW);
	static public String descriptionOf_s3_ParentSphereColor=
		"Specifies the color for the parent(s) of the concept at the center of the scene";
	static public Color3f s3_RootSphereColor = new Color3f(Color.ORANGE);
	static public String descriptionOf_s3_RootSphereColor=
		"Specifies the color for the ancestor root(s) of the concept at the center of the scene";
	static public float s3_LabelChildSphereScale=.016f;
	static public String descriptionOf_s3_LabelChildSphereScale=
		"Represents the scale factor of labels relative to child concepts";
	static public float s3_LabelParentSphereScale=.013f;
	static public String descriptionOf_s3_LabelParentSphereScale=
		"Represents the scale factor of labels relative to the parent(s) of the concept at the center of the scene";
	static public float s3_InheritedRadiusMultiplier =1.3f;
	static public String descriptionOf_s3_InheritedRadiusMultiplier=
		"Specifies how much an inherited semantic relation should be more distant " +
		"from the center with respect to a direct relation (if both are displayed concurrently)";
	static public float s3_InheritedElmTransparency = 0.6f;
	static public String descriptionOf_s3_InheritedElmTransparency=
		"Specifies the transparency level for inherited relations";
	static public float s3_InheritedSelfLoopMultiplier=1.4f;
	static public String descriptionOf_s3_InheritedSelfLoopMultiplier=
		"Specifies how much the radius of an inherited self loop relation should be bigger " +
		"than a direct one";
	static public float s3_SemRelationHeigth=.185f;
	static public String descriptionOf_s3_SemRelationHeigth=
		"Specifies the level of height for Incoming and Datatype relations";
	static public float s3_ParentSphereRadius=.015f;
	static public String descriptionOf_s3_ParentSphereRadius=
		"Specifies the ratio between the size of a parent sphere w.r.t the central one";
	static public float s3_ChildSphereRadius=.015f;
	static public String descriptionOf_s3_ChildSphereRadius=
		"Specifies the ratio between the size of a child sphere w.r.t the central one";
	static public float s3_SphereRadius=.1f;
	static public String descriptionOf_s3_SphereRadius=
		"Specifies the radius of the concept sphere at the center of the scene";
	static public float s3_Segment2ArcCorrectionFactor=1.1f;
	static public String descriptionOf_s3_Segment2ArcCorrectionFactor=
		"The radius of circles of siblings is proportional to their number. " +
		"Increasing this factor results in increasing such circle.";
	static public float s3_RelationTransparency = .4f;
	static public String descriptionOf_s3_RelationTransparency=
		"Specifies the transparency level of the isA relations " +
		"(the same value is used for the sphere of parent(s) and root(s)";
	static public float s3_TreeLevelHeight=.25f;
	static public String descriptionOf_s3_TreeLevelHeight=
		"Specifies the vertical distance between elements belonging to 2 consecutive levels";
	static public float s3_SemanticRelationDistanceMultiplier=3f;
	static public String descriptionOf_s3_SemanticRelationDistanceMultiplier=
		"Regulates the lenght of semantic relations";
	static public boolean s3_ShowChildern=true;
	static public String descriptionOf_s3_ShowChildern=
		"Boolean value telling if default visualization starts with the children of the selected node" +
		" (if present) or with logical disjointness of the class (if specified for that class)";
	
	/**
	 * Setting for elements related to scene 4: Visualizing instances
	 */
	static public Color3f s4_MainSphereColor= new Color3f(.352f, .745f, .9f);
	static public String descriptionOf_s4_MainSphereColor=
		"Specifies the color of the earth-like sphere where instances are placed";
	static public float s4_cubeSide= .04f;
	static public String descriptionOf_s4_cubeSide=
		"Specifies the side of an instance cube";
	static public Color3f s4_InstanceWithFactsColor = new Color3f(Color.LIGHT_GRAY);
	static public String descriptionOf_s4_InstanceWithFactsColor=
		"Specifies the sphere color for an instance involved in a net of facts";
	static public Color3f s4_InstanceWithoutFactsColor = new Color3f(Color.BLUE);
	static public String descriptionOf_s4_InstanceWithoutFactsColor=
		"Specifies the sphere color for an instance not involved in a net of facts";
	static public Color3f s4_LabelInstanceNoChild = new Color3f(Color.BLACK);
	static public String descriptionOf_s4_LabelInstanceNoChild=
		"Specifies the label color for an instance not involved in any fact";
	static public Color3f s4_LabelInstanceWithChildren = new Color3f(Color.RED);
	static public String descriptionOf_s4_LabelInstanceWithChildren=
		"Specifies the label color for the parent(s) of the concept at the center of the scene";
	static public float s4_LabelInstanceScale=.012f;
	static public String descriptionOf_s4_LabelInstanceScale=
		"Represents the scale factor of labels relative to instances";
	static public float s4_InstanceSideLimit=0.1f;
	static public String descriptionOf_s4_InstanceSideLimit=
		"Represent the maximum size of the cube side associated to an instance having facts";
	static public float s4_InstanceSideMultiplier=.005f;
	static public String descriptionOf_s4_InstanceSideMultiplier=
		"The side of cubes related to instances increases proportionally with " +
		"the number of facts involving them. This values represents the side " +
		"increase factor";
	
	/**
	 * Setting for elements related to scene 5: Facts Focus
	 */
	static public float s5_cubeSide= .06f;
	static public String descriptionOf_s5_cubeSide=
		"Specifies the side of an instance cube";
	static public float s5_ConceptHeight = .3f;
	static public String descriptionOf_s5_ConceptHeight=
		"Defines the vertical distance between the instance and the concept sphere "+
		"representing the pertaining concept";
	static public float s5_ConceptSphereRadius= .02f;
	static public String descriptionOf_s5_ConceptSphereRadius=
		"Represents the radius value for the pertaining concept sphere";
	static public Color3f s5_ConceptSphereColor = new Color3f(Color.YELLOW);
	static public String descriptionOf_s5_ConceptSphereColor=
		"Represents the color of the concept sphere";
	static public float s5_Rel2ConceptTransparency =.4f;
	static public String descriptionOf_s5_Rel2ConceptTransparency=
		"Represent the transparency level of the relation towards the pertaining concept sphere";
	static public float s5_SemRelationHeigth=.185f;
	static public String descriptionOf_s5_SemRelationHeigth="Specifies the level of height for Incoming and Datatype relations";
	/**
	 * Setting for elements related to scene 6: Dependency Tree
	 */
	static public float s6_CurvedArrowCurvature = 0.3f;
	static public String descriptionOf_s6_CurvedArrowCurvature=
		"Specifies the curvature factor (the higher is the value the wider is the curve) " +
		"of semantic relations between parent and children";
	static public float s6_ScreenPercent_as_RootVerticalPosition = 0.2f;
	static public String descriptionOf_s6_ScreenPercent_as_RootVerticalPosition=
		"Vertically places the root element of the hierarchy at a given percentage of the " +
		"screen height";
	static public float s6_FamilyRadiusMultiplier=1.9f;
	static public String descriptionOf_s6_FamilyRadiusMultiplier=
		"Represents a factor for increasing/decreasing the radius of siblings circles";

	
	static public float conceptSphereTransparency=.0f;
	static public float conceptSphereRadius=.04f;
	// Color of the cylinder forming the "line"
	static public Color3f colorLine = new Color3f(Color.DARK_GRAY);
	static public Color3f labelConceptSphereColor = new Color3f(.0f, .0f, .0f);
	//scale factor to render on the scene the label that is originally
	//created using a pointsize of 2 (see awt font & java3D font3D classes for details)
	static public float defaultLabelSphereScale=.014f;
	static public float defaultLabelCubeScale=.014f;
	static public float defaultLabelScale=.014f;
	// label font modifier
	static public  int fontModifier = Font.PLAIN;

	private static int colorCounter=-1;
	
	/**
	 * It loads previously saved visualization preferences from a internal configuration file of the project  
	 *
	 */
	/**
	 * It loads previously saved visualization preferences from a internal configuration file of the project  
	 *
	 */
	public static void initializeVisualizationsOptions(){
		try {
			File f=new File(VisualizationOptions.fname);
			if(!f.exists())
				return;
			
			Class options = VisualizationOptions.class;
			BufferedReader in = new BufferedReader(new FileReader(f));
			String s;
			while( (s=in.readLine()) != null){
				String propertyName, type, methodName, classname, value;
				//reads the property name
				Scanner scan=new Scanner(s);
				propertyName=scan.next();
				methodName=new String("setS"+propertyName.substring(1));
				type=scan.next();
				
				//Parameter type & value to pass to the setXX method 
				Class[] classes=new Class[1];
				Object[] objs=new Object[1];

				if(type.equals("float")){
					classes[0] = java.lang.Float.TYPE;
					value=scan.next();
					objs[0]= new Float(value);
				}
				else if(type.equals("int")){
					classes[0] = java.lang.Integer.TYPE;
					value=scan.next();
					objs[0]= new Integer(value);
				}
				else if(type.equals("class")){
					classname=scan.next();
					if(classname.equals("javax.vecmath.Color3f")){
						classes[0] = Color3f.class;
						value=scan.nextLine().trim();
						value=value.substring(1,value.length()-1);
						float x,y,z;
						StringTokenizer paramTokenizer=new StringTokenizer(value,", ");
						String token=paramTokenizer.nextToken();
						x=Float.parseFloat(token);
						token=paramTokenizer.nextToken();
						y=Float.parseFloat(token);
						token=paramTokenizer.nextToken();
						z=Float.parseFloat(token);
						Color3f color=new Color3f(x,y,z);
						objs[0]= color;
					}
					else if(classname.equals("java.lang.String")){
						classes[0] = String.class;
						value=scan.next();
						objs[0]= new String(value);
					}
				}
				
				try {
					Method metodo = options.getMethod(methodName,classes);
					metodo.invoke(options,objs);
				} catch (Exception e1) {
					e1.printStackTrace();
				} 
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	static public void saveVisualizationOptions(){
		try{
		PrintWriter fw=new PrintWriter(new File(VisualizationOptions.fname));
		Class options = VisualizationOptions.class;
		Field[] fields = options.getFields();
		for(Field f :fields){
			if(!f.getName().matches("s._.*"))
				continue;
			if (f.getType() == java.lang.Float.TYPE) {
				fw.println(f.getName()+" "+f.getType()+" "+f.getFloat(options));
			}
			else if(f.getType() == java.lang.Integer.TYPE){
				fw.println(f.getName()+" "+f.getType()+" "+f.getInt(options));
			}
			else if(f.getType() == java.lang.String.class){
				fw.println(f.getName()+" "+f.getType()+" "+f.get(options));
			}
			else if(f.getType() == Color3f.class){
				fw.println(f.getName()+" "+f.getType()+" "+f.get(options));
			}
		}
		fw.flush();
		fw.close();
		System.out.println("visualization options saved...");
		}catch(Exception e){e.printStackTrace();}
	}
	
	/**
	 * Utility methods for assigning colors to spheres
	 * using their level as decision measure.
	 * @param k The concept's level
	 * @return the color
	 */
	public static Color3f chooseColor(int k) {
		Color3f color = null;
		int k2 = k % 5;
		switch (k2) {
		case 0:
			color = new Color3f(java.awt.Color.DARK_GRAY);
			break;
		case 1:
			color = new Color3f(java.awt.Color.BLUE);
			break;
		case 2:
			color = new Color3f(java.awt.Color.YELLOW);
			break;
		case 3:
			color = new Color3f(java.awt.Color.MAGENTA);
			break;
		case 4:
			color = new Color3f(java.awt.Color.CYAN);
			break;
		}
		return color;
	}
	
	public static Color3f chooseColor() {
		return chooseColor(++colorCounter);
	}

	/**
	 * @param labelParentSphereColor The s1_LabelParentSphereColor to set.
	 */
	public static void setS1_LabelParentSphereColor(Color3f labelParentSphereColor) {
		s1_LabelParentSphereColor = labelParentSphereColor;
	}

	/**
	 * @param labelParentSphereScale The s1_LabelParentSphereScale to set.
	 */
	public static void setS1_LabelParentSphereScale(float labelParentSphereScale) {
		s1_LabelParentSphereScale = labelParentSphereScale;
	}

	/**
	 * @param labelSimpleSphereColor The s1_LabelSimpleSphereColor to set.
	 */
	public static void setS1_LabelSimpleSphereColor(Color3f labelSimpleSphereColor) {
		s1_LabelSimpleSphereColor = labelSimpleSphereColor;
	}

	/**
	 * @param labelSimpleSphereScale The s1_LabelSimpleSphereScale to set.
	 */
	public static void setS1_LabelSimpleSphereScale(float labelSimpleSphereScale) {
		s1_LabelSimpleSphereScale = labelSimpleSphereScale;
	}

	/**
	 * @param mainSphereColor The s1_MainSphereColor to set.
	 */
	public static void setS1_MainSphereColor(Color3f mainSphereColor) {
		s1_MainSphereColor = mainSphereColor;
	}

	/**
	 * @param mainSphereTransparency The s1_MainSphereTransparency to set.
	 */
	public static void setS1_MainSphereTransparency(float mainSphereTransparency) {
		s1_MainSphereTransparency = mainSphereTransparency;
	}

	/**
	 * @param parentSphereColor The s1_ParentSphereColor to set.
	 */
	public static void setS1_ParentSphereColor(Color3f parentSphereColor) {
		s1_ParentSphereColor = parentSphereColor;
	}

	/**
	 * @param parentSphereRadiusLimit The s1_ParentSphereRadiusLimit to set.
	 */
	public static void setS1_ParentSphereRadiusLimit(float parentSphereRadiusLimit) {
		s1_ParentSphereRadiusLimit = parentSphereRadiusLimit;
	}

	/**
	 * @param parentSphereRadiusMultiplier The s1_ParentSphereRadiusMultiplier to set.
	 */
	public static void setS1_ParentSphereRadiusMultiplier(
			float parentSphereRadiusMultiplier) {
		s1_ParentSphereRadiusMultiplier = parentSphereRadiusMultiplier;
	}

	/**
	 * @param segment2ArcCorrectionFactor The s1_Segment2ArcCorrectionFactor to set.
	 */
	public static void setS1_Segment2ArcCorrectionFactor(
			float segment2ArcCorrectionFactor) {
		s1_Segment2ArcCorrectionFactor = segment2ArcCorrectionFactor;
	}

	/**
	 * @param simpleSphereColor The s1_SimpleSphereColor to set.
	 */
	public static void setS1_SimpleSphereColor(Color3f simpleSphereColor) {
		s1_SimpleSphereColor = simpleSphereColor;
	}

	/**
	 * @param simpleSphereRadius The s1_SimpleSphereRadius to set.
	 */
	public static void setS1_SimpleSphereRadius(float simpleSphereRadius) {
		s1_SimpleSphereRadius = simpleSphereRadius;
	}

	/**
	 * @param curvedArrowCurvature The s2_CurvedArrowCurvature to set.
	 */
	public static void setS2_CurvedArrowCurvature(float curvedArrowCurvature) {
		s2_CurvedArrowCurvature = curvedArrowCurvature;
	}

	/**
	 * @param familyRadiusMultiplier The s2_FamilyRadiusMultiplier to set.
	 */
	public static void setS2_FamilyRadiusMultiplier(float familyRadiusMultiplier) {
		s2_FamilyRadiusMultiplier = familyRadiusMultiplier;
	}

	/**
	 * @param labelParentSphereColor The s2_LabelParentSphereColor to set.
	 */
	public static void setS2_LabelParentSphereColor(Color3f labelParentSphereColor) {
		s2_LabelParentSphereColor = labelParentSphereColor;
	}

	/**
	 * @param labelParentSphereScale The s2_LabelParentSphereScale to set.
	 */
	public static void setS2_LabelParentSphereScale(float labelParentSphereScale) {
		s2_LabelParentSphereScale = labelParentSphereScale;
	}

	/**
	 * @param labelSimpleSphereColor The s2_LabelSimpleSphereColor to set.
	 */
	public static void setS2_LabelSimpleSphereColor(Color3f labelSimpleSphereColor) {
		s2_LabelSimpleSphereColor = labelSimpleSphereColor;
	}

	/**
	 * @param labelSimpleSphereScale The s2_LabelSimpleSphereScale to set.
	 */
	public static void setS2_LabelSimpleSphereScale(float labelSimpleSphereScale) {
		s2_LabelSimpleSphereScale = labelSimpleSphereScale;
	}

	/**
	 * @param parentSphereColor The s2_ParentSphereColor to set.
	 */
	public static void setS2_ParentSphereColor(Color3f parentSphereColor) {
		s2_ParentSphereColor = parentSphereColor;
	}

	/**
	 * @param parentSphereRadiusLimit The s2_ParentSphereRadiusLimit to set.
	 */
	public static void setS2_ParentSphereRadiusLimit(float parentSphereRadiusLimit) {
		s2_ParentSphereRadiusLimit = parentSphereRadiusLimit;
	}

	/**
	 * @param parentSphereRadiusMultiplier The s2_ParentSphereRadiusMultiplier to set.
	 */
	public static void setS2_ParentSphereRadiusMultiplier(
			float parentSphereRadiusMultiplier) {
		s2_ParentSphereRadiusMultiplier = parentSphereRadiusMultiplier;
	}

	/**
	 * @param percentageOfScreen_NotVisibleButValid The s2_PercentageOfScreen_NotVisibleButValid to set.
	 */
	public static void setS2_PercentageOfScreen_NotVisibleButValid(
			float percentageOfScreen_NotVisibleButValid) {
		s2_PercentageOfScreen_NotVisibleButValid = percentageOfScreen_NotVisibleButValid;
	}

	/**
	 * @param screenPercent_as_RootVerticalPosition The s2_ScreenPercent_as_RootVerticalPosition to set.
	 */
	public static void setS2_ScreenPercent_as_RootVerticalPosition(
			float screenPercent_as_RootVerticalPosition) {
		s2_ScreenPercent_as_RootVerticalPosition = screenPercent_as_RootVerticalPosition;
	}

	/**
	 * @param segment2ArcCorrectionFactor The s2_Segment2ArcCorrectionFactor to set.
	 */
	public static void setS2_Segment2ArcCorrectionFactor(
			float segment2ArcCorrectionFactor) {
		s2_Segment2ArcCorrectionFactor = segment2ArcCorrectionFactor;
	}

	/**
	 * @param simpleSphereRadius The s2_SimpleSphereRadius to set.
	 */
	public static void setS2_SimpleSphereRadius(float simpleSphereRadius) {
		s2_SimpleSphereRadius = simpleSphereRadius;
	}

	/**
	 * @param treeLevelHeight The s2_TreeLevelHeight to set.
	 */
	public static void setS2_TreeLevelHeight(float treeLevelHeight) {
		s2_TreeLevelHeight = treeLevelHeight;
	}

	/**
	 * @param childSphereColor The s3_ChildSphereColor to set.
	 */
	public static void setS3_ChildSphereColor(Color3f childSphereColor) {
		s3_ChildSphereColor = childSphereColor;
	}

	/**
	 * @param childSphereRadius The s3_ChildSphereRadius to set.
	 */
	public static void setS3_ChildSphereRadius(
			float childSphereRadius) {
		s3_ChildSphereRadius = childSphereRadius;
	}

	/**
	 * @param inheritedElmTransparency The s3_InheritedElmTransparency to set.
	 */
	public static void setS3_InheritedElmTransparency(float inheritedElmTransparency) {
		s3_InheritedElmTransparency = inheritedElmTransparency;
	}

	/**
	 * @param inheritedRadiusMultiplier The s3_InheritedRadiusMultiplier to set.
	 */
	public static void setS3_InheritedRadiusMultiplier(
			float inheritedRadiusMultiplier) {
		s3_InheritedRadiusMultiplier = inheritedRadiusMultiplier;
	}

	/**
	 * @param inheritedSelfLoopMultiplier The s3_InheritedSelfLoopMultiplier to set.
	 */
	public static void setS3_InheritedSelfLoopMultiplier(
			float inheritedSelfLoopMultiplier) {
		s3_InheritedSelfLoopMultiplier = inheritedSelfLoopMultiplier;
	}

	/**
	 * @param labelChildSphereColor The s3_LabelChildSphereColor to set.
	 */
	public static void setS3_LabelChildSphereColor(Color3f labelChildSphereColor) {
		s3_LabelChildSphereColor = labelChildSphereColor;
	}

	/**
	 * @param labelChildSphereScale The s3_LabelChildSphereScale to set.
	 */
	public static void setS3_LabelChildSphereScale(float labelChildSphereScale) {
		s3_LabelChildSphereScale = labelChildSphereScale;
	}

	/**
	 * @param labelParentSphereColor The s3_LabelParentSphereColor to set.
	 */
	public static void setS3_LabelParentSphereColor(Color3f labelParentSphereColor) {
		s3_LabelParentSphereColor = labelParentSphereColor;
	}

	/**
	 * @param labelParentSphereScale The s3_LabelParentSphereScale to set.
	 */
	public static void setS3_LabelParentSphereScale(float labelParentSphereScale) {
		s3_LabelParentSphereScale = labelParentSphereScale;
	}

	/**
	 * @param parentSphereColor The s3_ParentSphereColor to set.
	 */
	public static void setS3_ParentSphereColor(Color3f parentSphereColor) {
		s3_ParentSphereColor = parentSphereColor;
	}

	/**
	 * @param parentSphereRadius The s3_ParentSphereRadius to set.
	 */
	public static void setS3_ParentSphereRadius(
			float parentSphereRadius) {
		s3_ParentSphereRadius = parentSphereRadius;
	}

	/**
	 * @param relationTransparency The s3_RelationTransparency to set.
	 */
	public static void setS3_RelationTransparency(float relationTransparency) {
		s3_RelationTransparency = relationTransparency;
	}

	/**
	 * @param rootSphereColor The s3_RootSphereColor to set.
	 */
	public static void setS3_RootSphereColor(Color3f rootSphereColor) {
		s3_RootSphereColor = rootSphereColor;
	}

	/**
	 * @param segment2ArcCorrectionFactor The s3_Segment2ArcCorrectionFactor to set.
	 */
	public static void setS3_Segment2ArcCorrectionFactor(
			float segment2ArcCorrectionFactor) {
		s3_Segment2ArcCorrectionFactor = segment2ArcCorrectionFactor;
	}

	/**
	 * @param semanticRelationDistanceMultiplier The s3_SemanticRelationDistanceMultiplier to set.
	 */
	public static void setS3_SemanticRelationDistanceMultiplier(
			float semanticRelationDistanceMultiplier) {
		s3_SemanticRelationDistanceMultiplier = semanticRelationDistanceMultiplier;
	}

	/**
	 * @param sphereRadius The s3_SphereRadius to set.
	 */
	public static void setS3_SphereRadius(float sphereRadius) {
		s3_SphereRadius = sphereRadius;
	}

	/**
	 * @param treeLevelHeight The s3_TreeLevelHeight to set.
	 */
	public static void setS3_TreeLevelHeight(float treeLevelHeight) {
		s3_TreeLevelHeight = treeLevelHeight;
	}

	/**
	 * @param side The s4_cubeSide to set.
	 */
	public static void setS4_cubeSide(float side) {
		s4_cubeSide = side;
	}

	/**
	 * @param instanceSideLimit The s4_InstanceSideLimit to set.
	 */
	public static void setS4_InstanceSideLimit(float instanceSideLimit) {
		s4_InstanceSideLimit = instanceSideLimit;
	}

	/**
	 * @param instanceSideMultiplier The s4_InstanceSideMultiplier to set.
	 */
	public static void setS4_InstanceSideMultiplier(float instanceSideMultiplier) {
		s4_InstanceSideMultiplier = instanceSideMultiplier;
	}

	/**
	 * @param instanceWithFactsColor The s4_InstanceWithFactsColor to set.
	 */
	public static void setS4_InstanceWithFactsColor(Color3f instanceWithFactsColor) {
		s4_InstanceWithFactsColor = instanceWithFactsColor;
	}

	/**
	 * @param instanceWithoutFactsColor The s4_InstanceWithoutFactsColor to set.
	 */
	public static void setS4_InstanceWithoutFactsColor(
			Color3f instanceWithoutFactsColor) {
		s4_InstanceWithoutFactsColor = instanceWithoutFactsColor;
	}

	/**
	 * @param labelInstanceNoChild The s4_LabelInstanceNoChild to set.
	 */
	public static void setS4_LabelInstanceNoChild(Color3f labelInstanceNoChild) {
		s4_LabelInstanceNoChild = labelInstanceNoChild;
	}

	/**
	 * @param labelInstanceScale The s4_LabelInstanceScale to set.
	 */
	public static void setS4_LabelInstanceScale(float labelInstanceScale) {
		s4_LabelInstanceScale = labelInstanceScale;
	}

	/**
	 * @param labelInstanceWithChildren The s4_LabelInstanceWithChildren to set.
	 */
	public static void setS4_LabelInstanceWithChildren(
			Color3f labelInstanceWithChildren) {
		s4_LabelInstanceWithChildren = labelInstanceWithChildren;
	}

	/**
	 * @param mainSphereColor The s4_MainSphereColor to set.
	 */
	public static void setS4_MainSphereColor(Color3f mainSphereColor) {
		s4_MainSphereColor = mainSphereColor;
	}

	/**
	 * @param conceptHeight The s5_ConceptHeight to set.
	 */
	public static void setS5_ConceptHeight(float conceptHeight) {
		s5_ConceptHeight = conceptHeight;
	}

	/**
	 * @param conceptSphereColor The s5_ConceptSphereColor to set.
	 */
	public static void setS5_ConceptSphereColor(Color3f conceptSphereColor) {
		s5_ConceptSphereColor = conceptSphereColor;
	}

	/**
	 * @param conceptSphereRadius The s5_ConceptSphereRadius to set.
	 */
	public static void setS5_ConceptSphereRadius(float conceptSphereRadius) {
		s5_ConceptSphereRadius = conceptSphereRadius;
	}

	/**
	 * @param side The s5_cubeSide to set.
	 */
	public static void setS5_cubeSide(float side) {
		s5_cubeSide = side;
	}

	/**
	 * @param rel2ConceptTransparency The s5_Rel2ConceptTransparency to set.
	 */
	public static void setS5_Rel2ConceptTransparency(float rel2ConceptTransparency) {
		s5_Rel2ConceptTransparency = rel2ConceptTransparency;
	}

	/**
	 * @param curvedArrowCurvature The s6_CurvedArrowCurvature to set.
	 */
	public static void setS6_CurvedArrowCurvature(float curvedArrowCurvature) {
		s6_CurvedArrowCurvature = curvedArrowCurvature;
	}

	/**
	 * @param familyRadiusMultiplier The s6_FamilyRadiusMultiplier to set.
	 */
	public static void setS6_FamilyRadiusMultiplier(float familyRadiusMultiplier) {
		s6_FamilyRadiusMultiplier = familyRadiusMultiplier;
	}

	/**
	 * @param screenPercent_as_RootVerticalPosition The s6_ScreenPercent_as_RootVerticalPosition to set.
	 */
	public static void setS6_ScreenPercent_as_RootVerticalPosition(
			float screenPercent_as_RootVerticalPosition) {
		s6_ScreenPercent_as_RootVerticalPosition = screenPercent_as_RootVerticalPosition;
	}

	/**
	 * @param sg_background The sG_background to set.
	 */
	public static void setSG_background(Color3f sg_background) {
		sG_background = sg_background;
	}

	/**
	 * @param sg_distance_min The sG_distance_min to set.
	 */
	public static void setSG_distance_min(float sg_distance_min) {
		sG_distance_min = sg_distance_min;
	}

	/**
	 * @param fontFamily The sG_FontFamily to set.
	 */
	public static void setSG_FontFamily(String fontFamily) {
		sG_FontFamily = fontFamily;
	}

	/**
	 * @param fontModifier The sG_FontModifier to set.
	 */
	public static void setSG_FontModifier(int fontModifier) {
		sG_FontModifier = fontModifier;
	}

	/**
	 * @param shellRadiusMultiplier The sG_instanceShellRadiusMultiplier to set.
	 */
	public static void setSG_instanceShellRadiusMultiplier(
			float shellRadiusMultiplier) {
		sG_instanceShellRadiusMultiplier = shellRadiusMultiplier;
	}

	/**
	 * @param instanceShellTransparency The sG_InstanceShellTransparency to set.
	 */
	public static void setSG_InstanceShellTransparency(
			float instanceShellTransparency) {
		sG_InstanceShellTransparency = instanceShellTransparency;
	}

	/**
	 * @param height The sR_coneHeight to set.
	 */
	public static void setSR_coneHeight(float height) {
		sR_coneHeight = height;
	}

	/**
	 * @param radius The sR_coneRadius to set.
	 */
	public static void setSR_coneRadius(float radius) {
		sR_coneRadius = radius;
	}

	/**
	 * @param side The sR_cubeSide to set.
	 */
	public static void setSR_cubeSide(float side) {
		sR_cubeSide = side;
	}

	/**
	 * @param radius The sR_cylinderRadius to set.
	 */
	public static void setSR_cylinderRadius(float radius) {
		sR_cylinderRadius = radius;
	}

	/**
	 * @param typeCylinderHeight The sR_dataTypeCylinderHeight to set.
	 */
	public static void setSR_dataTypeCylinderHeight(float typeCylinderHeight) {
		sR_dataTypeCylinderHeight = typeCylinderHeight;
	}

	/**
	 * @param typeCylinderRadius The sR_dataTypeCylinderRadius to set.
	 */
	public static void setSR_dataTypeCylinderRadius(float typeCylinderRadius) {
		sR_dataTypeCylinderRadius = typeCylinderRadius;
	}

	/**
	 * @param displayRelationsLabels The sR_DisplayRelationsLabels to set.
	 */
	public static void setSR_DisplayRelationsLabels(boolean displayRelationsLabels) {
		sR_DisplayRelationsLabels = displayRelationsLabels;
	}

	/**
	 * @param labelPositionOffset The sR_LabelPositionOffset to set.
	 */
	public static void setSR_LabelPositionOffset(float labelPositionOffset) {
		sR_LabelPositionOffset = labelPositionOffset;
	}

	/**
	 * @param multiRelationArrowHeadColor The sR_MultiRelationArrowHeadColor to set.
	 */
	public static void setSR_MultiRelationArrowHeadColor(
			Color3f multiRelationArrowHeadColor) {
		sR_MultiRelationArrowHeadColor = multiRelationArrowHeadColor;
	}

	/**
	 * @param multiRelationHeightLimit The sR_MultiRelationHeightLimit to set.
	 */
	public static void setSR_MultiRelationHeightLimit(float multiRelationHeightLimit) {
		sR_MultiRelationHeightLimit = multiRelationHeightLimit;
	}

	/**
	 * @param multiRelationMultiplier The sR_MultiRelationMultiplier to set.
	 */
	public static void setSR_MultiRelationMultiplier(float multiRelationMultiplier) {
		sR_MultiRelationMultiplier = multiRelationMultiplier;
	}

	/**
	 * @param multiRelationRadiusLimit The sR_MultiRelationRadiusLimit to set.
	 */
	public static void setSR_MultiRelationRadiusLimit(float multiRelationRadiusLimit) {
		sR_MultiRelationRadiusLimit = multiRelationRadiusLimit;
	}

	/**
	 * @param holderLabelScale The sR_placeHolderLabelScale to set.
	 */
	public static void setSR_placeHolderLabelScale(float holderLabelScale) {
		sR_placeHolderLabelScale = holderLabelScale;
	}

	/**
	 * @param relationsLabeScale The sR_RelationsLabeScale to set.
	 */
	public static void setSR_RelationsLabeScale(float relationsLabeScale) {
		sR_RelationsLabeScale = relationsLabeScale;
	}

	/**
	 * @param loopRadiusMultiplier The sR_selfLoopRadiusMultiplier to set.
	 */
	public static void setSR_selfLoopRadiusMultiplier(float loopRadiusMultiplier) {
		sR_selfLoopRadiusMultiplier = loopRadiusMultiplier;
	}

	/**
	 * @param loopverticalMultiplier The sR_selfLoopverticalMultiplier to set.
	 */
	public static void setSR_selfLoopverticalMultiplier(float loopverticalMultiplier) {
		sR_selfLoopverticalMultiplier = loopverticalMultiplier;
	}

	/**
	 * @param radius The sR_sphereRadius to set.
	 */
	public static void setSR_sphereRadius(float radius) {
		sR_sphereRadius = radius;
	}

	public static void setSG_useAnimations(boolean animations) {
		sG_useAnimations = animations;
	}

	public static void setS3_SemRelationHeigth(float semRelationHeigth) {
		s3_SemRelationHeigth = semRelationHeigth;
	}

	public static void setS5_SemRelationHeigth(float semRelationHeigth) {
		s5_SemRelationHeigth = semRelationHeigth;
	}

	public static void setSR_DatatypeRelationColor(Color3f datatypeRelationColor) {
		sR_DatatypeRelationColor = datatypeRelationColor;
	}

	public static void setSR_IncomingRelationColor(Color3f incomingRelationColor) {
		sR_IncomingRelationColor = incomingRelationColor;
	}

	public static void setSR_OutgoingRelationColor(Color3f outgoingRelationColor) {
		sR_OutgoingRelationColor = outgoingRelationColor;
	}


}